@@ -19,12 +19,7 @@ public func DoThrow(object clonk, int angle)
 	var javelin=TakeObject();
 	if (king_size)
 		javelin->MakeKingSize();
-	
-	// how fast the javelin is thrown depends very much on
-	// the speed of the clonk
-	//var speed = 1200 * clonk->GetPhysical("Throw") / 12000 + 150 * Abs(clonk->GetXDir());
-	//var xdir = Sin(angle,+speed);
-	//var ydir = Cos(angle,-speed);
+
 	// The clonk can convert some, indicated by div in percentages, of its own kinetic energy to change the momenta of the javelin, a miracle.
 	var div = 60; // 40% is converted to the direction of the throwing angle.
 	var xdir = clonk->GetXDir(1000);
@@ -42,6 +37,6 @@ public func DoThrow(object clonk, int angle)
 	
 	Sound("ThrowJavelin?");
 	
-	fAiming = -1;
+	aiming = -1;
 	clonk->UpdateAttach();
 }
