@@ -16,11 +16,11 @@ func DoInit(int first_player)
 	elevator->SetOwner(first_player);
 	elevator.case->SetPosition(elevator.case->GetX(), elevator.case->GetY()+190);
 	// Create the start buildings: 2 x flag, windmill, armory, cabin
-	CreateObject(Flagpole, 193*8, 43*8, first_player);
-	CreateObject(Flagpole, 221*8, 46*8, first_player);
-	CreateObject(WoodenCabin, 228*8, 47*8, first_player);
-	CreateObject(Armory, 212*8, 47*8, first_player);
-	CreateObject(WindGenerator, 197*8, 44*8, first_player);
+	CreateObjectAbove(Flagpole, 193*8, 43*8, first_player);
+	CreateObjectAbove(Flagpole, 221*8, 46*8, first_player);
+	CreateObjectAbove(WoodenCabin, 228*8, 47*8, first_player);
+	CreateObjectAbove(Armory, 212*8, 47*8, first_player);
+	CreateObjectAbove(WindGenerator, 197*8, 44*8, first_player);
 	// Create start material: Hammer, shovel, axe
 	var clonk1 = GetCrew(first_player, 0);
 	var clonk2 = GetCrew(first_player, 1);
@@ -36,7 +36,7 @@ func DoInit(int first_player)
 		var location = FindLocation(Loc_Material("Earth"));//, Loc_Space(5, false), Loc_Space(5, true));
 		if(location)
 		{
-			CreateObject(stuff[Random(GetLength(stuff))], location.x, location.y);
+			CreateObjectAbove(stuff[Random(GetLength(stuff))], location.x, location.y);
 		}
 	}
 
@@ -51,3 +51,9 @@ func InitializePlayer(int plr)
 	if (!g_is_initialized) g_is_initialized = DoInit(plr);
 	return true;
 }
+
+func OnGoalsFulfilled()
+{
+	GainScenarioAchievement("Done");
+	return false;
+}
