@@ -24,8 +24,6 @@
 // ladder climbing
 #include Library_CanClimbLadder
 
-local pInventory;
-
 /* Initialization */
 
 protected func Construction()
@@ -147,29 +145,29 @@ protected func Death(int killed_by)
 		return;
 	
 	// Some effects on dying.
-	if(gender == 0)
-		Sound("Die");
-	else
-		Sound("FDie");
+	if (!this.silent_death)
+	{
+		if(gender == 0)
+			Sound("Die");
+		else
+			Sound("FDie");
+			
+		DeathAnnounce();
+	}
 	CloseEyes(1);
 	
-	DeathAnnounce();
-	return;
+	return true;
 }
 
-protected func Destruction()
+protected func Destruction(...)
 {
 	_inherited(...);
 	// If the clonk wasn't dead yet, he will be now.
-	if (GetAlive())
-		GameCallEx("OnClonkDeath", this, GetKiller());
-	// If this is the last crewmember, do broadcast.
-	if (GetCrew(GetOwner()) == this)
-	if (GetCrewCount(GetOwner()) == 1)
-		// Only if the player is still alive and not yet elimnated.
-			if (GetPlayerName(GetOwner()))
-				GameCallEx("RelaunchPlayer", GetOwner(), GetKiller());
-	return;
+	// Always kill clonks first. This will ensure relaunch scripts, enemy kill counters, etc. are called
+	// even if clonks die in some weird way that causes direct removal
+	// (To prevent a death callback, you can use SetAlive(false); RemoveObject();)
+	if (GetAlive()) { this.silent_death=true; Kill(); }
+	return true;
 }
 
 protected func DeepBreath()
@@ -204,14 +202,33 @@ func DigOutObject(object obj)
 	return false;
 }
 
+// Building material bridges (like loam bridge)
+func Bridge()
+{
+	var proc = GetProcedure();
+	// Clonk must stand on ground. Allow during SCALE; but Clonk won't keep animation if he's not actually near the ground
+	if (proc != "WALK" && proc != "SCALE")
+		return false;
+	if (proc == "WALK")
+		SetAction("BridgeStand");
+	else
+		SetAction("BridgeScale");
+	SetComDir(COMD_Stop);
+	SetXDir(0);
+	SetYDir(0);
+	return true;
+}
+
 /* Status */
 
 // TODO: Make this more sophisticated, readd turn animation and other
 // adaptions
 public func IsClonk() { return true; }
+public func IsPrey() { return true; }
 
 public func IsJumping(){return WildcardMatch(GetAction(), "*Jump*");}
 public func IsWalking(){return GetProcedure() == "WALK";}
+public func IsBridging(){return WildcardMatch(GetAction(), "Bridge*");}
 
 /* Carry items on the clonk */
 
@@ -311,8 +328,6 @@ func DoUpdateAttach(bool sec)
 			iHandMesh[sec] = AttachMesh(obj, pos_hand, bone, trans);
 			PlayAnimation(closehand, 6, Anim_Const(GetAnimationLength(closehand)), Anim_Const(1000));
 		}
-		else
-			; // Don't display
 	}
 	else if(iAttachMode == CARRY_HandBack)
 	{
@@ -342,8 +357,6 @@ func DoUpdateAttach(bool sec)
 			PlayAnimation("CarryArms", 6, Anim_Const(obj->~GetCarryPhase(this)), Anim_Const(1000));
 			fBothHanded = 1;
 		}
-		else
-			; // Don't display
 	}
 	else if(iAttachMode == CARRY_Spear)
 	{
@@ -416,7 +429,7 @@ func HasHandAction(sec, just_wear)
 func HasActionProcedure()
 {
 	var action = GetAction();
-	if (action == "Walk" || action == "Jump" || action == "WallJump" || action == "Kneel" || action == "Ride")
+	if (action == "Walk" || action == "Jump" || action == "WallJump" || action == "Kneel" || action == "Ride" || action == "BridgeStand")
 		return true;
 	return false;
 }
@@ -532,6 +545,12 @@ func QueryCatchBlow(object obj)
 	var r=0;
 	var e=0;
 	var i=0;
+	// Blocked by object effects?
+	while(e=GetEffect("*", obj, i++))
+		if(EffectCall(obj, e, "QueryHitClonk", this))
+			return true;
+	// Blocked by Clonk effects?
+	i=0;
 	while(e=GetEffect("*Control*", this, i++))
 	{
 		if(EffectCall(this, e, "QueryCatchBlow", obj))
@@ -542,31 +561,35 @@ func QueryCatchBlow(object obj)
 		
 	}
 	if(r) return r;
+	// No blocking
 	return _inherited(obj, ...);
 }
 
-local gender;
+local gender, skin, skin_name;
 
-func SetSkin(int skin)
+func SetSkin(int new_skin)
 {
+	// Remember skin
+	skin = new_skin;
+	
 	//Adventurer
 	if(skin == 0)
-	{	SetGraphics();
+	{	SetGraphics(skin_name = nil);
 		gender = 0;	}
 
 	//Steampunk
 	if(skin == 1)
-	{	SetGraphics("Steampunk");
+	{	SetGraphics(skin_name = "Steampunk");
 		gender = 1; }
 
 	//Alchemist
 	if(skin == 2)
-	{	SetGraphics("Alchemist");
+	{	SetGraphics(skin_name = "Alchemist");
 		gender = 0;	}
 	
 	//Farmer
 	if(skin == 3)
-	{	SetGraphics("Farmer");
+	{	SetGraphics(skin_name = "Farmer");
 		gender = 1;	}
 
 	RemoveBackpack(); //add a backpack
@@ -577,35 +600,76 @@ func SetSkin(int skin)
 }
 func GetSkinCount() { return 4; }
 
+func GetSkin() { return skin; }
+func GetSkinName() { return skin_name; }
+
+//Portrait definition of this Clonk for messages
+func GetPortrait()
+{
+	return this.portrait ?? { Source = GetID(), Name = Format("Portrait%s", skin_name ?? ""), Color = GetColor() };
+}
+
+func SetPortrait(proplist custom_portrait)
+{
+	this.portrait = custom_portrait;
+	return true;
+}
+
 /* Scenario saving */
 
 func SaveScenarioObject(props)
 {
 	if (!inherited(props, ...)) return false;
+	// Skins override mesh material
+	if (skin)
+	{
+		props->Remove("MeshMaterial");
+		props->AddCall("Skin", this, "SetSkin", skin);
+	}
 	// Direction is randomized at creation and there's no good way to find
 	// out if the user wanted that specific direction. So just always save
 	// it, because that's what scenario designer usually wants.
 	if (!props->HasProp("Dir")) props->AddCall("Dir", this, "SetDir", GetConstantNameByValueSafe(GetDir(),"DIR_"));
+	// Custom portraits for dialogues
+	if (this.portrait) props->AddCall("Portrait", this, "SetPortrait", this.portrait);
 	return true;
 }
 
 
 /* AI editor helper */
 
-func EditCursorSelection()
+func EditCursorSelection(...)
 {
 	var ai = S2AI->GetAI(this);
-	if (ai) Call(S2AI.EditCursorSelection, ai);
+	if (ai) Call(S2AI.EditCursorSelection, ai, ...);
 	return _inherited(...);
 }
 
-func EditCursorDeselection()
+func EditCursorDeselection(...)
 {
 	var ai = S2AI->GetAI(this);
-	if (ai) Call(S2AI.EditCursorDeselection, ai);
+	if (ai) Call(S2AI.EditCursorDeselection, ai, ...);
 	return _inherited(...);
 }
 
+func AI_Add()
+{
+	// Create AI and re-select
+	S2AI->AddAI(this);
+	EditCursorDeselection();
+	EditCursorSelection();
+	return true;
+}
+
+func FlipDir()
+{
+	// Look the other way. If an AI is attached, also update its home position.
+	var new_dir = 1-GetDir();
+	if (this.ai) this.ai.home_dir = new_dir;
+	return SetDir(new_dir);
+}
+
+local EditCursorCommands = ["AI_Add()", "FlipDir()"];
 
 /* Act Map */
 
@@ -732,9 +796,9 @@ Dig = {
 //	InLiquidAction = "Swim",
 	Attach = CNAT_Left | CNAT_Right | CNAT_Bottom,
 },
-Bridge = {
+BridgeStand = {
 	Prototype = Action,
-	Name = "Bridge",
+	Name = "BridgeStand",
 	Procedure = DFA_THROW,
 	Directions = 2,
 	Length = 16,
@@ -743,7 +807,22 @@ Bridge = {
 	Y = 60,
 	Wdt = 8,
 	Hgt = 20,
-	NextAction = "Bridge",
+	NextAction = "BridgeStand",
+	StartCall = "StartStand",
+	InLiquidAction = "Swim",
+},
+BridgeScale = {
+	Prototype = Action,
+	Name = "BridgeScale",
+	Procedure = DFA_THROW,
+	Directions = 2,
+	Length = 16,
+	Delay = 1,
+	X = 0,
+	Y = 60,
+	Wdt = 8,
+	Hgt = 20,
+	NextAction = "BridgeScale",
 	InLiquidAction = "Swim",
 },
 Swim = {
