@@ -52,7 +52,7 @@ protected func Construction()
 	
 	
 	// wealth display
-	wealth = CreateObject(GUI_Wealth,0,0,GetOwner());
+	wealth = CreateObjectAbove(GUI_Wealth,0,0,GetOwner());
 	wealth->SetPosition(-16-GUI_Wealth->GetDefHeight()/2,8+GUI_Wealth->GetDefHeight()/2);
 	wealth->Update();
 	
@@ -118,7 +118,7 @@ private func MakeInventory()
 	var y = 200;
 
 	// and the carry heavy slot
-	var bt = CreateObject(GUI_Backpack_Slot_Icon,0,0,GetOwner());
+	var bt = CreateObjectAbove(GUI_Backpack_Slot_Icon,0,0,GetOwner());
 	bt->SetHUDController(this);
 	bt->SetPosition(40+d, y);
 	bt->SetSlotId(-1);
@@ -151,7 +151,7 @@ public func OnGoalUpdate(object goal)
 	{
 		if (!HUDgoal)
 		{
-			HUDgoal = CreateObject(GUI_Goal, 0, 0, GetOwner());
+			HUDgoal = CreateObjectAbove(GUI_Goal, 0, 0, GetOwner());
 			HUDgoal->SetPosition(-64-16-GUI_Goal->GetDefHeight()/2,8+GUI_Goal->GetDefHeight()/2);
 		}
 		HUDgoal->SetGoal(goal);
@@ -177,7 +177,7 @@ global func AddHUDMarker(int player, picture, string altpicture, string text, in
 	var padding = GUI_Marker->GetDefHeight()+5;
 	var hud = FindObject(Find_ID(GUI_Controller),Find_Owner(player));
 	number = hud->GetFreeMarkerPosition();
-	hud.markers[number] = CreateObject(GUI_Marker,0,0,player);
+	hud.markers[number] = CreateObjectAbove(GUI_Marker,0,0,player);
 	hud.markers[number] -> SetPosition(5+(GUI_Marker->GetDefWidth()/2),-240-(GUI_Marker->GetDefHeight()/2) - number*padding);
 	hud.markers[number] -> SetVisual(picture, altpicture);
 	if(inform) hud.markers[number].toInform = inform;
@@ -479,11 +479,18 @@ public func FxIntSearchInteractionObjectsTimer(object target, effect, int time)
 				ActionButton(target, i++, interaction.Object, ACTIONTYPE_EXTRA, hotkey++, nil, interaction);
 	}
 
-	// add structures
+	// add structures (exit/enter)
 	var structures = FindObjects(Find_AtPoint(target->GetX()-GetX(),target->GetY()-GetY()),Find_OCF(OCF_Entrance),Find_NoContainer(), Find_Layer(target->GetObjectLayer()));
 	for(var structure in structures)
 	{
-		ActionButton(target,i++,structure,ACTIONTYPE_STRUCTURE,hotkey++);
+		// Only show the button when in front of the entrance, or if inside.
+		var x = structure->GetDefCoreVal("Entrance", "DefCore", 0);
+		var y = structure->GetDefCoreVal("Entrance", "DefCore", 1);
+		var wdt = structure->GetDefCoreVal("Entrance", "DefCore", 2);
+		var hgt = structure->GetDefCoreVal("Entrance", "DefCore", 3);
+		var at_entrance = Inside(target->GetX() - structure->GetX(), x, x + wdt) && Inside(target->GetY() - structure->GetY(), y, y + hgt);
+		if (Contained() || at_entrance)
+			ActionButton(target,i++,structure,ACTIONTYPE_STRUCTURE,hotkey++);
 	}
 	
 	// add extra-interactions after everything
@@ -549,7 +556,7 @@ private func InventoryButton()
 	var i = GetLength(inventory);
 	
 	// create inventory slots
-	var bt = CreateObject(GUI_Backpack_Slot_Icon,0,0,GetOwner());
+	var bt = CreateObjectAbove(GUI_Backpack_Slot_Icon,0,0,GetOwner());
 	bt->SetHUDController(this);
 	bt->SetPosition(40, y + d*i);
 	bt->SetSlotId(i);
@@ -595,7 +602,7 @@ private func ActionButton(object forClonk, int pos, object interaction, int acti
 	// no object yet... create it
 	if(!bt)
 	{
-		bt = CreateObject(GUI_ObjectSelector,0,0,GetOwner());
+		bt = CreateObjectAbove(GUI_ObjectSelector,0,0,GetOwner());
 	}
 
 	bt->SetPosition(401 + pos * spacing, -45);
@@ -644,7 +651,7 @@ private func ClearButtonMessages()
 */
 private func CreateSelectorFor(object clonk)
 {
-	var selector = CreateObject(GUI_CrewSelector,10,10,-1);
+	var selector = CreateObjectAbove(GUI_CrewSelector,10,10,-1);
 	selector->SetCrew(clonk);
 	clonk->SetSelector(selector);
 	return selector;
