@@ -18,7 +18,7 @@
 	{
 		// Plant!
 		clonk->DoKneel();
-		CreateObject(Wheat, x, y, clonk->GetOwner())->SetCon(1);
+		CreateObjectAbove(Wheat, x, y, clonk->GetOwner())->SetCon(1);
 		RemoveObject();
 	}
 	else
@@ -39,4 +39,4 @@ local Name = "$Name$";
 local Description = "$Description$";
 local UsageHelp = "$UsageHelp$";
 local Rebuy = true;
-local Plane = 460;
\ No newline at end of file
+local Plane = 460;
