@@ -4,6 +4,7 @@ local Collectible = 1;
 local Name = "$Name$";
 local Description = "$Description$";
 local Plane = 480;
+local graphics_index = 0;
 
 // returns the color of the gem (used for effects)
 func GetGemColor()
@@ -14,8 +15,8 @@ func GetGemColor()
 func Initialize()
 {
 	AddEffect("Sparkle", this, 1, 30 + RandomX(-3, 3), this);
-	var gfx = Random(4);
-	if (gfx) SetGraphics(Format("%d", gfx+1));
+	graphics_index = Random(4);
+	if (graphics_index) SetGraphics(Format("%d", graphics_index+1));
 	return true;
 }
 
