@@ -8,11 +8,16 @@ protected func Hit()
 
 func Incineration()
 {
-	SetClrModulation (RGB(48, 32, 32));
+	SetClrModulation(RGB(48, 32, 32));
 }
 
 public func IsFuel() { return true; }
-public func GetFuelAmount() { return 50; }
+public func GetFuelAmount(bool get_partial) 
+{ 
+	if (get_partial)
+		return GetCon() / 2;
+	return 50;
+}
 public func IsSawmillProduct() { return true; }
 
 local Collectible = 1;
