@@ -8,6 +8,7 @@ local Description = "$Description$";
 local UsageHelp = "$UsageHelp$";
 local Collectible = 1;
 local Rebuy = true;
+local ApertureOffsetY = 3; // pump from bottom vertex
 
 protected func Hit()
 {
@@ -42,7 +43,7 @@ protected func ControlUse(object clonk, int x, int y)
 	}
 	
 	// Create and connect pipe.
-	var pipe = CreateObject(PipeLine, 0, 0, NO_OWNER);
+	var pipe = CreateObjectAbove(PipeLine, 0, 0, NO_OWNER);
 	pipe->SetActionTargets(this, liquid_pump);
 	Sound("Connect");
 	
