@@ -472,6 +472,8 @@ protected func Ejection(object obj)
 		for(var c = 0; c < ContentsCount(); ++c)
 		{
 			var o = Contents(c);
+			if (!o)
+				continue;			
 			if(o->~IsCarryHeavy())
 				continue;
 			if (GetItemPos(o) == nil)
@@ -515,6 +517,9 @@ protected func RejectCollect(id objid, object obj)
 {
 	// collection of that object magically disabled?
 	if(GetEffect("NoCollection", obj)) return true;
+	
+	// NPCs only collect on demand (so they don't steal stuff form the player)
+	if (!force_collection && GetController() == NO_OWNER) return true;
 
 	// Carry heavy only gets picked up if none held already
 	if(obj->~IsCarryHeavy())
@@ -1134,7 +1139,7 @@ public func ObjectControl(int plr, int ctrl, int x, int y, int strength, bool re
 			if (ctrl == CON_ThrowAlt)
 			{
 				CancelUse();
-				if (proc == "SCALE" || proc == "HANGLE")
+				if (proc == "SCALE" || proc == "HANGLE" || proc == "SWIM")
 					return ObjectCommand("Drop", contents2);
 				else
 					return ObjectCommand("Throw", contents2, x, y);
@@ -1272,6 +1277,7 @@ private func GetUseCallString(string action) {
 private func StartUseControl(int ctrl, int x, int y, object obj)
 {
 	using = obj;
+	obj->SetController(GetController());
 	using_type = DetermineUsageType(obj);
 	alt = ctrl != CON_Use;
 	
@@ -1639,6 +1645,10 @@ private func ObjectControlPush(int plr, int ctrl)
 	{
 		// ungrab only if he pushes
 		if (proc != "PUSH") return false;
+		
+		// vehicles might have set their own view and it's not reset on release controls
+		// So reset cursor view immediately when ungrabbing
+		ResetCursorView(GetController());
 
 		ObjectCommand("UnGrab");
 		return true;
@@ -1712,7 +1722,7 @@ private func VirtualCursor()
 	}
 	if (!virtual_cursor)
 	{
-		virtual_cursor = CreateObject(GUI_Crosshair,0,0,GetOwner());
+		virtual_cursor = CreateObjectAbove(GUI_Crosshair,0,0,GetOwner());
 	}
 	
 	return virtual_cursor;
@@ -1958,4 +1968,4 @@ public func ControlJump()
 func FxIsWallKickStart(object target, int num, bool temp)
 {
 	return 1;
-}
\ No newline at end of file
+}
