@@ -341,7 +341,9 @@ private func FuelNeed(id product) { return product->~GetFuelNeed(); }
 private func LiquidNeed(id product) { return product->~GetLiquidNeed(); }
 private func MaterialNeed(id product) { return product->~GetMaterialNeed(); }
 
-private func PowerNeed() { return 200; }
+public func PowerNeed() { return 80; }
+
+public func GetConsumerPriority() { return 50; }
 
 private func Produce(id product)
 {
@@ -409,7 +411,7 @@ public func CheckFuel(id product, bool remove)
 		var fuel_amount = 0;
 		// Find fuel in this producer.
 		for (var fuel in FindObjects(Find_Container(this), Find_Func("IsFuel")))
-			fuel_amount += fuel->~GetFuelAmount();
+			fuel_amount += fuel->~GetFuelAmount(false);
 		if (fuel_amount < FuelNeed(product))
 			return false;
 		else if (remove)
@@ -418,7 +420,7 @@ public func CheckFuel(id product, bool remove)
 			fuel_amount = 0;
 			for (var fuel in FindObjects(Find_Container(this), Find_Func("IsFuel")))
 			{
-				fuel_amount += fuel->~GetFuelAmount();
+				fuel_amount += fuel->~GetFuelAmount(false);
 				fuel->RemoveObject();
 				if (fuel_amount >= FuelNeed(product))
 					break;
@@ -440,6 +442,10 @@ public func CheckLiquids(id product, bool remove)
 		for (var liq_container in FindObjects(Find_Container(this), Find_Func("IsLiquidContainer")))
 			if (liq_container->~GetBarrelMaterial() == liquid)
 				liquid_amount += liq_container->~GetFillLevel();
+		// Find objects that "are" liquid (e.g. ice)
+		for (var liq_object in FindObjects(Find_Container(this), Find_Func("IsLiquid")))
+			if (liq_object->~IsLiquid() == liquid)
+				liquid_amount += liq_object->~GetLiquidAmount();
 		if (liquid_amount < need)
 			return false;
 		else if (remove)
@@ -451,8 +457,16 @@ public func CheckLiquids(id product, bool remove)
 				var val = liq_container->~GetLiquid(liquid, need - extracted);
 				extracted += val[1];
 				if (extracted >= need)
-					break;			
-			}			
+					return true;
+			}
+			for (var liq_object in FindObjects(Find_Container(this), Find_Func("IsLiquid")))
+			{
+				if (liq_object->~IsLiquid() != liquid) continue;
+				extracted += liq_object->~GetLiquidAmount();
+				liq_object->RemoveObject();
+				if (extracted >= need)
+					break;
+			}
 		}		
 	}
 	return true;
@@ -517,9 +531,12 @@ protected func FxProcessProductionStart(object target, proplist effect, int temp
 	// Callback to the producer.
 	this->~OnProductionStart(effect.Product);
 	
-	// consume power
-	if(PowerNeed() > 0)
-		MakePowerConsumer(PowerNeed());
+	// Consume power by registering as a consumer for the needed amount.
+	// But first hold the production until the power system gives it ok.
+	// Always register the power request even if power need is zero. The
+	// power network handles this correctly and a producer may decide to
+	// change its power need during production.
+	RegisterPowerRequest(this->PowerNeed());
 	
 	return 1;
 }
@@ -527,7 +544,7 @@ protected func FxProcessProductionStart(object target, proplist effect, int temp
 public func OnNotEnoughPower()
 {
 	var effect = GetEffect("ProcessProduction", this);
-	if(effect)
+	if (effect)
 	{
 		effect.Active = false;
 		this->~OnProductionHold(effect.Product, effect.Duration);
@@ -540,7 +557,7 @@ public func OnNotEnoughPower()
 public func OnEnoughPower()
 {
 	var effect = GetEffect("ProcessProduction", this);
-	if(effect)
+	if (effect)
 	{
 		effect.Active = true;
 		this->~OnProductionContinued(effect.Product, effect.Duration);
@@ -572,7 +589,7 @@ protected func FxProcessProductionStop(object target, proplist effect, int reaso
 	if(temp) return;
 	
 	// no need to consume power anymore
-	UnmakePowerConsumer();
+	UnregisterPowerRequest();
 		
 	if (reason != 0)
 		return 1;
@@ -581,7 +598,7 @@ protected func FxProcessProductionStop(object target, proplist effect, int reaso
 	//Log("Production finished on %i after %d frames", effect.Product, effect.Duration);
 	this->~OnProductionFinish(effect.Product);
 	// Create product. 	
-	var product = CreateObject(effect.Product);
+	var product = CreateObjectAbove(effect.Product);
 	OnProductEjection(product);
 	
 	return 1;
@@ -590,8 +607,11 @@ protected func FxProcessProductionStop(object target, proplist effect, int reaso
 // Standard behaviour for product ejection.
 public func OnProductEjection(object product)
 {
-	// Vehicles in front fo buildings.
-	if (product->GetCategory() & C4D_Vehicle)
+	// Safety for the product removing itself on construction.
+	if (!product)
+		return;	
+	// Vehicles in front of buildings, and objects with special needs as well.
+	if (product->GetCategory() & C4D_Vehicle || product->~OnCompletionEjectProduct())
 	{
 		var x = GetX();
 		var y = GetY() + GetDefHeight()/2 - product->GetDefHeight()/2;
@@ -682,6 +702,14 @@ protected func RejectEntrance(object obj)
 			if (FuelNeed(product) > 0)
 				return false;
 	}
+	// Liquid objects may be collected if a product needs them.
+	if (obj->~IsLiquid())
+	{
+		for (var product in GetProducts())
+			if (LiquidNeed(product))
+				if (LiquidNeed(product)[0] == obj->~IsLiquid())
+					return false;
+	}
 	// Liquid containers may be collected if a product needs them.
 	if (obj->~IsLiquidContainer())
 	{
