@@ -20,7 +20,7 @@ public func IsProduct(id product_id)
 }
 
 private func ProductionTime(id toProduce) { return 100; }
-private func PowerNeed() { return 100; }
+public func PowerNeed() { return 80; }
 
 public func OnProductionStart(id product)
 {
@@ -71,5 +71,6 @@ local ActMap = {
 
 local Name = "$Name$";
 local Description ="$Description$";
+local ContainBlast = true;
 local BlastIncinerate = 100;
 local HitPoints = 70;
\ No newline at end of file
