@@ -1,6 +1,6 @@
 /*-- Spin Wheel --*/
 
-local targetdoor;
+local targetdoor, temp_light;
 
 public func Initialize()
 {
@@ -10,13 +10,20 @@ public func Initialize()
 public func SetStoneDoor(object door)
 {
 	targetdoor = door;
-	return;
+	return true;
 }
 
 public func ControlUp(object clonk)
 {
 	if (GetAction() == "Still" && targetdoor)
 	{
+		if (clonk)
+		{
+			SetPlrView(clonk->GetController(), targetdoor);
+			if (temp_light) temp_light->RemoveObject();
+			var y_off = targetdoor->~GetFloorOffset();
+			temp_light = Global->CreateLight(targetdoor->GetX(), targetdoor->GetY() + y_off, 30, Fx_Light.LGT_Temp, clonk->GetController(), 30, 50);
+		}
 		targetdoor->OpenDoor();
 		SetAction("SpinLeft");
 		Sound("Chain");
@@ -27,6 +34,13 @@ public func ControlDown(object clonk)
 {
 	if (GetAction() == "Still" && targetdoor)
 	{
+		if (clonk)
+		{
+			SetPlrView(clonk->GetController(), targetdoor);
+			if (temp_light) temp_light->RemoveObject();
+			var y_off = targetdoor->~GetFloorOffset();
+			temp_light = Global->CreateLight(targetdoor->GetX(), targetdoor->GetY() + y_off, 30, Fx_Light.LGT_Temp, clonk->GetController(), 30, 50);
+		}
 		targetdoor->CloseDoor();
 		SetAction("SpinRight");
 		Sound("Chain");
@@ -40,6 +54,14 @@ public func SaveScenarioObject(props)
 	return true;
 }
 
+func ConnectNearestDoor()
+{
+	// EditCursor helper command: Connect to nearest door. Return connected door.
+	var door = FindObject(Find_ID(StoneDoor), Sort_Distance());
+	if (door) SetStoneDoor(door);
+	return door;
+}
+
 local ActMap = {
 	Still = {
 		Prototype = Action,
@@ -71,9 +93,12 @@ local ActMap = {
 };
 local Name = "$Name$";
 local Touchable = 2;
+local EditCursorCommands = ["ControlUp()", "ControlDown()", "ConnectNearestDoor()"];
 
 func Definition(def)
 {
 	SetProperty("PictureTransformation", Trans_Mul(Trans_Scale(800), Trans_Translate(0,0,0),Trans_Rotate(-20,1,0,0),Trans_Rotate(-30,0,1,0)), def);
 	SetProperty("MeshTransformation", Trans_Rotate(-13,0,1,0), def);
 }
+
+
