@@ -2,9 +2,29 @@
 
 #include Library_Plant
 
+
+func Place(int amount, proplist rectangle, proplist settings)
+{
+	var max_tries = 2 * amount;
+	var loc_area = nil;
+	if (rectangle) loc_area = Loc_InRect(rectangle);
+	
+	while ((amount > 0) && (--max_tries > 0))
+	{
+		var spot = FindLocation(Loc_Material("Water"), Loc_Wall(CNAT_Bottom), loc_area);
+		if (!spot) continue;
+		
+		var f = CreateObjectAbove(this, spot.x, spot.y, NO_OWNER);
+		--amount;
+	}
+	return true;
+}
+
 private func Initialize()
 {
 	SetAction("Sway");
+	SetPhase(this.Action.Length); // ensure that not all seaweed are synced on scenario load
+	return true;
 }
 
 private func Check()
@@ -15,7 +35,6 @@ private func Check()
 func SaveScenarioObject(props)
 {
 	if (!inherited(props, ...)) return false;
-	if (GetPhase()) props->AddCall("Phase", this, "SetPhase", GetPhase()); // ensure that not all seaweed are synced on scenario load
 	return true;
 }
 
