@@ -6,10 +6,10 @@
 
 // disable ShiftContents for objects with ClonkControl.ocd
 
-global func ShiftContents()
+global func ShiftContents(...)
 {
 	if (this)
 		if (this->~HandObjects() != nil)
 			return false;
 	return _inherited(...);
-}
\ No newline at end of file
+}
