@@ -40,6 +40,8 @@
 #include <locale.h>
 #endif
 
+#include <C4Application.h>
+
 void C4ConfigGeneral::CompileFunc(StdCompiler *pComp)
 {
 	// For those without the ability to intuitively guess what the falses and trues mean:
@@ -56,6 +58,9 @@ void C4ConfigGeneral::CompileFunc(StdCompiler *pComp)
 	// assimilate old data
 	pComp->Value(mkNamingAdapt(s(Adopt.PlayerPath), "PlayerPath",       ""));
 
+	// temporary path only set during updates
+	pComp->Value(mkNamingAdapt(s(TempUpdatePath),   "TempUpdatePath",     ""));
+
 	pComp->Value(mkNamingAdapt(s(MissionAccess),    "MissionAccess",      "", false, true));
 	pComp->Value(mkNamingAdapt(FPS,                 "FPS",                0              ));
 	pComp->Value(mkNamingAdapt(DefRec,              "DefRec",             0              ));
@@ -75,8 +80,10 @@ void C4ConfigGeneral::CompileFunc(StdCompiler *pComp)
 
 void C4ConfigDeveloper::CompileFunc(StdCompiler *pComp)
 {
-	pComp->Value(mkNamingAdapt(AutoFileReload,      "AutoFileReload",     1              ,false, true));
-	pComp->Value(mkNamingAdapt(ExtraWarnings,      "ExtraWarnings",     0              ,false, true));
+	pComp->Value(mkNamingAdapt(AutoFileReload,      "AutoFileReload",     1                   , false, true));
+	pComp->Value(mkNamingAdapt(ExtraWarnings,      "ExtraWarnings",       0                   , false, true));
+	pComp->Value(mkNamingAdapt(s(TodoFilename),    "TodoFilename",       "{SCENARIO}/TODO.txt", false, true));
+	pComp->Value(mkNamingAdapt(s(AltTodoFilename), "AltTodoFilename",    "TODO.txt"           , false, true));
 }
 
 void C4ConfigGraphics::CompileFunc(StdCompiler *pComp)
@@ -86,19 +93,17 @@ void C4ConfigGraphics::CompileFunc(StdCompiler *pComp)
 	pComp->Value(mkNamingAdapt(WindowX,               "WindowX",              800           ,false, true));
 	pComp->Value(mkNamingAdapt(WindowY,               "WindowY",              600           ,false, true));
 	pComp->Value(mkNamingAdapt(RefreshRate,           "RefreshRate",          0             ));
-	pComp->Value(mkNamingAdapt(ShowAllResolutions,    "ShowAllResolutions",   0             ,false, true));
 	pComp->Value(mkNamingAdapt(SplitscreenDividers,   "SplitscreenDividers",  1             ));
 	pComp->Value(mkNamingAdapt(ShowStartupMessages,   "ShowStartupMessages",  1             ,false, true));
 	pComp->Value(mkNamingAdapt(ColorAnimation,        "ColorAnimation",       0             ,false, true));
 	pComp->Value(mkNamingAdapt(HighResLandscape,      "HighResLandscape",     1             ,false, true));
-	pComp->Value(mkNamingAdapt(SmokeLevel,            "SmokeLevel",           200           ,false, true));
 	pComp->Value(mkNamingAdapt(VerboseObjectLoading,  "VerboseObjectLoading", 0             ));
 	pComp->Value(mkNamingAdapt(VideoModule,           "VideoModule",          0             ,false, true));
 	pComp->Value(mkNamingAdapt(MenuTransparency,      "MenuTransparency",     1             ,false, true));
 	pComp->Value(mkNamingAdapt(UpperBoard,            "UpperBoard",           1             ,false, true));
 	pComp->Value(mkNamingAdapt(ShowClock,             "ShowClock",            0             ,false, true));
 	pComp->Value(mkNamingAdapt(ShowCrewNames,         "ShowCrewNames",        1             ,false, true));
-	pComp->Value(mkNamingAdapt(ShowCrewCNames,        "ShowCrewCNames",       1             ,false, true));
+	pComp->Value(mkNamingAdapt(ShowCrewCNames,        "ShowCrewCNames",       0             ,false, true));
 	pComp->Value(mkNamingAdapt(BitDepth,              "BitDepth",             32            ,false, true));
 	pComp->Value(mkNamingAdapt(Windowed,              "Windowed",             0             ,false, true));
 	pComp->Value(mkNamingAdapt(PXSGfx,                "PXSGfx"  ,             1             ));
@@ -111,10 +116,9 @@ void C4ConfigGraphics::CompileFunc(StdCompiler *pComp)
 	pComp->Value(mkNamingAdapt(Monitor,               "Monitor",              0             )); // 0 = D3DADAPTER_DEFAULT
 	pComp->Value(mkNamingAdapt(FireParticles,         "FireParticles",        1         ));
 	pComp->Value(mkNamingAdapt(MaxRefreshDelay,       "MaxRefreshDelay",      30            ));
-	pComp->Value(mkNamingAdapt(EnableShaders,         "Shader",               0             ,false, true));
 	pComp->Value(mkNamingAdapt(NoOffscreenBlits,      "NoOffscreenBlits",     1             ));
-	pComp->Value(mkNamingAdapt(ClipManuallyE,         "ClipManuallyE",        1             ));
 	pComp->Value(mkNamingAdapt(MultiSampling,         "MultiSampling",        4             ));
+	pComp->Value(mkNamingAdapt(AutoFrameSkip,         "AutoFrameSkip",        1          ));
 }
 
 void C4ConfigSound::CompileFunc(StdCompiler *pComp)
@@ -153,17 +157,22 @@ void C4ConfigNetwork::CompileFunc(StdCompiler *pComp)
 	pComp->Value(mkNamingAdapt(MasterReferencePeriod,   "MasterReferencePeriod",120           ));
 	pComp->Value(mkNamingAdapt(LeagueServerSignUp,      "LeagueServerSignUp",   0             ));
 	pComp->Value(mkNamingAdapt(UseAlternateServer,      "UseAlternateServer",   0             ));
-	pComp->Value(mkNamingAdapt(s(AlternateServerAddress),"AlternateServerAddress", "boom.openclonk.org:80/server/"));
+	pComp->Value(mkNamingAdapt(s(AlternateServerAddress),"AlternateServerAddress", "league.openclonk.org:80/league.php"));
 	pComp->Value(mkNamingAdapt(s(LastPassword),         "LastPassword",         "Wipf"        ));
 #ifdef WITH_AUTOMATIC_UPDATE
-	pComp->Value(mkNamingAdapt(s(UpdateServerAddress),  "UpdateServerAddress",     "boom.openclonk.org:80/server/"));
+	pComp->Value(mkNamingAdapt(s(UpdateServerAddress),  "UpdateServerAddress",     "www.openclonk.org:80/update/"));
 	pComp->Value(mkNamingAdapt(AutomaticUpdate,         "AutomaticUpdate",      0             ,false, true));
 	pComp->Value(mkNamingAdapt(LastUpdateTime,          "LastUpdateTime",       0             ));
 #endif
 	pComp->Value(mkNamingAdapt(AsyncMaxWait,            "AsyncMaxWait",         2             ));
+	pComp->Value(mkNamingAdapt(PacketLogging,           "PacketLogging",        0             ));
+	
 
 	pComp->Value(mkNamingAdapt(s(PuncherAddress),       "PuncherAddress",       "clonk.de:11115")); // maybe store default for this one?
-
+	pComp->Value(mkNamingAdapt(mkParAdapt(LastLeagueServer, StdCompiler::RCT_All),     "LastLeagueServer",     ""            ));
+	pComp->Value(mkNamingAdapt(mkParAdapt(LastLeaguePlayerName, StdCompiler::RCT_All), "LastLeaguePlayerName", ""            ));
+	pComp->Value(mkNamingAdapt(mkParAdapt(LastLeagueAccount, StdCompiler::RCT_All),    "LastLeagueAccount",    ""            ));
+	pComp->Value(mkNamingAdapt(mkParAdapt(LastLeagueLoginToken, StdCompiler::RCT_All), "LastLeagueLoginToken", ""            ));
 }
 
 void C4ConfigLobby::CompileFunc(StdCompiler *pComp)
@@ -174,7 +183,7 @@ void C4ConfigLobby::CompileFunc(StdCompiler *pComp)
 
 void C4ConfigIRC::CompileFunc(StdCompiler *pComp)
 {
-	pComp->Value(mkNamingAdapt(s(Server),               "Server",               "irc.ham.de.euirc.net", false, true));
+	pComp->Value(mkNamingAdapt(s(Server),               "Server",               "irc.euirc.net", false, true));
 	pComp->Value(mkNamingAdapt(s(Nick),                 "Nick",                 ""                    , false, true));
 	pComp->Value(mkNamingAdapt(s(RealName),             "RealName",             ""                    , false, true));
 	pComp->Value(mkNamingAdapt(s(Channel),              "Channel",              "#openclonk"    , false, true));
@@ -235,7 +244,7 @@ void C4ConfigControls::CompileFunc(StdCompiler *pComp)
 {
 #ifndef USE_CONSOLE
 	pComp->Value(mkNamingAdapt(UserSets, "UserSets",    C4PlayerControlAssignmentSets()));
-	pComp->Value(mkNamingAdapt(MouseAScroll,      "MouseAutoScroll",      0));
+	pComp->Value(mkNamingAdapt(MouseAutoScroll,      "MouseAutoScroll",      0 /* change default 33 to enable */ ));
 	pComp->Value(mkNamingAdapt(GamepadGuiControl, "GamepadGuiControl",    0,     false, true));
 #endif
 }
@@ -454,20 +463,15 @@ void C4ConfigGeneral::DeterminePaths()
 	SCopy(ExePath.getMData(),SystemDataPath);
 #elif defined(__APPLE__)
 	SCopy(::Application.GetGameDataPath().getData(),SystemDataPath);
-#elif defined(__linux__)
-
-#ifdef OC_SYSTEM_DATA_DIR
-#ifdef WITH_AUTOMATIC_UPDATE
+#elif defined(WITH_AUTOMATIC_UPDATE)
 	// WITH_AUTOMATIC_UPDATE builds are our tarball releases and
 	// development snapshots, i.e. where the game data is at the
 	// same location as the executable.
 	SCopy(ExePath.getMData(),SystemDataPath);
-#else
+#elif defined(OC_SYSTEM_DATA_DIR)
 	SCopy(OC_SYSTEM_DATA_DIR, SystemDataPath);
-#endif
 #else
 #error Please define OC_SYSTEM_DATA_DIR!
-#endif
 #endif
 	AppendBackslash(SystemDataPath);
 
@@ -477,10 +481,10 @@ void C4ConfigGeneral::DeterminePaths()
 	else
 #if defined(_WIN32)
 		SCopy("%APPDATA%\\" C4ENGINENAME, UserDataPath);
-#elif defined(__linux__)
-		SCopy("$HOME/.clonk/" C4ENGINENICK, UserDataPath);
 #elif defined(__APPLE__)
 		SCopy("$HOME/Library/Application Support/" C4ENGINENAME, UserDataPath);
+#else
+		SCopy("$HOME/.clonk/" C4ENGINENICK, UserDataPath);
 #endif
 	C4Config::ExpandEnvironmentVariables(UserDataPath, CFG_MaxString);
 	AppendBackslash(UserDataPath);
@@ -576,7 +580,7 @@ const char* C4ConfigNetwork::GetLeagueServerAddress()
 		return AlternateServerAddress;
 	// Standard (hardcoded) official league server
 	else
-		return "boom.openclonk.org:80/server/";
+		return "league.openclonk.org:80/league.php";
 }
 
 void C4ConfigNetwork::CheckPortsForCollisions()
@@ -595,6 +599,29 @@ void C4ConfigNetwork::CheckPortsForCollisions()
 		PortDiscovery = C4NetStdPortDiscovery;
 	}
 }
+
+void C4ConfigNetwork::SetLeagueLoginData(const char *szServer, const char *szPlayerName, const char *szAccount, const char *szLoginToken)
+{
+	// ideally, there would be a list to store multiple logins
+	// however, we don't really support multiplayer at one computer at the moment anyway
+	LastLeagueServer.Copy(szServer);
+	LastLeaguePlayerName.Copy(szPlayerName);
+	LastLeagueAccount.Copy(szAccount);
+	LastLeagueLoginToken.Copy(szLoginToken);
+}
+
+bool C4ConfigNetwork::GetLeagueLoginData(const char *szServer, const char *szPlayerName, StdStrBuf *pAccount, StdStrBuf *pLoginToken) const
+{
+	// check if last login matches and store if desired
+	if (LastLeagueServer == szServer && LastLeaguePlayerName == szPlayerName)
+	{
+		pAccount->Copy(LastLeagueAccount);
+		pLoginToken->Copy(LastLeagueLoginToken);
+		return true;
+	}
+	return false;
+}
+
 void C4ConfigControls::ResetKeys()
 {
 	UserSets.Clear();
@@ -760,4 +787,33 @@ void C4Config::ExpandEnvironmentVariables(char *strPath, size_t iMaxLen)
 #endif
 }
 
+void C4Config::CleanupTempUpdateFolder()
+{
+	// Get rid of update path present from before update
+	if (*General.TempUpdatePath)
+	{
+		EraseItem(General.TempUpdatePath);
+		*General.TempUpdatePath = '\0';
+	}
+}
+
+const char *C4Config::MakeTempUpdateFolder()
+{
+	// just pick a temp name
+	StdStrBuf sTempName;
+	sTempName.Copy(AtTempPath("update"));
+	MakeTempFilename(&sTempName);
+	SCopy(sTempName.getData(), General.TempUpdatePath);
+	CreatePath(General.TempUpdatePath);
+	return General.TempUpdatePath;
+}
+
+const char *C4Config::AtTempUpdatePath(const char *szFilename)
+{
+	SCopy(General.TempUpdatePath,AtPathFilename,_MAX_PATH-1);
+	AppendBackslash(AtPathFilename);
+	SAppend(szFilename,AtPathFilename,_MAX_PATH);
+	return AtPathFilename;
+}
+
 C4Config Config;
