@@ -27,6 +27,7 @@
 #include <C4FullScreen.h>
 #include <C4PlayerList.h>
 #include <C4GameControl.h>
+#include <C4Components.h>
 
 // *** C4PlayerInfo
 
@@ -48,6 +49,8 @@ void C4PlayerInfo::Clear()
 	iLeagueProjectedGain = -1;
 	eType = C4PT_User;
 	idExtraData = C4ID::None;
+	iLeaguePerformance = 0;
+	sLeagueProgressData.Clear();
 }
 
 void C4PlayerInfo::DeleteTempFile()
@@ -248,6 +251,8 @@ void C4PlayerInfo::CompileFunc(StdCompiler *pComp)
 	pComp->Value(mkNamingAdapt(mkIntPackAdapt(iLeagueRankSymbol), "LeagueRankSymbol", 0));
 	pComp->Value(mkNamingAdapt(mkIntPackAdapt(iLeagueProjectedGain), "ProjectedGain", -1));
 	pComp->Value(mkNamingAdapt(mkParAdapt(sClanTag, StdCompiler::RCT_All), "ClanTag", ""));
+	pComp->Value(mkNamingAdapt(mkIntPackAdapt(iLeaguePerformance), "LeaguePerformance", 0));
+	pComp->Value(mkNamingAdapt(sLeagueProgressData, "LeagueProgressData", ""));
 
 	// file resource
 	if (pComp->isCompiler() && Game.C4S.Head.Replay)
