@@ -84,7 +84,7 @@ namespace
 	{
 		GdkPixbuf* pixbuf = gdk_pixbuf_new_from_inline(-1, pixbuf_data, false, NULL);
 		GtkWidget* image = gtk_image_new_from_pixbuf(pixbuf);
-		gdk_pixbuf_unref(pixbuf);
+		g_object_unref(pixbuf);
 		return image;
 	}
 }
@@ -1301,12 +1301,12 @@ void C4ToolsDlg::State::UpdatePreview()
 			*data = (dw >> 16) & 0xff; ++ data;
 			*data = (dw >> 8 ) & 0xff; ++ data;
 			*data = (dw      ) & 0xff; ++ data;
-			*data = 0xff - ((dw >> 24) & 0xff); ++ data;
+			*data = (dw >> 24) & 0xff; ++ data;
 		}
 
 	sfcPreview->Unlock();
 	gtk_image_set_from_pixbuf(GTK_IMAGE(preview), pixbuf);
-	gdk_pixbuf_unref(pixbuf);
+	g_object_unref(pixbuf);
 	delete sfcPreview;
 }
 
