@@ -150,10 +150,10 @@ bool C4EditCursor::Move(float iX, float iY, DWORD dwKeyState)
 		// Shift always indicates a target outside the current selection
 		else
 		{
-			Target = ((dwKeyState & MK_SHIFT) && Selection.Last) ? Selection.Last->Obj : NULL;
+			Target = (dwKeyState & MK_SHIFT) ? Selection.GetLastObject() : NULL;
 			do
 			{
-				Target = Game.FindObject(C4ID::None,X,Y,0,0,OCF_NotContained, Target);
+				Target = Game.FindObject(NULL,X,Y,0,0,OCF_NotContained, Target);
 			}
 			while ((dwKeyState & MK_SHIFT) && Target && Selection.GetLink(Target));
 		}
@@ -223,16 +223,26 @@ bool C4EditCursor::RemoveFromSelection(C4Object *remove_obj)
 	return true;
 }
 
-void C4EditCursor::ClearSelection()
+void C4EditCursor::ClearSelection(C4Object *next_selection)
 {
 	// remove all objects from selection and do script callbacks
 	// iterate safely because callback might delete selected objects!
 	C4Object *obj;
-	while (obj = Selection.GetObject(0))
+	while ((obj = Selection.GetObject(0)))
 	{
 		Selection.Remove(obj);
 		if (obj->Status)
-			::Control.DoInput(CID_EMMoveObj, new C4ControlEMMoveObject(EMMO_Deselect, Fix0, Fix0, obj), CDT_Decide);
+		{
+			int32_t next_selection_count = 0, *next_selection_nums = NULL;
+			if (next_selection && next_selection->Status)
+			{
+				// Pass next selection. Always create new array becase the pointer is freed by C4ControlEMMoveObject dtor
+				++next_selection_count;
+				next_selection_nums = new int32_t[1];
+				*next_selection_nums = next_selection->Number;
+			}
+			::Control.DoInput(CID_EMMoveObj, new C4ControlEMMoveObject(EMMO_Deselect, Fix0, Fix0, obj, next_selection_count, next_selection_nums), CDT_Decide);
+		}
 	}
 	Selection.Clear();
 }
@@ -259,13 +269,20 @@ bool C4EditCursor::LeftButtonDown(DWORD dwKeyState)
 			// Click on unselected: select single
 			if (Target)
 			{
-				C4ObjectLink * it;
-				for(it = Selection.First; it; it = it->Next){
-					if(it->Obj->At(X, Y))
+				bool found = false;
+				for (C4Object *obj : Selection)
+				{
+					if(obj->At(X, Y))
+					{
+						found = true;
 						break;
+					}
+				}
+				if(!found) // means loop didn't break
+				{
+					ClearSelection(Target);
+					AddToSelection(Target);
 				}
-				if(!it) // means loop didn't break
-					{ ClearSelection(); AddToSelection(Target); }
 			}
 			// Click on nothing: drag frame
 			if (!Target)
@@ -306,18 +323,20 @@ bool C4EditCursor::RightButtonDown(DWORD dwKeyState)
 		{
 			// Check whether cursor is on anything in the selection
 			bool fCursorIsOnSelection = false;
-			for (C4ObjectLink *pLnk = Selection.First; pLnk; pLnk = pLnk->Next)
-				if (pLnk->Obj->At(X,Y))
+			for (C4Object *obj : Selection)
+			{
+				if (obj->At(X,Y))
 				{
 					fCursorIsOnSelection = true;
 					break;
 				}
+			}
 			if (!fCursorIsOnSelection)
 			{
 				// Click on unselected
 				if (Target && !Selection.GetLink(Target))
 				{
-					ClearSelection(); AddToSelection(Target);
+					ClearSelection(Target); AddToSelection(Target);
 				}
 				// Click on nothing
 				if (!Target) ClearSelection();
@@ -478,8 +497,7 @@ void C4EditCursor::Draw(C4TargetFacet &cgo)
 {
 	ZoomDataStackItem zdsi(cgo.X, cgo.Y, cgo.Zoom);
 	// Draw selection marks
-	C4Object *cobj; C4ObjectLink *clnk;
-	for (clnk=Selection.First; clnk && (cobj=clnk->Obj); clnk=clnk->Next)
+	for (C4Object *cobj : Selection)
 	{
 		// target pos (parallax)
 		float offX, offY, newzoom;
@@ -539,21 +557,33 @@ void C4EditCursor::DrawSelectMark(C4Facet &cgo, FLOAT_RECT frame)
 
 	if (!cgo.Surface) return;
 
-	pDraw->DrawPix(cgo.Surface,frame.left,frame.top,0xFFFFFFFF);
-	pDraw->DrawPix(cgo.Surface,frame.left+1,frame.top,0xFFFFFFFF);
-	pDraw->DrawPix(cgo.Surface,frame.left,frame.top+1,0xFFFFFFFF);
+	const float EDGE_WIDTH = 2.f;
 
-	pDraw->DrawPix(cgo.Surface,frame.left,frame.bottom-1,0xFFFFFFFF);
-	pDraw->DrawPix(cgo.Surface,frame.left+1,frame.bottom-1,0xFFFFFFFF);
-	pDraw->DrawPix(cgo.Surface,frame.left,frame.bottom-2,0xFFFFFFFF);
+	const C4BltVertex vertices[] = {
+		{ 0.f, 0.f, { 0xFF, 0xFF, 0xFF, 0xFF }, frame.left+EDGE_WIDTH, frame.top, 0.f },
+		{ 0.f, 0.f, { 0xFF, 0xFF, 0xFF, 0xFF }, frame.left, frame.top, 0.f },
+		{ 0.f, 0.f, { 0xFF, 0xFF, 0xFF, 0xFF }, frame.left, frame.top, 0.f },
+		{ 0.f, 0.f, { 0xFF, 0xFF, 0xFF, 0xFF }, frame.left, frame.top+EDGE_WIDTH, 0.f },
 
-	pDraw->DrawPix(cgo.Surface,frame.right-1,frame.top,0xFFFFFFFF);
-	pDraw->DrawPix(cgo.Surface,frame.right-2,frame.top,0xFFFFFFFF);
-	pDraw->DrawPix(cgo.Surface,frame.right-1,frame.top+1,0xFFFFFFFF);
+		{ 0.f, 0.f, { 0xFF, 0xFF, 0xFF, 0xFF }, frame.left+EDGE_WIDTH, frame.bottom-1, 0.f },
+		{ 0.f, 0.f, { 0xFF, 0xFF, 0xFF, 0xFF }, frame.left, frame.bottom-1, 0.f },
+		{ 0.f, 0.f, { 0xFF, 0xFF, 0xFF, 0xFF }, frame.left, frame.bottom-1, 0.f },
+		{ 0.f, 0.f, { 0xFF, 0xFF, 0xFF, 0xFF }, frame.left, frame.bottom-1-EDGE_WIDTH, 0.f },
 
-	pDraw->DrawPix(cgo.Surface,frame.right-1,frame.bottom-1,0xFFFFFFFF);
-	pDraw->DrawPix(cgo.Surface,frame.right-2,frame.bottom-1,0xFFFFFFFF);
-	pDraw->DrawPix(cgo.Surface,frame.right-1,frame.bottom-2,0xFFFFFFFF);
+		{ 0.f, 0.f, { 0xFF, 0xFF, 0xFF, 0xFF }, frame.right-1-EDGE_WIDTH, frame.top, 0.f },
+		{ 0.f, 0.f, { 0xFF, 0xFF, 0xFF, 0xFF }, frame.right-1, frame.top, 0.f },
+		{ 0.f, 0.f, { 0xFF, 0xFF, 0xFF, 0xFF }, frame.right-1, frame.top, 0.f },
+		{ 0.f, 0.f, { 0xFF, 0xFF, 0xFF, 0xFF }, frame.right-1, frame.top+EDGE_WIDTH, 0.f },
+
+		{ 0.f, 0.f, { 0xFF, 0xFF, 0xFF, 0xFF }, frame.right-1-EDGE_WIDTH, frame.bottom-1, 0.f },
+		{ 0.f, 0.f, { 0xFF, 0xFF, 0xFF, 0xFF }, frame.right-1, frame.bottom-1, 0.f },
+		{ 0.f, 0.f, { 0xFF, 0xFF, 0xFF, 0xFF }, frame.right-1, frame.bottom-1, 0.f },
+		{ 0.f, 0.f, { 0xFF, 0xFF, 0xFF, 0xFF }, frame.right-1, frame.bottom-1-EDGE_WIDTH, 0.f },
+	};
+
+	const unsigned int n_vertices = sizeof(vertices) / sizeof(vertices[0]);
+
+	pDraw->PerformMultiLines(cgo.Surface, vertices, n_vertices, 1.);
 }
 
 
@@ -565,13 +595,14 @@ void C4EditCursor::MoveSelection(C4Real XOff, C4Real YOff)
 void C4EditCursor::FrameSelection()
 {
 	ClearSelection();
-	C4Object *cobj; C4ObjectLink *clnk;
-	for (clnk=::Objects.First; clnk && (cobj=clnk->Obj); clnk=clnk->Next)
-		if (cobj->Status) if (cobj->OCF & OCF_NotContained)
-			{
-				if (Inside(cobj->GetX(),Min(X,X2),Max(X,X2)) && Inside(cobj->GetY(),Min(Y,Y2),Max(Y,Y2)))
-					AddToSelection(cobj);
-			}
+	for (C4Object *cobj : Objects)
+	{
+		if (cobj->Status && cobj->OCF & OCF_NotContained)
+		{
+			if (Inside(cobj->GetX(),Min(X,X2),Max(X,X2)) && Inside(cobj->GetY(),Min(Y,Y2),Max(Y,Y2)))
+				AddToSelection(cobj);
+		}
+	}
 	OnSelectionChanged();
 }
 
@@ -712,32 +743,74 @@ bool C4EditCursor::DoContextMenu(DWORD dwKeyState)
 	SetMenuItemText(hContext,IDM_VIEWPORT_DUPLICATE,LoadResStr("IDS_MNU_DUPLICATE"));
 	SetMenuItemText(hContext,IDM_VIEWPORT_CONTENTS,LoadResStr("IDS_MNU_CONTENTS"));
 
-	ObjselectDelItems();
+	// Add selection and custom command entries for any objects at the cursor
+	ObjselectDelItems(); // clear previous entries
 	C4FindObjectAtPoint pFO(X,Y);
 	C4ValueArray * atcursor; atcursor = pFO.FindMany(::Objects, ::Objects.Sectors); // needs freeing (single object ptr)
 	int itemcount = atcursor->GetSize();
 	if(itemcount > 0)
 	{
-		const int maxitems = 25; // Maximum displayed objects. if you raise it, also change note with IDM_VPORTDYN_FIRST in resource.h
-		if(itemcount > maxitems) itemcount = maxitems+1;
-		itemsObjselect.resize(itemcount+1); // +1 for a separator
+		// Count required entries for all objects and their custom commands
+		int entrycount = itemcount;
+		for (int i_item = 0; i_item < itemcount; ++i_item)
+		{
+			C4Object *pObj = (*atcursor)[i_item].getObj(); assert(pObj);
+			C4ValueArray *custom_commands = pObj->GetPropertyArray(P_EditCursorCommands);
+			if (custom_commands) entrycount += custom_commands->GetSize();
+		}
+		// If too many entries would be shown, add a "..." in the end
+		const int maxentries = 25; // Maximum displayed objects. if you raise it, also change note with IDM_VPORTDYN_FIRST in resource.h
+		bool has_too_many_entries = (entrycount > maxentries);
+		if (has_too_many_entries) entrycount = maxentries + 1;
+		itemsObjselect.resize(entrycount + 1); // +1 for a separator
+		// Add a separator bar
 		itemsObjselect[0].ItemId = IDM_VPORTDYN_FIRST;
 		itemsObjselect[0].Object = NULL;
+		itemsObjselect[0].Command.Clear();
 		AppendMenu(hContext, MF_SEPARATOR, IDM_VPORTDYN_FIRST, NULL);
-		int i = 1;
-		for(std::vector<ObjselItemDt>::iterator it = itemsObjselect.begin() + 1; it != itemsObjselect.end(); ++it, ++i)
+		// Add all objects
+		int i_entry = 0;
+		for (int i_item = 0; i_item < itemcount; ++i_item)
 		{
-			C4Object * obj = (*atcursor)[i-1].getObj();
-			assert(obj);
-			it->ItemId = IDM_VPORTDYN_FIRST+i;
-			it->Object = obj;
-			AppendMenu(hContext, MF_STRING, it->ItemId, FormatString("%s #%i (%i/%i)", obj->GetName(), obj->Number, obj->GetX(), obj->GetY()).GetWideChar());
+			++i_entry; if (i_entry >= maxentries) break;
+			// Add selection entry
+			C4Object *obj = (*atcursor)[i_item].getObj();
+			itemsObjselect[i_entry].ItemId = IDM_VPORTDYN_FIRST + i_entry;
+			itemsObjselect[i_entry].Object = obj;
+			itemsObjselect[i_entry].Command.Clear();
+			AppendMenu(hContext, MF_STRING, IDM_VPORTDYN_FIRST + i_entry, FormatString("%s #%i (%i/%i)", obj->GetName(), obj->Number, obj->GetX(), obj->GetY()).GetWideChar());
+			// Add custom command entries
+			C4ValueArray *custom_commands = obj->GetPropertyArray(P_EditCursorCommands);
+			if (custom_commands) for (int i_cmd = 0; i_cmd < custom_commands->GetSize(); ++i_cmd)
+			{
+				++i_entry; if (i_entry >= maxentries) break;
+				const C4Value &cmd = custom_commands->GetItem(i_cmd);
+				StdStrBuf custom_command_szstr; C4AulFunc *custom_command; C4String *custom_command_string;
+				// Custom command either by string or by function pointer
+				if ((custom_command = cmd.getFunction()))
+					custom_command_szstr.Format("%s()", custom_command->GetName());
+				else if ((custom_command_string = cmd.getStr()))
+					custom_command_szstr.Copy(custom_command_string->GetData()); // copy just in case script get reloaded inbetween
+				if (custom_command_szstr.getLength())
+				{
+					itemsObjselect[i_entry].ItemId = IDM_VPORTDYN_FIRST + i_entry;
+					itemsObjselect[i_entry].Object = obj;
+					itemsObjselect[i_entry].Command.Take(custom_command_szstr); 
+					AppendMenu(hContext, MF_STRING, IDM_VPORTDYN_FIRST + i_entry, FormatString("%s->%s", obj->GetName(), custom_command_szstr.getData()).GetWideChar());
+				}
+				else
+				{
+					// invalid entry in commands list. do not create a menu item.
+					itemsObjselect[i_entry].ItemId = 0;
+				}
+			}
 		}
-		if(atcursor->GetSize() > maxitems)
+		if (has_too_many_entries)
 		{
-			AppendMenu(hContext, MF_GRAYED, IDM_VPORTDYN_FIRST+maxitems+1, L"...");
-			itemsObjselect[maxitems+1].ItemId = IDM_VPORTDYN_FIRST+maxitems+1;
-			itemsObjselect[maxitems+1].Object = NULL;
+			AppendMenu(hContext, MF_GRAYED, IDM_VPORTDYN_FIRST + maxentries + 1, L"...");
+			itemsObjselect[maxentries + 1].ItemId = IDM_VPORTDYN_FIRST + maxentries + 1;
+			itemsObjselect[maxentries + 1].Object = NULL;
+			itemsObjselect[maxentries + 1].Command.Clear();
 		}
 	}
 	delete atcursor;
@@ -758,7 +831,10 @@ bool C4EditCursor::DoContextMenu(DWORD dwKeyState)
 		for(std::vector<ObjselItemDt>::iterator it = itemsObjselect.begin() + 1; it != itemsObjselect.end(); ++it)
 			if(it->ItemId == iItem)
 			{
-				DoContextObjsel(it->Object, (dwKeyState & MK_SHIFT) == 0);
+				if (it->Command.getLength())
+					DoContextObjCommand(it->Object, it->Command.getData());
+				else
+					DoContextObjsel(it->Object, (dwKeyState & MK_SHIFT) == 0);
 				break;
 			}
 		break;
@@ -817,19 +893,20 @@ void C4EditCursor::GrabContents()
 
 void C4EditCursor::UpdateDropTarget(DWORD dwKeyState)
 {
-	C4Object *cobj; C4ObjectLink *clnk;
 
 	DropTarget=NULL;
 
 	if (dwKeyState & MK_CONTROL)
 		if (Selection.GetObject())
-			for (clnk=::Objects.First; clnk && (cobj=clnk->Obj); clnk=clnk->Next)
+			for (C4Object *cobj : Objects)
+			{
 				if (cobj->Status)
 					if (!cobj->Contained)
 						if (Inside<int32_t>(X-(cobj->GetX()+cobj->Shape.x),0,cobj->Shape.Wdt-1))
 							if (Inside<int32_t>(Y-(cobj->GetY()+cobj->Shape.y),0,cobj->Shape.Hgt-1))
 								if (!Selection.GetLink(cobj))
 									{ DropTarget=cobj; break; }
+			}
 
 }
 
@@ -915,9 +992,11 @@ void C4EditCursor::EMMoveObject(C4ControlEMObjectAction eAction, C4Real tx, C4Re
 		pObjIDs = new int32_t [iObjCnt];
 		// fill
 		int32_t i = 0;
-		for (C4ObjectLink *pLnk = pObjs->First; pLnk; pLnk = pLnk->Next, i++)
-			if (pLnk->Obj && pLnk->Obj->Status)
-				pObjIDs[i] = pLnk->Obj->Number;
+		for (C4Object *obj : *pObjs)
+		{
+			if (obj && obj->Status)
+				pObjIDs[i] = obj->Number;
+		}
 	}
 
 	// execute control
@@ -1012,3 +1091,10 @@ void C4EditCursor::DoContextObjsel(C4Object * obj, bool clear)
 	Selection.Add(obj, C4ObjectList::stNone);
 	OnSelectionChanged();
 }
+
+void C4EditCursor::DoContextObjCommand(C4Object * obj, const char *cmd)
+{
+	// Command going through queue for sync
+	if (!obj || !cmd) return;
+	In(FormatString("Object(%d)->%s", obj->Number, cmd).getData());
+}
