@@ -20,8 +20,6 @@
 #include <C4ObjectListDlg.h>
 #include <C4Console.h>
 #include <C4Object.h>
-#include <C4Language.h>
-#include <C4Game.h>
 #include <C4GameObjects.h>
 
 
@@ -792,13 +790,7 @@ void C4ObjectListDlg::Open()
 		g_signal_connect(G_OBJECT(treeview), "row-activated", G_CALLBACK(OnRowActivated), this);
 
 		GtkTreeViewColumn * col = gtk_tree_view_column_new();
-		GtkCellRenderer * renderer;
-#if 0
-		renderer = gtk_cell_renderer_pixbuf_new();
-		gtk_tree_view_column_pack_start(col, renderer, false);
-		gtk_tree_view_column_set_cell_data_func(col, renderer, icon_cell_data_func, g_hash_table_new_full(NULL, NULL, NULL, (GDestroyNotify)g_object_unref), (GDestroyNotify)g_hash_table_unref);
-#endif
-		renderer = gtk_cell_renderer_text_new();
+		GtkCellRenderer * renderer = gtk_cell_renderer_text_new();
 		gtk_tree_view_column_pack_start(col, renderer, true);
 		gtk_tree_view_column_set_cell_data_func(col, renderer, name_cell_data_func, NULL, NULL);
 
