@@ -95,15 +95,25 @@ bool C4Application::DoInit(int argc, char * argv[])
 			Config.Load();
 		}
 	}
+	// Open log
+	OpenLog();
+
+	Revision.Ref(C4REVISION);
+
+	// Engine header message
+	Log(C4ENGINEINFOLONG);
+	LogF("Version: %s %s (%s)", C4VERSION, C4_OS, Revision.getData());
+	LogF("ExePath: \"%s\"", Config.General.ExePath.getData());
+	LogF("SystemDataPath: \"%s\"", Config.General.SystemDataPath);
+	LogF("UserDataPath: \"%s\"", Config.General.UserDataPath);
+
 	// Init C4Group
 	C4Group_SetProcessCallback(&ProcessCallback);
 	C4Group_SetTempPath(Config.General.TempPath.getData());
 	C4Group_SetSortList(C4CFN_FLS);
 
-	// Open log
-	OpenLog();
-
-	Revision.Ref(C4REVISION);
+	// Cleanup temp folders left behind
+	Config.CleanupTempUpdateFolder();
 
 	// Initialize game data paths
 	Reloc.Init();
@@ -123,6 +133,9 @@ bool C4Application::DoInit(int argc, char * argv[])
 	// Parse command line
 	ParseCommandLine(argc, argv);
 
+	// Open additional logs that depend on command line
+	OpenExtraLogs();
+
 	// Init external language packs
 	Languages.Init();
 	// Load language string table
@@ -161,12 +174,8 @@ bool C4Application::DoInit(int argc, char * argv[])
 	// init timers (needs window)
 	Add(pGameTimer = new C4ApplicationGameTimer());
 
-	// Engine header message
-	Log(C4ENGINEINFOLONG);
-	LogF("Version: %s %s (%s)", C4VERSION, C4_OS, Revision.getData());
-
 	// Initialize OpenGL
-	bool success = DDrawInit(this, !!isEditor, false, GetConfigWidth(), GetConfigHeight(), Config.Graphics.BitDepth, Config.Graphics.Monitor);
+	bool success = DDrawInit(this, GetConfigWidth(), GetConfigHeight(), Config.Graphics.BitDepth, Config.Graphics.Monitor);
 	if (!success) { LogFatal(LoadResStr("IDS_ERR_DDRAW")); Clear(); ShowGfxErrorDialog(); return false; }
 
 	if (!isEditor)
@@ -242,6 +251,7 @@ void C4Application::ParseCommandLine(int argc, char * argv[])
 			{"tcpport", required_argument, 0, 't'},
 			{"join", required_argument, 0, 'j'},
 			{"language", required_argument, 0, 'L'},
+			{"scenpar", required_argument, 0, 'S'},
 
 			{"observe", no_argument, 0, 'o'},
 			{"nonetwork", no_argument, 0, 'N'},
@@ -249,6 +259,8 @@ void C4Application::ParseCommandLine(int argc, char * argv[])
 			{"record", no_argument, 0, 'r'},
 
 			{"lobby", required_argument, 0, 'l'},
+
+			{"debug-opengl", no_argument, &Config.Graphics.DebugOpenGL, 1},
 			{0, 0, 0, 0}
 		};
 		int option_index = 0;
@@ -334,6 +346,15 @@ void C4Application::ParseCommandLine(int argc, char * argv[])
 		case 'D': Game.DebugPort = atoi(optarg); break;
 		case 'P': Game.DebugPassword = optarg; break;
 		case 'H': Game.DebugHost = optarg; break;
+		// set custom scenario parameter by command line
+		case 'S':
+			{
+			StdStrBuf sopt, soptval; sopt.Copy(optarg);
+			int32_t val=1;
+			if (sopt.SplitAtChar('=', &soptval)) val=atoi(soptval.getData());
+			Game.StartupScenarioParameters.SetValue(sopt.getData(), val, false);
+			}
+			break;
 		// debug configs
 		case 'h':
 			Game.NetworkActive = true;
@@ -490,6 +511,13 @@ bool C4Application::PreInit()
 	// startup dialog: Only use if no next mission has been provided
 	bool fUseStartupDialog = !Game.HasScenario();
 
+	// Load graphics early, before we draw anything, since we need shaders
+	// loaded to draw.
+	Game.SetInitProgress(0.0f);
+	Log(LoadResStr("IDS_PRC_GFXRES"));
+	if (!GraphicsResource.Init()) return false;
+	Game.SetInitProgress(fUseStartupDialog ? 10.0f : 1.0f);
+
 	// Startup message board
 	if (!isEditor)
 		if (Config.Graphics.ShowStartupMessages || Game.NetworkActive)
@@ -497,7 +525,6 @@ bool C4Application::PreInit()
 			C4Facet cgo; cgo.Set(FullScreen.pSurface,0,0,C4GUI::GetScreenWdt(), C4GUI::GetScreenHgt());
 			GraphicsSystem.MessageBoard.Init(cgo,true);
 		}
-	Game.SetInitProgress(0.0f);
 
 	// init loader: Black screen for first start if a video is to be shown; otherwise default spec
 	if (fUseStartupDialog && !isEditor)
@@ -505,7 +532,7 @@ bool C4Application::PreInit()
 		if (!::GraphicsSystem.InitLoaderScreen(C4CFN_StartupBackgroundMain))
 			{ LogFatal(LoadResStr("IDS_PRC_ERRLOADER")); return false; }
 	}
-	Game.SetInitProgress(fUseStartupDialog ? 10.0f : 1.0f);
+	Game.SetInitProgress(fUseStartupDialog ? 20.0f : 2.0f);
 
 	if (!Game.PreInit()) return false;
 
@@ -513,16 +540,16 @@ bool C4Application::PreInit()
 	if (!MusicSystem.Init("Frontend.*"))
 		Log(LoadResStr("IDS_PRC_NOMUSIC"));
 
-	// Play some music!
-	if (fUseStartupDialog && !isEditor && Config.Sound.FEMusic)
-		MusicSystem.Play();
-
 	Game.SetInitProgress(fUseStartupDialog ? 34.0f : 2.0f);
 
 	// Sound
 	if (!SoundSystem.Init())
 		Log(LoadResStr("IDS_PRC_NOSND"));
 
+	// Play some music! - after sound init because sound system might be needed by music system
+	if (fUseStartupDialog && !isEditor && Config.Sound.FEMusic)
+		MusicSystem.Play();
+
 	Game.SetInitProgress(fUseStartupDialog ? 35.0f : 3.0f);
 
 	if (fUseStartupDialog)
@@ -643,6 +670,7 @@ void C4Application::GameTick()
 		break;
 	case C4AS_Startup:
 		SoundSystem.Execute();
+		MusicSystem.Execute();
 		// wait for the user to start a game
 		break;
 	case C4AS_StartGame:
@@ -681,9 +709,9 @@ void C4Application::GameTick()
 		// Game
 		if (Game.IsRunning)
 			Game.Execute();
-		Game.DoSkipFrame = false;
 		// Sound
 		SoundSystem.Execute();
+		MusicSystem.Execute();
 		// Gamepad
 		if (pGamePadControl) pGamePadControl->Execute();
 		break;
@@ -693,15 +721,13 @@ void C4Application::GameTick()
 void C4Application::Draw()
 {
 	// Graphics
-	if (!Game.DoSkipFrame)
-	{
-		// Fullscreen mode
-		if (!isEditor)
-			FullScreen.Execute();
-		// Console mode
-		else
-			Console.Execute();
-	}
+
+	// Fullscreen mode
+	if (!isEditor)
+		FullScreen.Execute();
+	// Console mode
+	else
+		Console.Execute();
 }
 
 void C4Application::SetGameTickDelay(int iDelay)
@@ -836,26 +862,28 @@ bool C4Application::FullScreenMode()
 
 C4ApplicationGameTimer::C4ApplicationGameTimer()
 		: CStdMultimediaTimerProc(26),
-		tLastGameTick(C4TimeMilliseconds::NegativeInfinity), iGameTickDelay(0)
+		tLastGameTick(C4TimeMilliseconds::NegativeInfinity), iGameTickDelay(28), iExtraGameTickDelay(0)
 {
 }
 
 void C4ApplicationGameTimer::SetGameTickDelay(uint32_t iDelay)
 {
+	// Remember delay
+	iGameTickDelay = iDelay;
 	// Smaller than minimum refresh delay?
 	if (iDelay < uint32_t(Config.Graphics.MaxRefreshDelay))
 	{
 		// Set critical timer
 		SetDelay(iDelay);
 		// No additional breaking needed
-		iGameTickDelay = 0;
+		iExtraGameTickDelay = 0;
 	}
 	else
 	{
 		// Set critical timer
 		SetDelay(Config.Graphics.MaxRefreshDelay);
 		// Slow down game tick
-		iGameTickDelay = iDelay;
+		iExtraGameTickDelay = iDelay;
 	}
 }
 
@@ -865,9 +893,9 @@ bool C4ApplicationGameTimer::Execute(int iTimeout, pollfd *)
 	if (!CheckAndReset()) return true;
 	C4TimeMilliseconds tNow = C4TimeMilliseconds::Now();
 	// Execute
-	if (tNow >= tLastGameTick + iGameTickDelay || Game.GameGo)
+	if (tNow >= tLastGameTick + iExtraGameTickDelay || Game.GameGo)
 	{
-		if(iGameTickDelay)
+		if (iGameTickDelay)
 			tLastGameTick += iGameTickDelay;
 		else
 			tLastGameTick = tNow;
@@ -878,8 +906,18 @@ bool C4ApplicationGameTimer::Execute(int iTimeout, pollfd *)
 
 		Application.GameTick();
 	}
-	// Draw always
-	Application.Draw();
+	// Draw
+	if (!Game.DoSkipFrame)
+	{
+		C4TimeMilliseconds tPreGfxTime = C4TimeMilliseconds::Now();
+
+		Application.Draw();
+
+		// Automatic frame skip if graphics are slowing down the game (skip max. every 2nd frame)
+		Game.DoSkipFrame = Game.Parameters.AutoFrameSkip && (tPreGfxTime + iGameTickDelay < C4TimeMilliseconds::Now());
+	} else {
+		Game.DoSkipFrame=false;
+	}
 	return true;
 }
 
