@@ -87,7 +87,7 @@ C4FindObject *C4FindObject::CreateByValue(const C4Value &DataVal, C4SortObject *
 		return new C4FindObjectExclude(Data[1].getObj());
 
 	case C4FO_ID:
-		return new C4FindObjectID(Data[1].getC4ID());
+		return new C4FindObjectDef(Data[1].getPropList());
 
 
 	// #973: For all criteria using coordinates: If FindObject et al. are called in object context, offset by object center
@@ -191,7 +191,7 @@ C4FindObject *C4FindObject::CreateByValue(const C4Value &DataVal, C4SortObject *
 	{
 		int index = 0;
 		if (Data.GetSize() >= 3)
-			index = BoundBy(Data[2].getInt(), 0, 1);
+			index = Clamp(Data[2].getInt(), 0, 1);
 		return new C4FindObjectActionTarget(Data[1].getObj(), index);
 	}
 
@@ -213,6 +213,9 @@ C4FindObject *C4FindObject::CreateByValue(const C4Value &DataVal, C4SortObject *
 	case C4FO_Layer:
 		return new C4FindObjectLayer(Data[1].getObj());
 
+	case C4FO_InArray:
+		return new C4FindObjectInArray(Data[1].getArray());
+
 	}
 	return NULL;
 }
@@ -226,10 +229,9 @@ int32_t C4FindObject::Count(const C4ObjectList &Objs)
 		return Objs.ObjectCount();
 	// Count
 	int32_t iCount = 0;
-	for (C4ObjectLink *pLnk = Objs.First; pLnk; pLnk = pLnk->Next)
-		if (pLnk->Obj->Status)
-			if (Check(pLnk->Obj))
-				iCount++;
+	for (C4Object *obj : Objs)
+		if (obj->Status && Check(obj))
+			iCount++;
 	return iCount;
 }
 
@@ -241,17 +243,17 @@ C4Object *C4FindObject::Find(const C4ObjectList &Objs)
 	// Search
 	// Double-check object status, as object might be deleted after Check()!
 	C4Object *pBestResult = NULL;
-	for (C4ObjectLink *pLnk = Objs.First; pLnk; pLnk = pLnk->Next)
-		if (pLnk->Obj->Status)
-			if (Check(pLnk->Obj))
-				if (pLnk->Obj->Status)
+	for (C4Object *obj : Objs)
+		if (obj->Status)
+			if (Check(obj))
+				if (obj->Status)
 				{
 					// no sorting: Use first object found
-					if (!pSort) return pLnk->Obj;
+					if (!pSort) return obj;
 					// Sorting: Check if found object is better
-					if (!pBestResult || pSort->Compare(pLnk->Obj, pBestResult) > 0)
-						if (pLnk->Obj->Status)
-							pBestResult = pLnk->Obj;
+					if (!pBestResult || pSort->Compare(obj, pBestResult) > 0)
+						if (obj->Status)
+							pBestResult = obj;
 				}
 	return pBestResult;
 }
@@ -266,15 +268,15 @@ C4ValueArray *C4FindObject::FindMany(const C4ObjectList &Objs)
 	C4ValueArray *pArray = new C4ValueArray(32);
 	int32_t iSize = 0;
 	// Search
-	for (C4ObjectLink *pLnk = Objs.First; pLnk; pLnk = pLnk->Next)
-		if (pLnk->Obj->Status)
-			if (Check(pLnk->Obj))
+	for (C4Object *obj : Objs)
+		if (obj->Status)
+			if (Check(obj))
 			{
 				// Grow the array, if neccessary
 				if (iSize >= pArray->GetSize())
 					pArray->SetSize(iSize * 2);
 				// Add object
-				(*pArray)[iSize++] = C4VObj(pLnk->Obj);
+				(*pArray)[iSize++] = C4VObj(obj);
 			}
 	// Shrink array
 	pArray->SetSize(iSize);
@@ -308,12 +310,12 @@ int32_t C4FindObject::Count(const C4ObjectList &Objs, const C4LSectors &Sct)
 		uint32_t iMarker = ::Objects.GetNextMarker();
 		int32_t iCount = 0;
 		for (; pLst; pLst=Area.NextObjectShapes(pLst, &pSct))
-			for (C4ObjectLink *pLnk = pLst->First; pLnk; pLnk = pLnk->Next)
-				if (pLnk->Obj->Status)
-					if (pLnk->Obj->Marker != iMarker)
+			for (C4Object *obj : Objs)
+				if (obj->Status)
+					if (obj->Marker != iMarker)
 					{
-						pLnk->Obj->Marker = iMarker;
-						if (Check(pLnk->Obj))
+						obj->Marker = iMarker;
+						if (Check(obj))
 							iCount++;
 					}
 		return iCount;
@@ -398,18 +400,18 @@ C4ValueArray *C4FindObject::FindMany(const C4ObjectList &Objs, const C4LSectors
 		// Create marker, search all areas
 		uint32_t iMarker = ::Objects.GetNextMarker();
 		for (; pLst; pLst=Area.NextObjectShapes(pLst, &pSct))
-			for (C4ObjectLink *pLnk = pLst->First; pLnk; pLnk = pLnk->Next)
-				if (pLnk->Obj->Status)
-					if (pLnk->Obj->Marker != iMarker)
+			for (C4Object *obj : *pLst)
+				if (obj->Status)
+					if (obj->Marker != iMarker)
 					{
-						pLnk->Obj->Marker = iMarker;
-						if (Check(pLnk->Obj))
+						obj->Marker = iMarker;
+						if (Check(obj))
 						{
 							// Grow the array, if neccessary
 							if (iSize >= pArray->GetSize())
 								pArray->SetSize(iSize * 2);
 							// Add object
-							(*pArray)[iSize++] = C4VObj(pLnk->Obj);
+							(*pArray)[iSize++] = C4VObj(obj);
 						}
 					}
 	}
@@ -420,15 +422,15 @@ C4ValueArray *C4FindObject::FindMany(const C4ObjectList &Objs, const C4LSectors
 		// Search
 		C4LArea Area(&::Objects.Sectors, *pBounds); C4LSector *pSct;
 		for (C4ObjectList *pLst=Area.FirstObjects(&pSct); pLst; pLst=Area.NextObjects(pLst, &pSct))
-			for (C4ObjectLink *pLnk = pLst->First; pLnk; pLnk = pLnk->Next)
-				if (pLnk->Obj->Status)
-					if (Check(pLnk->Obj))
+			for (C4Object *obj : *pLst)
+				if (obj->Status)
+					if (Check(obj))
 					{
 						// Grow the array, if neccessary
 						if (iSize >= pArray->GetSize())
 							pArray->SetSize(iSize * 2);
 						// Add object
-						(*pArray)[iSize++] = C4VObj(pLnk->Obj);
+						(*pArray)[iSize++] = C4VObj(obj);
 					}
 	}
 	// Shrink array
@@ -606,15 +608,14 @@ bool C4FindObjectExclude::Check(C4Object *pObj)
 	return pObj != pExclude;
 }
 
-bool C4FindObjectID::Check(C4Object *pObj)
+bool C4FindObjectDef::Check(C4Object *pObj)
 {
-	return pObj->id == id;
+	return pObj->GetPrototype() == def;
 }
 
-bool C4FindObjectID::IsImpossible()
+bool C4FindObjectDef::IsImpossible()
 {
-	C4Def * pDef = C4Id2Def(id);
-	return !pDef || !pDef->Count;
+	return !def || !def->GetDef() || !def->GetDef()->Count;
 }
 
 bool C4FindObjectInRect::Check(C4Object *pObj)
@@ -768,6 +769,24 @@ bool C4FindObjectLayer::IsImpossible()
 	return false;
 }
 
+// *** C4FindObjectInArray
+
+bool C4FindObjectInArray::Check(C4Object *pObj)
+{
+	// O(n) array look-up
+	if (!pArray) return false;
+	int32_t sz = pArray->GetSize();
+	for (int i=0; i<sz; ++i)
+		if (pArray->_GetItem(i).getObj() == pObj)
+			return true;
+	return false;
+}
+
+bool C4FindObjectInArray::IsImpossible()
+{
+	return !pArray || !pArray->GetSize();
+}
+
 // *** C4SortObject
 
 C4SortObject *C4SortObject::CreateByValue(const C4Value &DataVal, const C4Object *context)
