@@ -33,7 +33,6 @@
 #include <C4Log.h>
 #include <C4MessageInput.h>
 #include <C4MouseControl.h>
-#include <C4ObjectInfoList.h>
 #include <C4Player.h>
 #include <C4PlayerList.h>
 #include <C4PXS.h>
@@ -41,6 +40,7 @@
 #include <C4Texture.h>
 #include <C4Weather.h>
 #include <C4Viewport.h>
+#include <C4FoW.h>
 
 // undocumented!
 static bool FnIncinerateLandscape(C4PropList * _this, long iX, long iY)
@@ -51,7 +51,7 @@ static bool FnIncinerateLandscape(C4PropList * _this, long iX, long iY)
 
 static C4Void FnSetGravity(C4PropList * _this, long iGravity)
 {
-	::Landscape.Gravity = C4REAL100(BoundBy<long>(iGravity,-1000,1000));
+	::Landscape.Gravity = C4REAL100(Clamp<long>(iGravity,-1000,1000));
 	return C4Void();
 }
 
@@ -118,13 +118,13 @@ static Nillable<long> FnGetY(C4PropList * _this, long iPrec)
 	return fixtoi(Object(_this)->fix_y, iPrec);
 }
 
-static C4Object *FnCreateObject(C4PropList * _this,
+static C4Object *FnCreateObjectAbove(C4PropList * _this,
                                 C4PropList * PropList, long iXOffset, long iYOffset, Nillable<long> owner)
 {
 	if (Object(_this)) // Local object calls override
 	{
-		iXOffset+=Object(_this)->GetX();
-		iYOffset+=Object(_this)->GetY();
+		iXOffset += Object(_this)->GetX();
+		iYOffset += Object(_this)->GetY();
 	}
 
 	long iOwner = owner;
@@ -136,7 +136,7 @@ static C4Object *FnCreateObject(C4PropList * _this,
 			iOwner = NO_OWNER;
 	}
 
-	C4Object *pNewObj = Game.CreateObject(PropList,Object(_this),iOwner,iXOffset,iYOffset);
+	C4Object *pNewObj = Game.CreateObject(PropList, Object(_this), iOwner, iXOffset, iYOffset);
 
 	// Set initial controller to creating controller, so more complicated cause-effect-chains can be traced back to the causing player
 	if (pNewObj && Object(_this) && Object(_this)->Controller > NO_OWNER) pNewObj->Controller = Object(_this)->Controller;
@@ -144,6 +144,33 @@ static C4Object *FnCreateObject(C4PropList * _this,
 	return pNewObj;
 }
 
+static C4Object *FnCreateObject(C4PropList * _this,
+	C4PropList * PropList, long iXOffset, long iYOffset, Nillable<long> owner)
+{
+	if (Object(_this)) // Local object calls override
+	{
+		iXOffset += Object(_this)->GetX();
+		iYOffset += Object(_this)->GetY();
+	}
+
+	long iOwner = owner;
+	if (owner.IsNil())
+	{
+		if (Object(_this))
+			iOwner = Object(_this)->Controller;
+		else
+			iOwner = NO_OWNER;
+	}
+
+	C4Object *pNewObj = Game.CreateObject(PropList, Object(_this), iOwner, iXOffset, iYOffset, 0, true);
+
+	// Set initial controller to creating controller, so more complicated cause-effect-chains can be traced back to the causing player
+	if (pNewObj && Object(_this) && Object(_this)->Controller > NO_OWNER) pNewObj->Controller = Object(_this)->Controller;
+
+	return pNewObj;
+}
+
+
 static C4Object *FnCreateConstruction(C4PropList * _this,
                                       C4PropList * PropList, long iXOffset, long iYOffset, Nillable<long> owner,
                                       long iCompletion, bool fTerrain, bool fCheckSite)
@@ -206,7 +233,7 @@ static bool FnCheckConstructionSite(C4PropList * _this, C4PropList * PropList, i
 {
 	// Make sure parameters are valid
 	if (!PropList || !PropList->GetDef())
-		return NULL;
+		return false;
 
 	// Local object calls override position offset, owner
 	if (Object(_this))
@@ -418,13 +445,13 @@ static bool FnGBackSky(C4PropList * _this, long x, long y)
 	return !GBackIFT(x, y);
 }
 
-static long FnExtractMaterialAmount(C4PropList * _this, long x, long y, long mat, long amount)
+static long FnExtractMaterialAmount(C4PropList * _this, long x, long y, long mat, long amount, bool distant_first)
 {
 	if (Object(_this)) { x+=Object(_this)->GetX(); y+=Object(_this)->GetY(); }
 	long extracted=0; for (; extracted<amount; extracted++)
 	{
 		if (GBackMat(x,y)!=mat) return extracted;
-		if (::Landscape.ExtractMaterial(x,y)!=mat) return extracted;
+		if (::Landscape.ExtractMaterial(x,y,distant_first)!=mat) return extracted;
 	}
 	return extracted;
 }
@@ -501,7 +528,7 @@ static bool FnSound(C4PropList * _this, C4String *szSound, bool fGlobal, Nillabl
 	return true;
 }
 
-static bool FnMusic(C4PropList * _this, C4String *szSongname, bool fLoop)
+static bool FnMusic(C4PropList * _this, C4String *szSongname, bool fLoop, long iFadeTime_ms)
 {
 	bool success;
 	if (!szSongname)
@@ -510,7 +537,7 @@ static bool FnMusic(C4PropList * _this, C4String *szSongname, bool fLoop)
 	}
 	else
 	{
-		success = Application.MusicSystem.Play(FnStringPar(szSongname), !!fLoop);
+		success = Application.MusicSystem.Play(FnStringPar(szSongname), !!fLoop, iFadeTime_ms);
 	}
 	if (::Control.SyncMode()) return true;
 	return success;
@@ -522,11 +549,19 @@ static long FnMusicLevel(C4PropList * _this, long iLevel)
 	return Application.MusicSystem.SetVolume(iLevel);
 }
 
-static long FnSetPlayList(C4PropList * _this, C4String *szPlayList)
+static long FnSetPlayList(C4PropList * _this, C4String *szPlayList, Nillable<long> iAtPlayer, bool fForceSwitch, long iFadeTime_ms)
 {
-	long iFilesInPlayList = Application.MusicSystem.SetPlayList(FnStringPar(szPlayList));
+	// If a player number is provided, set play list for clients where given player is local only
+	if (!iAtPlayer.IsNil() && iAtPlayer != NO_OWNER)
+	{
+		C4Player *at_plr = ::Players.Get(iAtPlayer);
+		if (!at_plr) return 0;
+		if (!at_plr->LocalControl) return 0;
+	}
+	// Set playlist; count entries
+	long iFilesInPlayList = Application.MusicSystem.SetPlayList(FnStringPar(szPlayList), fForceSwitch, iFadeTime_ms);
 	Game.PlayList.Copy(FnStringPar(szPlayList));
-	// network/record/replay: return 0
+	// network/record/replay: return 0 for sync reasons
 	if (::Control.SyncMode()) return 0;
 	return iFilesInPlayList;
 }
@@ -685,7 +720,8 @@ static const int PLRZOOM_Direct     = 0x01,
                  PLRZOOM_NoIncrease = 0x04,
                  PLRZOOM_NoDecrease = 0x08,
                  PLRZOOM_LimitMin   = 0x10,
-                 PLRZOOM_LimitMax   = 0x20;
+                 PLRZOOM_LimitMax   = 0x20,
+                 PLRZOOM_Set        = 0x40;
 
 static bool FnSetPlayerZoomByViewRange(C4PropList * _this, long plr_idx, long range_wdt, long range_hgt, long flags)
 {
@@ -705,13 +741,64 @@ static bool FnSetPlayerZoomByViewRange(C4PropList * _this, long plr_idx, long ra
 		C4Player *plr = ::Players.Get(plr_idx);
 		if (!plr) return false;
 		// adjust values in player
-		if (!(flags & (PLRZOOM_LimitMin | PLRZOOM_LimitMax)))
-			plr->SetZoomByViewRange(range_wdt, range_hgt, !!(flags & PLRZOOM_Direct), !!(flags & PLRZOOM_NoIncrease), !!(flags & PLRZOOM_NoDecrease));
-		else
+		if (flags & PLRZOOM_LimitMin) plr->SetMinZoomByViewRange(range_wdt, range_hgt, !!(flags & PLRZOOM_NoIncrease), !!(flags & PLRZOOM_NoDecrease));
+		if (flags & PLRZOOM_LimitMax) plr->SetMaxZoomByViewRange(range_wdt, range_hgt, !!(flags & PLRZOOM_NoIncrease), !!(flags & PLRZOOM_NoDecrease));
+		// set values after setting min/max to ensure old limits don't block new value
+		if ((flags & PLRZOOM_Set) || !(flags & (PLRZOOM_LimitMin | PLRZOOM_LimitMax)))
 		{
-			if (flags & PLRZOOM_LimitMin) plr->SetMinZoomByViewRange(range_wdt, range_hgt, !!(flags & PLRZOOM_NoIncrease), !!(flags & PLRZOOM_NoDecrease));
-			if (flags & PLRZOOM_LimitMax) plr->SetMaxZoomByViewRange(range_wdt, range_hgt, !!(flags & PLRZOOM_NoIncrease), !!(flags & PLRZOOM_NoDecrease));
+			plr->SetZoomByViewRange(range_wdt, range_hgt, !!(flags & PLRZOOM_Direct), !!(flags & PLRZOOM_NoIncrease), !!(flags & PLRZOOM_NoDecrease));
 		}
+
+	}
+	return true;
+}
+
+static C4PropList *FnGetPlayerZoomLimits(C4PropList * _this, long plr_idx)
+{
+	// get player
+	C4Player *plr = ::Players.Get(plr_idx);
+	if (!plr) return NULL;
+	// collect limits in a prop list
+	// if neither width not height is set for zoom limits, return engine defaults.
+	C4PropList *result = C4PropList::New();
+	if (!result) return NULL;
+	result->SetPropertyByS(::Strings.RegString("MaxWidth"), C4VInt((plr->ZoomLimitMaxWdt || plr->ZoomLimitMaxHgt) ? plr->ZoomLimitMaxWdt : C4VP_DefMaxViewRangeX));
+	result->SetPropertyByS(::Strings.RegString("MaxHeight"), C4VInt(plr->ZoomLimitMaxHgt));
+	result->SetPropertyByS(::Strings.RegString("MaxValue"), C4VInt(fixtoi(plr->ZoomLimitMaxVal, 100)));
+	result->SetPropertyByS(::Strings.RegString("MinWidth"), C4VInt((plr->ZoomLimitMinWdt || plr->ZoomLimitMinHgt) ? plr->ZoomLimitMinWdt : C4VP_DefMinViewRangeX));
+	result->SetPropertyByS(::Strings.RegString("MinHeight"), C4VInt(plr->ZoomLimitMinHgt));
+	result->SetPropertyByS(::Strings.RegString("MinValue"), C4VInt(fixtoi(plr->ZoomLimitMinVal, 100)));
+	return result;
+}
+
+static bool FnSetPlayerZoom(C4PropList * _this, long plr_idx, long zoom, long precision, long flags)
+{
+	// parameter safety. 0/0 means "reset to default".
+	if (zoom < 0 || precision < 0) return false;
+	// special player NO_OWNER: apply to all viewports
+	if (plr_idx == NO_OWNER)
+	{
+		for (C4Player *plr = ::Players.First; plr; plr=plr->Next)
+			if (plr->Number != NO_OWNER) // can't happen, but would be a crash if it did...
+				FnSetPlayerZoom(_this, plr->Number, zoom, precision, flags);
+		return true;
+	}
+	else
+	{
+		// zoom factor calculation
+		if (!precision) precision = 1;
+		C4Fixed fZoom = itofix(zoom, precision);
+		// safety check on player only, so function return result is always in sync
+		C4Player *plr = ::Players.Get(plr_idx);
+		if (!plr) return false;
+		// adjust values in player
+		if (flags & PLRZOOM_LimitMin) plr->SetMinZoom(fZoom, !!(flags & PLRZOOM_NoIncrease), !!(flags & PLRZOOM_NoDecrease));
+		if (flags & PLRZOOM_LimitMax) plr->SetMaxZoom(fZoom, !!(flags & PLRZOOM_NoIncrease), !!(flags & PLRZOOM_NoDecrease));
+		if ((flags & PLRZOOM_Set) || !(flags & (PLRZOOM_LimitMin | PLRZOOM_LimitMax)))
+		{
+			plr->SetZoom(fZoom, !!(flags & PLRZOOM_Direct), !!(flags & PLRZOOM_NoIncrease), !!(flags & PLRZOOM_NoDecrease));
+		}
+
 	}
 	return true;
 }
@@ -732,45 +819,63 @@ static bool FnSetPlayerViewLock(C4PropList * _this, long plr_idx, bool is_locked
 	return true;
 }
 
-static bool FnDoHomebaseMaterial(C4PropList * _this, long iPlr, C4ID id, long iChange)
+static bool FnDoBaseMaterial(C4PropList * _this, long iPlr, C4ID id, long iChange)
 {
 	// validity check
 	if (!ValidPlr(iPlr)) return false;
 	C4Def *pDef = C4Id2Def(id);
 	if (!pDef) return false;
 	// add to material
-	long iLastcount = ::Players.Get(iPlr)->HomeBaseMaterial.GetIDCount(id);
-	return ::Players.Get(iPlr)->HomeBaseMaterial.SetIDCount(id,iLastcount+iChange,true);
+	long iLastcount = ::Players.Get(iPlr)->BaseMaterial.GetIDCount(id);
+	return ::Players.Get(iPlr)->BaseMaterial.SetIDCount(id,iLastcount+iChange,true);
 }
 
-static bool FnDoHomebaseProduction(C4PropList * _this, long iPlr, C4ID id, long iChange)
+static bool FnDoBaseProduction(C4PropList * _this, long iPlr, C4ID id, long iChange)
 {
 	// validity check
 	if (!ValidPlr(iPlr)) return false;
 	C4Def *pDef = C4Id2Def(id);
 	if (!pDef) return false;
 	// add to material
-	long iLastcount = ::Players.Get(iPlr)->HomeBaseProduction.GetIDCount(id);
-	return ::Players.Get(iPlr)->HomeBaseProduction.SetIDCount(id,iLastcount+iChange,true);
+	long iLastcount = ::Players.Get(iPlr)->BaseProduction.GetIDCount(id);
+	return ::Players.Get(iPlr)->BaseProduction.SetIDCount(id,iLastcount+iChange,true);
 }
 
-static bool FnSetPlrKnowledge(C4PropList * _this, long iPlr, C4ID id, bool fRemove)
+bool FnSetPlrKnowledge(C4Player *player, C4ID id, bool fRemove)
 {
-	C4Player *pPlr=::Players.Get(iPlr);
-	if (!pPlr) return false;
 	if (fRemove)
 	{
-		long iIndex=pPlr->Knowledge.GetIndex(id);
+		long iIndex = player->Knowledge.GetIndex(id);
 		if (iIndex<0) return false;
-		return pPlr->Knowledge.DeleteItem(iIndex);
+		return player->Knowledge.DeleteItem(iIndex);
 	}
 	else
 	{
 		if (!C4Id2Def(id)) return false;
-		return pPlr->Knowledge.SetIDCount(id,1,true);
+		return player->Knowledge.SetIDCount(id, 1, true);
 	}
 }
 
+static bool FnSetPlrKnowledge(C4PropList * _this, Nillable<long> iPlr, C4ID id, bool fRemove)
+{
+	
+	bool success = false;
+	// iPlr == nil: Call for all players
+	if (iPlr.IsNil())
+	{
+		for (C4Player *player = ::Players.First; player; player = player->Next)
+			if (FnSetPlrKnowledge(player, id, fRemove))
+				success = true;
+	}
+	else
+	{
+		// Otherwise call for requested player
+		C4Player *player = ::Players.Get(iPlr);
+		if (player) success = FnSetPlrKnowledge(player, id, fRemove);
+	}
+	return success;
+}
+
 static C4Value FnGetPlrKnowledge(C4PropList * _this, int iPlr, C4ID id, int iIndex, int dwCategory)
 {
 	if (!ValidPlr(iPlr)) return C4VBool(false);
@@ -810,22 +915,22 @@ static C4Value FnGetComponent(C4PropList * _this, C4ID idComponent, int iIndex,
 	return C4Value();
 }
 
-static C4Value FnGetHomebaseMaterial(C4PropList * _this, int iPlr, C4ID id, int iIndex, int dwCategory)
+static C4Value FnGetBaseMaterial(C4PropList * _this, int iPlr, C4ID id, int iIndex, int dwCategory)
 {
 	if (!ValidPlr(iPlr)) return C4VBool(false);
 	// Search by id, return available count
-	if (id) return C4VInt(::Players.Get(iPlr)->HomeBaseMaterial.GetIDCount(id));
+	if (id) return C4VInt(::Players.Get(iPlr)->BaseMaterial.GetIDCount(id));
 	// Search indexed item of given category, return C4ID
-	return C4VPropList(C4Id2Def(::Players.Get(iPlr)->HomeBaseMaterial.GetID( ::Definitions, dwCategory, iIndex )));
+	return C4VPropList(C4Id2Def(::Players.Get(iPlr)->BaseMaterial.GetID( ::Definitions, dwCategory, iIndex )));
 }
 
-static C4Value FnGetHomebaseProduction(C4PropList * _this, int iPlr, C4ID id, int iIndex, int dwCategory)
+static C4Value FnGetBaseProduction(C4PropList * _this, int iPlr, C4ID id, int iIndex, int dwCategory)
 {
 	if (!ValidPlr(iPlr)) return C4VBool(false);
 	// Search by id, return available count
-	if (id) return C4VInt(::Players.Get(iPlr)->HomeBaseProduction.GetIDCount(id));
+	if (id) return C4VInt(::Players.Get(iPlr)->BaseProduction.GetIDCount(id));
 	// Search indexed item of given category, return C4ID
-	return C4VPropList(C4Id2Def(::Players.Get(iPlr)->HomeBaseProduction.GetID( ::Definitions, dwCategory, iIndex )));
+	return C4VPropList(C4Id2Def(::Players.Get(iPlr)->BaseProduction.GetID( ::Definitions, dwCategory, iIndex )));
 }
 
 static long FnGetWealth(C4PropList * _this, long iPlr)
@@ -927,11 +1032,14 @@ static bool FnSurrenderPlayer(C4PropList * _this, long iPlr)
 }
 
 // undocumented!
-static bool FnSetLeaguePerformance(C4PropList * _this, long iScore)
-{
-	Game.RoundResults.SetLeaguePerformance(iScore);
+static bool FnSetLeaguePerformance(C4PropList * _this, long iScore, long idPlayer)
+	{
+	if(!Game.Parameters.isLeague()) return false;
+	if(idPlayer && !Game.PlayerInfos.GetPlayerInfoByID(idPlayer)) return false;
+	Game.RoundResults.SetLeaguePerformance(iScore, idPlayer);
 	return true;
-}
+	}
+
 
 static const int32_t CSPF_FixedAttributes = 1<<0,
     CSPF_NoScenarioInit  = 1<<1,
@@ -1025,6 +1133,20 @@ static long FnGetTemperature(C4PropList * _this)
 	return ::Weather.GetTemperature();
 }
 
+static C4Void FnSetAmbientBrightness(C4PropList * _this, long iBrightness)
+{
+	if (::Landscape.pFoW)
+		::Landscape.pFoW->Ambient.SetBrightness(iBrightness / 100.);
+	return C4Void();
+}
+
+static long FnGetAmbientBrightness(C4PropList * _this)
+{
+	if (!::Landscape.pFoW)
+		return 100;
+	return static_cast<long>(::Landscape.pFoW->Ambient.GetBrightness() * 100. + 0.5);
+}
+
 static C4Void FnSetSeason(C4PropList * _this, long iSeason)
 {
 	::Weather.SetSeason(iSeason);
@@ -1132,6 +1254,34 @@ static C4String *FnGetLeague(C4PropList * _this, long idx)
 	return String(sIdxLeague.getData());
 }
 
+static int32_t FnGetLeagueScore(C4PropList * _this, long idPlayer)
+{
+	// security
+	if (idPlayer < 1) return 0;
+	// get info
+	C4PlayerInfo *pInfo = Game.PlayerInfos.GetPlayerInfoByID(idPlayer);
+	if (!pInfo) return 0;
+	// get league score
+	return pInfo->getLeagueScore();
+}
+
+static bool FnSetLeagueProgressData(C4PropList * _this, C4String *pNewData, long idPlayer)
+{
+	if(!Game.Parameters.League.getLength() || !idPlayer) return false;
+	C4PlayerInfo *info = Game.PlayerInfos.GetPlayerInfoByID(idPlayer);
+	if (!info) return false;
+	info->SetLeagueProgressData(pNewData ? pNewData->GetCStr() : NULL);
+	return true;
+}
+
+static C4String *FnGetLeagueProgressData(C4PropList * _this, long idPlayer)
+{
+	if(!Game.Parameters.League.getLength()) return NULL;
+	C4PlayerInfo *info = Game.PlayerInfos.GetPlayerInfoByID(idPlayer);
+	if (!info) return NULL;
+	return String(info->GetLeagueProgressData());
+}
+
 // undocumented!
 static bool FnTestMessageBoard(C4PropList * _this, long iForPlr, bool fTestIfInUse)
 {
@@ -1566,8 +1716,9 @@ static bool FnCreateParticle(C4PropList * _this, C4String *name, C4Value x, C4Va
 
 static bool FnClearParticles(C4PropList * _this)
 {
+#ifndef USE_CONSOLE
 	C4Object *obj;
-	if (obj = Object(_this))
+	if ((obj = Object(_this)))
 	{
 		if (obj->BackParticles)
 			obj->BackParticles->Clear();
@@ -1579,7 +1730,7 @@ static bool FnClearParticles(C4PropList * _this)
 		if (::Particles.GetGlobalParticles())
 			::Particles.GetGlobalParticles()->Clear();
 	}
-
+#endif
 	// always return true
 	return true;
 }
@@ -1611,13 +1762,14 @@ static C4ValueArray* FnPV_Direction(C4PropList * _this, C4Value factor)
 	return pArray;
 }
 
-static C4ValueArray* FnPV_Step(C4PropList * _this, C4Value step, C4Value startValue, C4Value delay)
+static C4ValueArray* FnPV_Step(C4PropList * _this, C4Value step, C4Value startValue, C4Value delay, C4Value maximumValue)
 {
-	C4ValueArray *pArray = new C4ValueArray(4);
+	C4ValueArray *pArray = new C4ValueArray(5);
 	pArray->SetItem(0, C4VInt(C4PV_Step));
 	pArray->SetItem(1, step);
 	pArray->SetItem(2, startValue);
 	pArray->SetItem(3, delay);
+	pArray->SetItem(4, maximumValue);
 	return pArray;
 }
 
@@ -1641,6 +1793,16 @@ static C4Value FnPV_KeyFrames(C4PropList * _this, C4Value *pars)
 	return C4Value(pArray);
 }
 
+static C4ValueArray* FnPV_Sin(C4PropList * _this, C4Value value, C4Value amplitude, C4Value offset)
+{
+	C4ValueArray *pArray = new C4ValueArray(5);
+	pArray->SetItem(0, C4VInt(C4PV_Sin));
+	pArray->SetItem(1, value);
+	pArray->SetItem(2, amplitude);
+	pArray->SetItem(3, offset);
+	return pArray;
+}
+
 static C4ValueArray* FnPV_Speed(C4PropList * _this, C4Value factor, C4Value startValue)
 {
 	C4ValueArray *pArray = new C4ValueArray(3);
@@ -1971,15 +2133,14 @@ static C4Value FnEffectCall(C4PropList * _this, C4Value * Pars)
 
 static bool FnSetViewOffset(C4PropList * _this, long iPlayer, long iX, long iY)
 {
-	if (!ValidPlr(iPlayer)) return 0;
+	if (!ValidPlr(iPlayer)) return false;
 	// get player viewport
 	C4Viewport *pView = ::Viewports.GetViewport(iPlayer);
 	if (!pView) return 1; // sync safety
 	// set
-	pView->ViewOffsX = iX;
-	pView->ViewOffsY = iY;
+	pView->SetViewOffset(iX, iY);
 	// ok
-	return 1;
+	return true;
 }
 
 // undocumented!
@@ -2180,7 +2341,7 @@ static bool FnCustomMessage(C4PropList * _this, C4String *pMsg, C4Object *pObj,
 {
 	// safeties
 	if (pSrc)
-		if(!pSrc->GetDef() && !pSrc->GetObject()) return false;
+		if(!pSrc->GetDef() && !pSrc->GetObject() && !pSrc->GetPropertyPropList(P_Source)) return false;
 	if (!pMsg) return false;
 	if (pObj && !pObj->Status) return false;
 	const char *szMsg = pMsg->GetCStr();
@@ -2222,7 +2383,7 @@ static bool FnCustomMessage(C4PropList * _this, C4String *pMsg, C4Object *pObj,
 
 /*static long FnSetSaturation(C4AulContext *ctx, long s)
   {
-  return pDraw->SetSaturation(BoundBy(s,0l,255l));
+  return pDraw->SetSaturation(Clamp(s,0l,255l));
   }*/
 
 // undocumented!
@@ -2365,6 +2526,48 @@ static int32_t FnGetStartupPlayerCount(C4PropList * _this)
 	return ::Game.StartupPlayerCount;
 }
 
+static bool FnGainScenarioAchievement(C4PropList * _this, C4String *achievement_name, Nillable<long> avalue, Nillable<long> player, C4String *for_scenario)
+{
+	// safety
+	if (!achievement_name || !achievement_name->GetData().getLength()) return false;
+	// default parameter
+	long value = avalue.IsNil() ? 1 : (long)avalue;
+	// gain achievement
+	bool result = true;
+	if (!player.IsNil() && player != NO_OWNER)
+	{
+		C4Player *plr = ::Players.Get(player);
+		if (!plr) return false;
+		result = plr->GainScenarioAchievement(achievement_name->GetCStr(), value, for_scenario ? for_scenario->GetCStr() : NULL);
+	}
+	else
+	{
+		for (C4Player *plr = ::Players.First; plr; plr = plr->Next)
+			if (!plr->GainScenarioAchievement(achievement_name->GetCStr(), value, for_scenario ? for_scenario->GetCStr() : NULL))
+				result = false;
+	}
+	return true;
+}
+
+static long FnGetPXSCount(C4PropList * _this, Nillable<long> iMaterial, Nillable<long> iX0, Nillable<long> iY0, Nillable<long> iWdt, Nillable<long> iHgt)
+{
+	if (iX0.IsNil())
+	{
+		// Search everywhere
+		// All materials everywhere
+		if (iMaterial.IsNil() || iMaterial == MNone) return ::PXS.GetCount();
+		// Specific material everywhere
+		return ::PXS.GetCount(iMaterial);
+	}
+	else
+	{
+		// Material in area; offset by caller
+		int32_t x = iX0, y = iY0;
+		if (Object(_this)) { x += Object(_this)->GetX(); y += Object(_this)->GetY(); }
+		return ::PXS.GetCount(iMaterial.IsNil() ? MNone : static_cast<int32_t>(iMaterial), x, y, iWdt, iHgt);
+	}
+}
+
 extern C4ScriptConstDef C4ScriptGameConstMap[];
 extern C4ScriptFnDef C4ScriptGameFnMap[];
 
@@ -2390,6 +2593,7 @@ void InitGameFunctionMap(C4AulScriptEngine *pEngine)
 	AddFunc(pEngine, "GetPlayerColor", FnGetPlayerColor);
 	AddFunc(pEngine, "GetPlrClonkSkin", FnGetPlrClonkSkin);
 	AddFunc(pEngine, "CreateObject", FnCreateObject);
+	AddFunc(pEngine, "CreateObjectAbove", FnCreateObjectAbove);
 	AddFunc(pEngine, "CreateConstruction", FnCreateConstruction);
 	AddFunc(pEngine, "FindConstructionSite", FnFindConstructionSite);
 	AddFunc(pEngine, "CheckConstructionSite", FnCheckConstructionSite);
@@ -2428,7 +2632,10 @@ void InitGameFunctionMap(C4AulScriptEngine *pEngine)
 	AddFunc(pEngine, "GetPlayerByIndex", FnGetPlayerByIndex);
 	AddFunc(pEngine, "EliminatePlayer", FnEliminatePlayer);
 	AddFunc(pEngine, "SurrenderPlayer", FnSurrenderPlayer);
+	AddFunc(pEngine, "FnGetLeagueScore", FnGetLeagueScore);
 	AddFunc(pEngine, "SetLeaguePerformance", FnSetLeaguePerformance);
+	AddFunc(pEngine, "SetLeagueProgressData", FnSetLeagueProgressData);
+	AddFunc(pEngine, "GetLeagueProgressData", FnGetLeagueProgressData);
 	AddFunc(pEngine, "CreateScriptPlayer", FnCreateScriptPlayer);
 	AddFunc(pEngine, "GetCursor", FnGetCursor);
 	AddFunc(pEngine, "GetViewCursor", FnGetViewCursor);
@@ -2448,14 +2655,18 @@ void InitGameFunctionMap(C4AulScriptEngine *pEngine)
 	AddFunc(pEngine, "CanInsertMaterial", FnCanInsertMaterial);
 	AddFunc(pEngine, "LandscapeWidth", FnLandscapeWidth);
 	AddFunc(pEngine, "LandscapeHeight", FnLandscapeHeight);
+	AddFunc(pEngine, "SetAmbientBrightness", FnSetAmbientBrightness);
+	AddFunc(pEngine, "GetAmbientBrightness", FnGetAmbientBrightness);
 	AddFunc(pEngine, "SetSeason", FnSetSeason);
 	AddFunc(pEngine, "GetSeason", FnGetSeason);
 	AddFunc(pEngine, "SetClimate", FnSetClimate);
 	AddFunc(pEngine, "GetClimate", FnGetClimate);
 	AddFunc(pEngine, "SetPlayerZoomByViewRange", FnSetPlayerZoomByViewRange);
+	AddFunc(pEngine, "GetPlayerZoomLimits", FnGetPlayerZoomLimits);
+	AddFunc(pEngine, "SetPlayerZoom", FnSetPlayerZoom);
 	AddFunc(pEngine, "SetPlayerViewLock", FnSetPlayerViewLock);
-	AddFunc(pEngine, "DoHomebaseMaterial", FnDoHomebaseMaterial);
-	AddFunc(pEngine, "DoHomebaseProduction", FnDoHomebaseProduction);
+	AddFunc(pEngine, "DoBaseMaterial", FnDoBaseMaterial);
+	AddFunc(pEngine, "DoBaseProduction", FnDoBaseProduction);
 	AddFunc(pEngine, "GainMissionAccess", FnGainMissionAccess);
 	AddFunc(pEngine, "IsNetwork", FnIsNetwork);
 	AddFunc(pEngine, "GetLeague", FnGetLeague);
@@ -2526,11 +2737,13 @@ void InitGameFunctionMap(C4AulScriptEngine *pEngine)
 	AddFunc(pEngine, "GetStartupPlayerCount", FnGetStartupPlayerCount);
 	AddFunc(pEngine, "PlayerObjectCommand", FnPlayerObjectCommand);
 	AddFunc(pEngine, "EditCursor", FnEditCursor);
+	AddFunc(pEngine, "GainScenarioAchievement", FnGainScenarioAchievement);
+	AddFunc(pEngine, "GetPXSCount", FnGetPXSCount);
 
 	F(GetPlrKnowledge);
 	F(GetComponent);
-	F(GetHomebaseMaterial);
-	F(GetHomebaseProduction);
+	F(GetBaseMaterial);
+	F(GetBaseProduction);
 	F(GetDefCoreVal);
 	F(GetObjectVal);
 	F(GetObjectInfoCoreVal);
@@ -2551,6 +2764,7 @@ void InitGameFunctionMap(C4AulScriptEngine *pEngine)
 	F(PV_Wind);
 	F(PV_Gravity);
 	// F(PV_KeyFrames); added below
+	F(PV_Sin);
 	F(PC_Die);
 	F(PC_Bounce);
 	F(PC_Stop);
@@ -2578,6 +2792,7 @@ C4ScriptConstDef C4ScriptGameConstMap[]=
 	{ "FX_Call_DmgScript"         ,C4V_Int,      C4FxCall_DmgScript         }, // damage through script call
 	{ "FX_Call_DmgBlast"          ,C4V_Int,      C4FxCall_DmgBlast          }, // damage through blast
 	{ "FX_Call_DmgFire"           ,C4V_Int,      C4FxCall_DmgFire           }, // damage through fire
+	{ "FX_Call_DmgChop"           ,C4V_Int,      C4FxCall_DmgChop           }, // damage through chopping
 	{ "FX_Call_Energy"            ,C4V_Int,      32                         }, // bitmask for generic energy loss
 	{ "FX_Call_EngScript"         ,C4V_Int,      C4FxCall_EngScript         }, // energy loss through script call
 	{ "FX_Call_EngBlast"          ,C4V_Int,      C4FxCall_EngBlast          }, // energy loss through blast
@@ -2630,6 +2845,7 @@ C4ScriptConstDef C4ScriptGameConstMap[]=
 	{ "C4FO_Controller"           ,C4V_Int,     C4FO_Controller     },
 	{ "C4FO_Func"                 ,C4V_Int,     C4FO_Func           },
 	{ "C4FO_Layer"                ,C4V_Int,     C4FO_Layer          },
+	{ "C4FO_InArray"              ,C4V_Int,     C4FO_InArray        },
 
 	{ "MD_DragSource"             ,C4V_Int,     C4MC_MD_DragSource  },
 	{ "MD_DropTarget"             ,C4V_Int,     C4MC_MD_DropTarget  },
@@ -2680,6 +2896,7 @@ C4ScriptConstDef C4ScriptGameConstMap[]=
 	{ "PLRZOOM_NoDecrease"        ,C4V_Int,      PLRZOOM_NoDecrease },
 	{ "PLRZOOM_LimitMin"          ,C4V_Int,      PLRZOOM_LimitMin },
 	{ "PLRZOOM_LimitMax"          ,C4V_Int,      PLRZOOM_LimitMax },
+	{ "PLRZOOM_Set"               ,C4V_Int,      PLRZOOM_Set },
 
 	{ "ATTACH_Front"              ,C4V_Int,      C4ATTACH_Front },
 	{ "ATTACH_Back"               ,C4V_Int,      C4ATTACH_Back },
