@@ -28,9 +28,16 @@
 #include "C4Rect.h"
 #include <C4Config.h>
 #include "StdMesh.h"
+#include <CSurface8.h>
 
 #include <stdio.h>
-#include <limits.h>
+
+// Instruct Optimus laptops to use nVidia GPU instead of integrated GPU
+#if defined(_WIN32) && !defined(USE_CONSOLE)
+extern "C" {
+	__declspec(dllexport) DWORD NvOptimusEnablement = 0x00000001;
+}
+#endif
 
 // Global access pointer
 C4Draw *pDraw=NULL;
@@ -182,9 +189,9 @@ void C4GammaControl::SetClrChannel(WORD *pBuf, BYTE c1, BYTE c2, int c3)
 	{
 		int i2=128-i;
 		// interpolate linear ramps with the rises r1 and r
-		*pBuf ++=BoundBy(((c1+r1*i/128) *i2  +  (c2-r*i2/128) *i) <<1, MinGamma, 0xffff);
+		*pBuf ++=Clamp(((c1+r1*i/128) *i2  +  (c2-r*i2/128) *i) <<1, MinGamma, 0xffff);
 		// interpolate linear ramps with the rises r and r2
-		*pBuf2++=BoundBy(((c2+r*i/128) *i2  +  (c3-r2*i2/128) *i) <<1, MinGamma, 0xffff);
+		*pBuf2++=Clamp(((c2+r*i/128) *i2  +  (c3-r2*i2/128) *i) <<1, MinGamma, 0xffff);
 	}
 }
 
@@ -205,178 +212,8 @@ DWORD C4GammaControl::ApplyTo(DWORD dwClr)
 
 //--------------------------------------------------------------------
 
-C4FogOfWar::~C4FogOfWar()
-{
-	delete[]pMap; delete pSurface;
-}
-
-void C4FogOfWar::Reset(int ResX, int ResY, int WdtPx, int HgtPx, int OffX, int OffY, unsigned char StartVis, int x0, int y0, uint32_t dwBackClr, class C4Surface *backsfc)
-{
-	// set values
-	ResolutionX = ResX; ResolutionY = ResY;
-	this->dwBackClr = dwBackClr;
-	this->OffX = -((OffX) % ResolutionX);
-	this->OffY = -((OffY) % ResolutionY);
-	// calc w/h required for map
-	Wdt = (WdtPx - this->OffX + ResolutionX-1) / ResolutionX + 1;
-	Hgt = (HgtPx - this->OffY + ResolutionY-1) / ResolutionY + 1;
-	this->OffX += x0;
-	this->OffY += y0;
-	size_t NewMapSize = Wdt * Hgt;
-	if (NewMapSize > MapSize || NewMapSize < MapSize / 2)
-	{
-		delete [] pMap;
-		pMap = new unsigned char[MapSize = NewMapSize];
-	}
-	if (!pSurface || pSurface->Wdt<Wdt || pSurface->Hgt<Hgt)
-	{
-		delete pSurface;
-		pSurface = new C4Surface(Max(Wdt, Hgt), Max(Wdt, Hgt)); // force larger texture size by making it squared!
-	}
-	// is a background color desired?
-	if (dwBackClr && backsfc)
-	{
-		// then draw a background now and fade against transparent later
-		// FIXME: don't do this if shaders are used
-		pDraw->DrawBoxDw(backsfc, x0,y0, x0+WdtPx-1, y0+HgtPx-1, dwBackClr);
-		FadeTransparent = true;
-	}
-	else
-		FadeTransparent = false;
-	// reset all of map to given values
-	memset(pMap, StartVis, MapSize);
-	pSurface->Lock();
-	pSurface->ClearBoxDw(0, 0, Wdt, Hgt);
-}
-
-C4Surface *C4FogOfWar::GetSurface()
-{
-	if (pSurface->IsLocked())
-	{
-		for (int x = 0; x < Wdt; ++x)
-			for (int y = 0; y < Hgt; ++y)
-				pSurface->SetPixDw(x, y, pMap[y * Wdt + x] << 24 | (dwBackClr & 0xFFFFFF));
-		//pSurface->SetPixDw(x, y, 0x7f000000 + (((0xff*x)/iWdt) << 24) + (((0xff*y)/iHgt) << 16));
-		pSurface->Unlock();
-	}
-	return pSurface;
-}
-
-void C4FogOfWar::ReduceModulation(int cx, int cy, int Radius, int (*VisProc)(int, int, int, int, int))
-{
-	// landscape coordinates: cx, cy, VisProc
-	// display coordinates: zx, zy, x, y
-	float zx = float(cx);
-	float zy = float(cy);
-	pDraw->ApplyZoom(zx, zy);
-	Radius = int(pDraw->Zoom * Radius);
-	// reveal all within iRadius1; fade off squared until iRadius2
-	int x = OffX, y = OffY, xe = Wdt*ResolutionX+OffX;
-	int RadiusSq = Radius*Radius;
-	for (unsigned int i = 0; i < MapSize; i++)
-	{
-		if ((x-zx)*(x-zx)+(y-zy)*(y-zy) < RadiusSq)
-		{
-			float lx = float(x);
-			float ly = float(y);
-			pDraw->RemoveZoom(lx, ly);
-			pMap[i] = Max<int>(pMap[i], VisProc(255, int(lx), int(ly), int(cx), int(cy)));
-		}
-		// next pos
-		x += ResolutionX;
-		if (x >= xe) { x = OffX; y += ResolutionY; }
-	}
-}
-
-void C4FogOfWar::AddModulation(int cx, int cy, int Radius, uint8_t Transparency)
-{
-	{
-		float x=float(cx); float y=float(cy);
-		pDraw->ApplyZoom(x,y);
-		cx=int(x); cy=int(y);
-	}
-	Radius = int(pDraw->Zoom * Radius);
-	// hide all within iRadius1; fade off squared until iRadius2
-	int x = OffX, y = OffY, xe = Wdt*ResolutionX+OffX;
-	int RadiusSq = Radius*Radius;
-	for (unsigned int i = 0; i < MapSize; i++)
-	{
-		int d = (x-cx)*(x-cx)+(y-cy)*(y-cy);
-		if (d < RadiusSq)
-			pMap[i] = Min<uint8_t>(Transparency, pMap[i]);
-		// next pos
-		x += ResolutionX;
-		if (x >= xe) { x = OffX; y += ResolutionY; }
-	}
-}
-
-uint32_t C4FogOfWar::GetModAt(int x, int y) const
-{
-#if 0
-	// fast but inaccurate method
-	x = BoundBy((x - iOffX + iResolutionX/2) / iResolutionX, 0, iWdt-1);
-	y = BoundBy((y - iOffY + iResolutionY/2) / iResolutionY, 0, iHgt-1);
-	return pMap[y * iWdt + x]->dwModClr;
-#else
-	// slower, more accurate method: Interpolate between 4 neighboured modulations
-	x -= OffX;
-	y -= OffY;
-	int tx = BoundBy(x / ResolutionX, 0, Wdt-1);
-	int ty = BoundBy(y / ResolutionY, 0, Hgt-1);
-	int tx2 = Min(tx + 1, Wdt-1);
-	int ty2 = Min(ty + 1, Hgt-1);
-
-	// TODO: Alphafixed. Correct?
-	unsigned char Vis = pMap[ty*Wdt+tx];
-	uint32_t c1 = FadeTransparent ? 0xffffff | (Vis << 24) : C4RGB(Vis, Vis, Vis);
-	Vis = pMap[ty*Wdt+tx2];
-	uint32_t c2 = FadeTransparent ? 0xffffff | (Vis << 24) : C4RGB(Vis, Vis, Vis);
-	Vis = pMap[ty2*Wdt+tx];
-	uint32_t c3 = FadeTransparent ? 0xffffff | (Vis << 24) : C4RGB(Vis, Vis, Vis);
-	Vis = pMap[ty2*Wdt+tx2];
-	uint32_t c4 = FadeTransparent ? 0xffffff | (Vis << 24) : C4RGB(Vis, Vis, Vis);
-	CColorFadeMatrix clrs(tx*ResolutionX, ty*ResolutionY, ResolutionX, ResolutionY, c1, c2, c3, c4);
-	return clrs.GetColorAt(x, y);
-#endif
-}
-
-// -------------------------------------------------------------------
-
-CColorFadeMatrix::CColorFadeMatrix(int iX, int iY, int iWdt, int iHgt, uint32_t dwClr1, uint32_t dwClr2, uint32_t dwClr3, uint32_t dwClr4)
-		: ox(iX), oy(iY), w(iWdt), h(iHgt)
-{
-	uint32_t dwMask = 0xff;
-	for (int iChan = 0; iChan < 4; (++iChan),(dwMask<<=8))
-	{
-		int c0 = (dwClr1 & dwMask) >> (iChan*8);
-		int cx = (dwClr2 & dwMask) >> (iChan*8);
-		int cy = (dwClr3 & dwMask) >> (iChan*8);
-		int ce = (dwClr4 & dwMask) >> (iChan*8);
-		clrs[iChan].c0 = c0;
-		clrs[iChan].cx = cx - c0;
-		clrs[iChan].cy = cy - c0;
-		clrs[iChan].ce = ce;
-	}
-}
-
-uint32_t CColorFadeMatrix::GetColorAt(int iX, int iY)
-{
-	iX -= ox; iY -= oy;
-	uint32_t dwResult = 0x00;
-	for (int iChan = 0; iChan < 4; ++iChan)
-	{
-		int clr = clrs[iChan].c0 + clrs[iChan].cx * iX / w + clrs[iChan].cy * iY / h;
-		clr += iX*iY * (clrs[iChan].ce - clr) / (w*h);
-		dwResult |= (BoundBy(clr, 0, 255) << (iChan*8));
-	}
-	return dwResult;
-}
-
-// -------------------------------------------------------------------
-
 void C4Draw::Default()
 {
-	Editor=true;
 	RenderTarget=NULL;
 	ClipAll=false;
 	Active=false;
@@ -384,8 +221,7 @@ void C4Draw::Default()
 	dwBlitMode = 0;
 	Gamma.Default();
 	DefRamp.Default();
-	// pClrModMap = NULL; - invalid if !fUseClrModMap anyway
-	fUseClrModMap = false;
+	pFoW = NULL;
 	ZoomX = 0; ZoomY = 0; Zoom = 1;
 	MeshTransform = NULL;
 	fUsePerspective = false;
@@ -398,7 +234,7 @@ void C4Draw::Clear()
 {
 	ResetGamma();
 	DisableGamma();
-	Active=BlitModulated=fUseClrModMap=false;
+	Active=BlitModulated=false;
 	dwBlitMode = 0;
 }
 
@@ -459,7 +295,7 @@ bool C4Draw::NoPrimaryClipper()
 }
 
 void C4Draw::BlitLandscape(C4Surface * sfcSource, float fx, float fy,
-                              C4Surface * sfcTarget, float tx, float ty, float wdt, float hgt, const C4Surface * textures[])
+                              C4Surface * sfcTarget, float tx, float ty, float wdt, float hgt)
 {
 	Blit(sfcSource, fx, fy, wdt, hgt, sfcTarget, tx, ty, wdt, hgt, false);
 }
@@ -482,13 +318,34 @@ void C4Draw::Blit8Fast(CSurface8 * sfcSource, int fx, int fy,
 		thgt *= Zoom;
 	}
 
-	// blit 8 bit pix
+	// blit 8 bit pix in batches of 1024 pixels
+	static const int BUF_SIZE = 1024;
+	C4BltVertex* vertices = new C4BltVertex[BUF_SIZE];
+	int bufcnt = 0;
+
 	for (int ycnt=0; ycnt<thgt; ++ycnt)
+	{
 		for (int xcnt=0; xcnt<twdt; ++xcnt)
 		{
 			BYTE byPix = sfcSource->GetPix(fx+wdt*xcnt/twdt, fy+hgt*ycnt/thgt);
-			if (byPix) PerformPix(sfcTarget,(float)(tfx+xcnt), (float)(tfy+ycnt), sfcSource->pPal->GetClr(byPix));
+			DWORD dwClr = byPix ? sfcSource->pPal->GetClr(byPix) : 0x00000000;
+
+			vertices[bufcnt].ftx = (float)(tx + xcnt / Zoom);
+			vertices[bufcnt].fty = (float)(ty + ycnt / Zoom);
+			DwTo4UB(dwClr, vertices[bufcnt].color);
+			++bufcnt;
+
+			if(bufcnt == BUF_SIZE)
+			{
+				PerformMultiPix(sfcTarget, vertices, BUF_SIZE);
+				bufcnt = 0;
+			}
 		}
+
+	}
+	if(bufcnt > 0)
+		PerformMultiPix(sfcTarget, vertices, bufcnt);
+	delete[] vertices;
 	// unlock
 	if (!fRender) sfcTarget->Unlock();
 }
@@ -507,77 +364,38 @@ bool C4Draw::BlitUnscaled(C4Surface * sfcSource, float fx, float fy, float fwdt,
 {
 	// safety
 	if (!sfcSource || !sfcTarget || !twdt || !thgt || !fwdt || !fhgt) return false;
-	// Apply Zoom
-	C4BltTransform t;
-	if (pTransform && Zoom != 1.0)
-	{
-		//tx = tx * Zoom - ZoomX * Zoom + ZoomX;
-		//ty = ty * Zoom - ZoomY * Zoom + ZoomY;
-		// The transformation is not location-independant, thus has to be zoomed, too.
-		t.Set(pTransform->mat[0]*Zoom, pTransform->mat[1]*Zoom, pTransform->mat[2]*Zoom + ZoomX*(1-Zoom),
-		      pTransform->mat[3]*Zoom, pTransform->mat[4]*Zoom, pTransform->mat[5]*Zoom + ZoomY*(1-Zoom),
-		      pTransform->mat[6], pTransform->mat[7], pTransform->mat[8]);
-		pTransform = &t;
-	}
-	else if (Zoom != 1.0)
-	{
-		ApplyZoom(tx, ty);
-		twdt *= Zoom;
-		thgt *= Zoom;
-	}
 	// emulated blit?
 	if (!sfcTarget->IsRenderTarget())
+	{
+		C4BltTransform t;
+		if(pTransform && Zoom != 1.0)
+		{
+			t.Set(pTransform->mat[0]*Zoom, pTransform->mat[1]*Zoom, pTransform->mat[2]*Zoom + ZoomX*(1-Zoom),
+			      pTransform->mat[3]*Zoom, pTransform->mat[4]*Zoom, pTransform->mat[5]*Zoom + ZoomY*(1-Zoom),
+			      pTransform->mat[6], pTransform->mat[7], pTransform->mat[8]);
+			pTransform = &t;
+		}
+		else if(Zoom != 1.0)
+		{
+			ApplyZoom(tx, ty);
+			twdt *= Zoom;
+			thgt *= Zoom;
+		}
+
 		return Blit8(sfcSource, int(fx), int(fy), int(fwdt), int(fhgt), sfcTarget, int(tx), int(ty), int(twdt), int(thgt), fSrcColKey, pTransform);
+	}
+
 	// calc stretch
 	float scaleX = twdt/fwdt;
 	float scaleY = thgt/fhgt;
 	// bound
 	if (ClipAll) return true;
-	// check exact
-	bool fExact = !pTransform && fwdt==twdt && fhgt==thgt;
-	// manual clipping? (primary surface only)
-	if (Config.Graphics.ClipManuallyE && !pTransform && sfcTarget->fPrimary)
-	{
-		float iOver;
-		// Left
-		iOver=tx-iClipX1;
-		if (iOver<0)
-		{
-			twdt+=iOver;
-			fwdt+=iOver/scaleX;
-			fx-=iOver/scaleX;
-			tx=float(iClipX1);
-		}
-		// Top
-		iOver=ty-iClipY1;
-		if (iOver<0)
-		{
-			thgt+=iOver;
-			fhgt+=iOver/scaleY;
-			fy-=iOver/scaleY;
-			ty=float(iClipY1);
-		}
-		// Right
-		iOver=iClipX2+1-(tx+twdt);
-		if (iOver<0)
-		{
-			fwdt+=iOver/scaleX;
-			twdt+=iOver;
-		}
-		// Bottom
-		iOver=iClipY2+1-(ty+thgt);
-		if (iOver<0)
-		{
-			fhgt+=iOver/scaleY;
-			thgt+=iOver;
-		}
-	}
 	// inside screen?
 	if (twdt<=0 || thgt<=0) return false;
 	// prepare rendering to surface
 	if (!PrepareRendering(sfcTarget)) return false;
 	// texture present?
-	if (!sfcSource->ppTex)
+	if (sfcSource->textures.empty())
 	{
 		// primary surface?
 		if (sfcSource->fPrimary)
@@ -587,14 +405,9 @@ bool C4Draw::BlitUnscaled(C4Surface * sfcSource, float fx, float fy, float fwdt,
 		}
 		return false;
 	}
-	// create blitting struct
-	C4BltData BltData;
-	// pass down pTransform
-	BltData.pTransform=pTransform;
 	// blit with basesfc?
 	bool fBaseSfc=false;
-	if (sfcSource->pMainSfc) if (sfcSource->pMainSfc->ppTex) fBaseSfc=true;
-	// set blitting state - done by PerformBlt
+	if (sfcSource->pMainSfc) if (!sfcSource->pMainSfc->textures.empty()) fBaseSfc = true;
 	// get involved texture offsets
 	int iTexSizeX=sfcSource->iTexSize;
 	int iTexSizeY=sfcSource->iTexSize;
@@ -605,14 +418,12 @@ bool C4Draw::BlitUnscaled(C4Surface * sfcSource, float fx, float fy, float fwdt,
 	// calc stretch regarding texture size and indent
 /*	float scaleX2 = scaleX * iTexSizeX;
 	float scaleY2 = scaleY * iTexSizeY;*/
-	// Enable textures
-	SetTexture();
 	// blit from all these textures
 	for (int iY=iTexY; iY<iTexY2; ++iY)
 	{
 		for (int iX=iTexX; iX<iTexX2; ++iX)
 		{
-			C4TexRef *pTex = *(sfcSource->ppTex + iY * sfcSource->iTexX + iX);
+			C4TexRef *pTex = &sfcSource->textures[iY * sfcSource->iTexX + iX];
 			// get current blitting offset in texture
 			int iBlitX=sfcSource->iTexSize*iX;
 			int iBlitY=sfcSource->iTexSize*iY;
@@ -640,52 +451,24 @@ bool C4Draw::BlitUnscaled(C4Surface * sfcSource, float fx, float fy, float fwdt,
 			tTexBlt.top   = (fTexBlt.top   + iBlitY - fy) * scaleY + ty;
 			tTexBlt.right = (fTexBlt.right + iBlitX - fx) * scaleX + tx;
 			tTexBlt.bottom= (fTexBlt.bottom+ iBlitY - fy) * scaleY + ty;
-			// prepare blit data texture matrix
-			// translate back to texture 0/0 regarding indent and blit offset
-			/*BltData.TexPos.SetMoveScale(-tTexBlt.left, -tTexBlt.top, 1, 1);
-			// apply back scaling and texture-indent - simply scale matrix down
-			int i;
-			for (i=0; i<3; ++i) BltData.TexPos.mat[i] /= scaleX2;
-			for (i=3; i<6; ++i) BltData.TexPos.mat[i] /= scaleY2;
-			// now, finally, move in texture - this must be done last, so no stupid zoom is applied...
-			BltData.TexPos.MoveScale(((float) fTexBlt.left) / iTexSize,
-			  ((float) fTexBlt.top) / iTexSize, 1, 1);*/
-			// Set resulting matrix directly
-			/*BltData.TexPos.SetMoveScale(
-			  fTexBlt.left / iTexSizeX - tTexBlt.left / scaleX2,
-			  fTexBlt.top / iTexSizeY - tTexBlt.top / scaleY2,
-			  1 / scaleX2,
-			  1 / scaleY2);*/
-
-			// get tex bounds
-			// The code below is commented out since the problem
-			// in question is currently fixed by using non-power-of-two
-			// and non-square textures.
-#if 0
-			// Size of this texture actually containing image data
-			const int iImgSizeX = (iX == sfcSource->iTexX-1) ? ((sfcSource->Wdt - 1) % iTexSizeX + 1) : (iTexSizeX);
-			const int iImgSizeY = (iY == sfcSource->iTexY-1) ? ((sfcSource->Hgt - 1) % iTexSizeY + 1) : (iTexSizeY);			
-			// Make sure we don't access border pixels. Normally this is prevented
-			// by GL_CLAMP_TO_EDGE anyway but for the bottom and rightmost textures
-			// this does not work as the textures might only be partially filled.
-			// This is the case if iImgSizeX != iTexSize or iImgSizeY != iTexSize.
-			// See bug #396.
-			fTexBlt.left  = Max<float>(fTexBlt.left, 0.5);
-			fTexBlt.top   = Max<float>(fTexBlt.top, 0.5);
-			fTexBlt.right = Min<float>(fTexBlt.right, iImgSizeX - 0.5);
-			fTexBlt.bottom= Min<float>(fTexBlt.bottom, iImgSizeY - 0.5);
-#endif
 
 			// set up blit data as rect
-			BltData.byNumVertices = 4;
-			BltData.vtVtx[0].ftx = tTexBlt.left;  BltData.vtVtx[0].fty = tTexBlt.top;
-			BltData.vtVtx[1].ftx = tTexBlt.right; BltData.vtVtx[1].fty = tTexBlt.top;
-			BltData.vtVtx[2].ftx = tTexBlt.right; BltData.vtVtx[2].fty = tTexBlt.bottom;
-			BltData.vtVtx[3].ftx = tTexBlt.left;  BltData.vtVtx[3].fty = tTexBlt.bottom;
-			BltData.vtVtx[0].tx = fTexBlt.left / iTexSizeX; BltData.vtVtx[0].ty = fTexBlt.top / iTexSizeY;
-			BltData.vtVtx[1].tx = fTexBlt.right / iTexSizeX; BltData.vtVtx[1].ty = fTexBlt.top / iTexSizeY;
-			BltData.vtVtx[2].tx = fTexBlt.right / iTexSizeX; BltData.vtVtx[2].ty = fTexBlt.bottom / iTexSizeY;
-			BltData.vtVtx[3].tx = fTexBlt.left / iTexSizeX; BltData.vtVtx[3].ty = fTexBlt.bottom / iTexSizeY;
+			C4BltVertex vertices[6];
+			vertices[0].ftx = tTexBlt.left;  vertices[0].fty = tTexBlt.top;
+			vertices[1].ftx = tTexBlt.right; vertices[1].fty = tTexBlt.top;
+			vertices[2].ftx = tTexBlt.right; vertices[2].fty = tTexBlt.bottom;
+			vertices[3].ftx = tTexBlt.left;  vertices[3].fty = tTexBlt.bottom;
+			vertices[0].tx = fTexBlt.left / iTexSizeX;  vertices[0].ty = fTexBlt.top / iTexSizeY;
+			vertices[1].tx = fTexBlt.right / iTexSizeX; vertices[1].ty = fTexBlt.top / iTexSizeY;
+			vertices[2].tx = fTexBlt.right / iTexSizeX; vertices[2].ty = fTexBlt.bottom / iTexSizeY;
+			vertices[3].tx = fTexBlt.left / iTexSizeX;  vertices[3].ty = fTexBlt.bottom / iTexSizeY;
+			DwTo4UB(0xffffffff, vertices[0].color);
+			DwTo4UB(0xffffffff, vertices[1].color);
+			DwTo4UB(0xffffffff, vertices[2].color);
+			DwTo4UB(0xffffffff, vertices[3].color);
+
+			// duplicate vertices
+			vertices[4] = vertices[0]; vertices[5] = vertices[2];
 
 			C4TexRef * pBaseTex = pTex;
 			// is there a base-surface to be blitted first?
@@ -694,23 +477,18 @@ bool C4Draw::BlitUnscaled(C4Surface * sfcSource, float fx, float fy, float fwdt,
 				// then get this surface as same offset as from other surface
 				// assuming this is only valid as long as there's no texture management,
 				// organizing partially used textures together!
-				pBaseTex = *(sfcSource->pMainSfc->ppTex + iY * sfcSource->iTexX + iX);
-			}
-			// base blit
-			PerformBlt(BltData, pBaseTex, BlitModulated ? BlitModulateClr : 0xffffffff, !!(dwBlitMode & C4GFXBLIT_MOD2), fExact);
-			// overlay
-			if (fBaseSfc)
-			{
-				DWORD dwModClr = sfcSource->ClrByOwnerClr;
-				// apply global modulation to overlay surfaces only if desired
-				if (BlitModulated && !(dwBlitMode & C4GFXBLIT_CLRSFC_OWNCLR))
-					ModulateClr(dwModClr, BlitModulateClr);
-				PerformBlt(BltData, pTex, dwModClr, !!(dwBlitMode & C4GFXBLIT_CLRSFC_MOD2), fExact);
+				pBaseTex = &sfcSource->pMainSfc->textures[iY * sfcSource->iTexX + iX];
 			}
+
+			C4TexRef* pNormalTex = NULL;
+			if (sfcSource->pNormalSfc)
+				pNormalTex = &sfcSource->pNormalSfc->textures[iY * sfcSource->iTexX + iX];
+
+			// ClrByOwner is always fully opaque
+			const DWORD dwOverlayClrMod = 0xff000000 | sfcSource->ClrByOwnerClr;
+			PerformMultiTris(sfcTarget, vertices, 6, pTransform, pBaseTex, fBaseSfc ? pTex : NULL, pNormalTex, dwOverlayClrMod);
 		}
 	}
-	// reset texture
-	ResetTexture();
 	// success
 	return true;
 }
@@ -847,67 +625,68 @@ bool C4Draw::BlitSurface(C4Surface * sfcSurface, C4Surface * sfcTarget, int tx,
 	}
 }
 
-bool C4Draw::BlitSurfaceTile(C4Surface * sfcSurface, C4Surface * sfcTarget, int iToX, int iToY, int iToWdt, int iToHgt, int iOffsetX, int iOffsetY, bool fSrcColKey)
+bool C4Draw::BlitSurfaceTile(C4Surface * sfcSurface, C4Surface * sfcTarget, float iToX, float iToY, float iToWdt, float iToHgt, float iOffsetX, float iOffsetY, bool fSrcColKey)
 {
-	int iSourceWdt,iSourceHgt,iX,iY,iBlitX,iBlitY,iBlitWdt,iBlitHgt;
+	int iSourceWdt,iSourceHgt;
+	float iX,iY,iBlitX,iBlitY,iBlitWdt,iBlitHgt;
 	// Get source surface size
 	if (!GetSurfaceSize(sfcSurface,iSourceWdt,iSourceHgt)) return false;
 	// reduce offset to needed size
-	iOffsetX %= iSourceWdt;
-	iOffsetY %= iSourceHgt;
+	iOffsetX = fmod(iOffsetX, iSourceWdt);
+	iOffsetY = fmod(iOffsetY, iSourceHgt);
 	// Vertical blits
 	for (iY=iToY+iOffsetY; iY<iToY+iToHgt; iY+=iSourceHgt)
 	{
 		// Vertical blit size
-		iBlitY=Max(iToY-iY,0); iBlitHgt=Min(iSourceHgt,iToY+iToHgt-iY)-iBlitY;
+		iBlitY=Max(iToY-iY,0.0f); iBlitHgt=Min<float>(iSourceHgt,iToY+iToHgt-iY)-iBlitY;
 		// Horizontal blits
 		for (iX=iToX+iOffsetX; iX<iToX+iToWdt; iX+=iSourceWdt)
 		{
 			// Horizontal blit size
-			iBlitX=Max(iToX-iX,0); iBlitWdt=Min(iSourceWdt,iToX+iToWdt-iX)-iBlitX;
+			iBlitX=Max(iToX-iX,0.0f); iBlitWdt=Min<float>(iSourceWdt,iToX+iToWdt-iX)-iBlitX;
 			// Blit
-			if (!Blit(sfcSurface,float(iBlitX),float(iBlitY),float(iBlitWdt),float(iBlitHgt),sfcTarget,float(iX+iBlitX),float(iY+iBlitY),float(iBlitWdt),float(iBlitHgt),fSrcColKey)) return false;
+			if (!Blit(sfcSurface, iBlitX, iBlitY, iBlitWdt, iBlitHgt, sfcTarget, iX+iBlitX, iY+iBlitY, iBlitWdt, iBlitHgt, fSrcColKey)) return false;
 		}
 	}
 	return true;
 }
 
-bool C4Draw::BlitSurfaceTile2(C4Surface * sfcSurface, C4Surface * sfcTarget, int iToX, int iToY, int iToWdt, int iToHgt, int iOffsetX, int iOffsetY, bool fSrcColKey)
+bool C4Draw::BlitSurfaceTile2(C4Surface * sfcSurface, C4Surface * sfcTarget, float iToX, float iToY, float iToWdt, float iToHgt, float iOffsetX, float iOffsetY, bool fSrcColKey)
 {
 	// if it's a render target, simply blit with repeating texture
 	// repeating textures, however, aren't currently supported
 	/*if (sfcTarget->IsRenderTarget())
 	  return Blit(sfcSurface, iOffsetX, iOffsetY, iToWdt, iToHgt, sfcTarget, iToX, iToY, iToWdt, iToHgt, false);*/
-	int tx,ty,iBlitX,iBlitY,iBlitWdt,iBlitHgt;
+	float tx,ty,iBlitX,iBlitY,iBlitWdt,iBlitHgt;
 	// get tile size
 	int iTileWdt=sfcSurface->Wdt;
 	int iTileHgt=sfcSurface->Hgt;
 	// adjust size of offsets
-	iOffsetX%=iTileWdt;
-	iOffsetY%=iTileHgt;
+	iOffsetX = fmod(iOffsetX, iTileWdt);
+	iOffsetY = fmod(iOffsetY, iTileHgt);
 	if (iOffsetX<0) iOffsetX+=iTileWdt;
 	if (iOffsetY<0) iOffsetY+=iTileHgt;
 	// get start pos for blitting
-	int iStartX=iToX-iOffsetX;
-	int iStartY=iToY-iOffsetY;
+	float iStartX=iToX-iOffsetX;
+	float iStartY=iToY-iOffsetY;
 	ty=0;
 	// blit vertical
-	for (int iY=iStartY; ty<iToHgt; iY+=iTileHgt)
+	for (float iY=iStartY; fabs(ty - iToHgt) > 1e-3; iY+=iTileHgt)
 	{
 		// get vertical blit bounds
 		iBlitY=0; iBlitHgt=iTileHgt;
 		if (iY<iToY) { iBlitY=iToY-iY; iBlitHgt+=iY-iToY; }
-		int iOver=ty+iBlitHgt-iToHgt; if (iOver>0) iBlitHgt-=iOver;
+		float iOver=ty+iBlitHgt-iToHgt; if (iOver>0) iBlitHgt-=iOver;
 		// blit horizontal
 		tx=0;
-		for (int iX=iStartX; tx<iToWdt; iX+=iTileWdt)
+		for (float iX=iStartX; fabs(tx - iToWdt) > 1e-3; iX+=iTileWdt)
 		{
 			// get horizontal blit bounds
 			iBlitX=0; iBlitWdt=iTileWdt;
 			if (iX<iToX) { iBlitX=iToX-iX; iBlitWdt+=iX-iToX; }
 			iOver=tx+iBlitWdt-iToWdt; if (iOver>0) iBlitWdt-=iOver;
 			// blit
-			if (!Blit(sfcSurface,float(iBlitX),float(iBlitY),float(iBlitWdt),float(iBlitHgt),sfcTarget,float(tx+iToX),float(ty+iToY),float(iBlitWdt),float(iBlitHgt),fSrcColKey))
+			if (!Blit(sfcSurface,iBlitX,iBlitY,iBlitWdt,iBlitHgt,sfcTarget,tx+iToX,ty+iToY,iBlitWdt,iBlitHgt,fSrcColKey))
 			{
 				// Ignore blit errors. This is usually due to blit border lying outside surface and shouldn't cause remaining blits to fail.
 			}
@@ -960,119 +739,82 @@ bool C4Draw::StringOut(const char *szText, C4Surface * sfcDest, float iTx, float
 
 void C4Draw::DrawPix(C4Surface * sfcDest, float tx, float ty, DWORD dwClr)
 {
-	ApplyZoom(tx, ty);
-	// FIXME: zoom to a box
-	// manual clipping?
-	if (Config.Graphics.ClipManuallyE)
-	{
-		if (tx < iClipX1) { return; }
-		if (ty < iClipY1) { return; }
-		if (iClipX2 < tx) { return; }
-		if (iClipY2 < ty) { return; }
-	}
-	// apply global modulation
-	ClrByCurrentBlitMod(dwClr);
-	// apply modulation map
-	if (fUseClrModMap)
-		ModulateClr(dwClr, pClrModMap->GetModAt((int)tx, (int)ty));
 	// Draw
-	PerformPix(sfcDest, tx, ty, dwClr);
+	C4BltVertex vtx;
+	vtx.ftx = tx;
+	vtx.fty = ty;
+	DwTo4UB(dwClr, vtx.color);
+	PerformMultiPix(sfcDest, &vtx, 1);
 }
 
 void C4Draw::DrawLineDw(C4Surface * sfcTarget, float x1, float y1, float x2, float y2, DWORD dwClr, float width)
 {
-	ApplyZoom(x1, y1);
-	ApplyZoom(x2, y2);
-	// manual clipping?
-	if (Config.Graphics.ClipManuallyE)
-	{
-		float i;
-		// sort left/right
-		if (x1>x2) { i=x1; x1=x2; x2=i; i=y1; y1=y2; y2=i; }
-		// clip horizontally
-		if (x1 < iClipX1)
-			if (x2 < iClipX1)
-				return; // left out
-			else
-				{ y1+=(y2-y1)*((float)iClipX1-x1)/(x2-x1); x1=(float)iClipX1; } // clip left
-		else if (x2 > iClipX2)
-		{
-			if (x1 > iClipX2)
-				return; // right out
-			else
-				{ y2-=(y2-y1)*(x2-(float)iClipX2)/(x2-x1); x2=(float)iClipX2; } // clip right
-		}
-		// sort top/bottom
-		if (y1>y2) { i=x1; x1=x2; x2=i; i=y1; y1=y2; y2=i; }
-		// clip vertically
-		if (y1 < iClipY1)
-		{
-			if (y2 < iClipY1)
-				return; // top out
-			else
-				{ x1+=(x2-x1)*((float)iClipY1-y1)/(y2-y1); y1=(float)iClipY1; } // clip top
-		}
-		else if (y2 > iClipY2)
-		{
-			if (y1 > iClipY2)
-				return; // bottom out
-			else
-				{ x2-=(x2-x1)*(y2-(float)iClipY2)/(y2-y1); y2=(float)iClipY2; } // clip bottom
-		}
-	}
-	// apply color modulation
-	ClrByCurrentBlitMod(dwClr);
-
-	PerformLine(sfcTarget, x1, y1, x2, y2, dwClr, width);
+	C4BltVertex vertices[2];
+	vertices[0].ftx = x1; vertices[0].fty = y1;
+	vertices[1].ftx = x2; vertices[1].fty = y2;
+	DwTo4UB(dwClr, vertices[0].color);
+	DwTo4UB(dwClr, vertices[1].color);
+	PerformMultiLines(sfcTarget, vertices, 2, width);
 }
 
 void C4Draw::DrawFrameDw(C4Surface * sfcDest, int x1, int y1, int x2, int y2, DWORD dwClr) // make these parameters float...?
 {
-	DrawLineDw(sfcDest,(float)x1,(float)y1,(float)x2,(float)y1, dwClr);
-	DrawLineDw(sfcDest,(float)x2,(float)y1,(float)x2,(float)y2, dwClr);
-	DrawLineDw(sfcDest,(float)x2,(float)y2,(float)x1,(float)y2, dwClr);
-	DrawLineDw(sfcDest,(float)x1,(float)y2,(float)x1,(float)y1, dwClr);
+	C4BltVertex vertices[8];
+	vertices[0].ftx = x1; vertices[0].fty = y1;
+	vertices[1].ftx = x2; vertices[1].fty = y1;
+	vertices[2] = vertices[1];
+	vertices[3].ftx = x2; vertices[3].fty = y2;
+	vertices[4] = vertices[3];
+	vertices[5].ftx = x1; vertices[5].fty = y2;
+	vertices[6] = vertices[5];
+	vertices[7] = vertices[0];
+
+	for(int i = 0; i < 8; ++i)
+		DwTo4UB(dwClr, vertices[i].color);
+
+	PerformMultiLines(sfcDest, vertices, 8, 1.0f);
 }
 
-// Globally locked surface variables - for DrawLine callback crap
-
-C4Surface *GLSBuffer=NULL;
-
-bool LockSurfaceGlobal(C4Surface * sfcTarget)
+void C4Draw::DrawQuadDw(C4Surface * sfcTarget, float *ipVtx, DWORD dwClr1, DWORD dwClr2, DWORD dwClr3, DWORD dwClr4)
 {
-	if (GLSBuffer) return false;
-	GLSBuffer=sfcTarget;
-	return !!sfcTarget->Lock();
-}
-
-bool UnLockSurfaceGlobal(C4Surface * sfcTarget)
-{
-	if (!GLSBuffer) return false;
-	sfcTarget->Unlock();
-	GLSBuffer=NULL;
-	return true;
-}
-
-bool DLineSPixDw(int32_t x, int32_t y, int32_t dwClr)
-{
-	if (!GLSBuffer) return false;
-	GLSBuffer->SetPixDw(x,y,(DWORD) dwClr);
-	return true;
+	C4BltVertex vertices[6];
+	vertices[0].ftx = ipVtx[0]; vertices[0].fty = ipVtx[1];
+	vertices[1].ftx = ipVtx[2]; vertices[1].fty = ipVtx[3];
+	vertices[2].ftx = ipVtx[4]; vertices[2].fty = ipVtx[5];
+	vertices[3].ftx = ipVtx[6]; vertices[3].fty = ipVtx[7];
+	DwTo4UB(dwClr1, vertices[0].color);
+	DwTo4UB(dwClr2, vertices[1].color);
+	DwTo4UB(dwClr3, vertices[2].color);
+	DwTo4UB(dwClr4, vertices[3].color);
+	vertices[4] = vertices[0];
+	vertices[5] = vertices[2];
+	PerformMultiTris(sfcTarget, vertices, 6, NULL, NULL, NULL, NULL, 0);
 }
 
 void C4Draw::DrawPatternedCircle(C4Surface * sfcDest, int x, int y, int r, BYTE col, C4Pattern & Pattern, CStdPalette &rPal)
 {
-	if (!sfcDest->Lock()) return;
+	bool fRenderTarget = sfcDest->IsRenderTarget();
+	if (!fRenderTarget) if (!sfcDest->Lock()) return;
 	for (int ycnt = -r; ycnt < r; ycnt++)
 	{
-		int lwdt = (int) sqrt(float(r * r - ycnt * ycnt));
+		int lwdt = (int)sqrt(float(r * r - ycnt * ycnt));
 		// Set line
-		for (int xcnt = x - lwdt; xcnt < x + lwdt; ++xcnt)
+		if (fRenderTarget)
 		{
-			sfcDest->SetPixDw(xcnt, y + ycnt, Pattern.PatternClr(xcnt, y + ycnt));
+			for (int xcnt = x - lwdt; xcnt < x + lwdt; ++xcnt)
+			{
+				DrawPix(sfcDest, xcnt, y + ycnt, Pattern.PatternClr(xcnt, y + ycnt));
+			}
+		}
+		else
+		{
+			for (int xcnt = x - lwdt; xcnt < x + lwdt; ++xcnt)
+			{
+				sfcDest->SetPixDw(xcnt, y + ycnt, Pattern.PatternClr(xcnt, y + ycnt));
+			}
 		}
 	}
-	sfcDest->Unlock();
+	if (!fRenderTarget) sfcDest->Unlock();
 }
 
 void C4Draw::Grayscale(C4Surface * sfcSfc, int32_t iOffset)
@@ -1089,7 +831,7 @@ void C4Draw::Grayscale(C4Surface * sfcSfc, int32_t iOffset)
 		{
 			DWORD dwColor = sfcSfc->GetPixDw(xcnt,ycnt,false);
 			uint32_t r = GetRedValue(dwColor), g = GetGreenValue(dwColor), b = GetBlueValue(dwColor), a = dwColor >> 24;
-			int32_t gray = BoundBy<int32_t>((r + g + b) / 3 + iOffset, 0, 255);
+			int32_t gray = Clamp<int32_t>((r + g + b) / 3 + iOffset, 0, 255);
 			sfcSfc->SetPixDw(xcnt, ycnt, RGBA(gray, gray, gray, a));
 		}
 	}
@@ -1104,6 +846,20 @@ bool C4Draw::GetPrimaryClipper(int &rX1, int &rY1, int &rX2, int &rY2)
 	return true;
 }
 
+C4Rect C4Draw::GetClipRect() const
+{
+	int iWdt=Min(iClipX2, RenderTarget->Wdt-1)-iClipX1+1;
+	int iHgt=Min(iClipY2, RenderTarget->Hgt-1)-iClipY1+1;
+	int iX=iClipX1; if (iX<0) { iWdt+=iX; iX=0; }
+	int iY=iClipY1; if (iY<0) { iHgt+=iY; iY=0; }
+	return C4Rect(iX, iY, iWdt, iHgt);
+}
+
+C4Rect C4Draw::GetOutRect() const
+{
+	return C4Rect(0, 0, RenderTarget->Wdt, RenderTarget->Hgt);
+}
+
 void C4Draw::SetGamma(DWORD dwClr1, DWORD dwClr2, DWORD dwClr3, int32_t iRampIndex)
 {
 	// No gamma effects
@@ -1145,7 +901,7 @@ void C4Draw::ApplyGamma()
 				// add offset
 				ChanOff[iChan]+=(int32_t) BYTE(dwGamma[iRamp*3+iCurve]>>(16-iChan*8)) - DefChanVal[iCurve];
 		// calc curve point
-		tGamma[iCurve]=C4RGB(BoundBy<int32_t>(DefChanVal[iCurve]+ChanOff[0], 0, 255), BoundBy<int32_t>(DefChanVal[iCurve]+ChanOff[1], 0, 255), BoundBy<int32_t>(DefChanVal[iCurve]+ChanOff[2], 0, 255));
+		tGamma[iCurve]=C4RGB(Clamp<int32_t>(DefChanVal[iCurve]+ChanOff[0], 0, 255), Clamp<int32_t>(DefChanVal[iCurve]+ChanOff[1], 0, 255), Clamp<int32_t>(DefChanVal[iCurve]+ChanOff[2], 0, 255));
 	}
 	// calc ramp
 	Gamma.Set(tGamma[0], tGamma[1], tGamma[2]);
@@ -1171,7 +927,7 @@ DWORD C4Draw::ApplyGammaTo(DWORD dwClr)
 	return Gamma.ApplyTo(dwClr);
 }
 
-void C4Draw::SetZoom(int X, int Y, float Zoom)
+void C4Draw::SetZoom(float X, float Y, float Zoom)
 {
 	this->ZoomX = X; this->ZoomY = Y; this->Zoom = Zoom;
 }
@@ -1188,7 +944,7 @@ void C4Draw::RemoveZoom(float & X, float & Y)
 	Y = (Y - ZoomY) / Zoom + ZoomY;
 }
 
-bool DDrawInit(C4AbstractApp * pApp, bool Editor, bool fUsePageLock, unsigned int iXRes, unsigned int iYRes, int iBitDepth, unsigned int iMonitor)
+bool DDrawInit(C4AbstractApp * pApp, unsigned int iXRes, unsigned int iYRes, int iBitDepth, unsigned int iMonitor)
 {
 	// create engine
     #ifndef USE_CONSOLE
@@ -1198,7 +954,7 @@ bool DDrawInit(C4AbstractApp * pApp, bool Editor, bool fUsePageLock, unsigned in
     #endif
 	if (!pDraw) return false;
 	// init it
-	if (!pDraw->Init(pApp, Editor, fUsePageLock, iXRes, iYRes, iBitDepth, iMonitor))
+	if (!pDraw->Init(pApp, iXRes, iYRes, iBitDepth, iMonitor))
 	{
 		delete pDraw;
 		return false;
@@ -1207,7 +963,7 @@ bool DDrawInit(C4AbstractApp * pApp, bool Editor, bool fUsePageLock, unsigned in
 	return true;
 }
 
-bool C4Draw::Init(C4AbstractApp * pApp, bool Editor, bool fUsePageLock, unsigned int iXRes, unsigned int iYRes, int iBitDepth, unsigned int iMonitor)
+bool C4Draw::Init(C4AbstractApp * pApp, unsigned int iXRes, unsigned int iYRes, int iBitDepth, unsigned int iMonitor)
 {
 	this->pApp = pApp;
 
@@ -1217,57 +973,17 @@ bool C4Draw::Init(C4AbstractApp * pApp, bool Editor, bool fUsePageLock, unsigned
 
 	pApp->pWindow->pSurface = new C4Surface(pApp, pApp->pWindow);
 
-	if (!CreatePrimarySurfaces(Editor, iXRes, iYRes, iBitDepth, iMonitor))
+	if (!CreatePrimarySurfaces(iXRes, iYRes, iBitDepth, iMonitor))
 		return false;
 
-	DebugLog("  Create Clipper");
-
 	if (!CreatePrimaryClipper(iXRes, iYRes))
 		return Error("  Clipper failure.");
 
-	this->Editor = Editor;
-
 	return true;
 }
 
 void C4Draw::DrawBoxFade(C4Surface * sfcDest, float iX, float iY, float iWdt, float iHgt, DWORD dwClr1, DWORD dwClr2, DWORD dwClr3, DWORD dwClr4, int iBoxOffX, int iBoxOffY)
 {
-	ApplyZoom(iX, iY);
-	iWdt *= Zoom;
-	iHgt *= Zoom;
-	// clipping not performed - this fn should be called for clipped rects only
-	// apply modulation map: Must sectionize blit
-	if (fUseClrModMap)
-	{
-		int iModResX = pClrModMap ? pClrModMap->GetResolutionX() : C4FogOfWar::DefResolutionX;
-		int iModResY = pClrModMap ? pClrModMap->GetResolutionY() : C4FogOfWar::DefResolutionY;
-		iBoxOffX %= iModResX;
-		iBoxOffY %= iModResY;
-		if (iWdt+iBoxOffX > iModResX || iHgt+iBoxOffY > iModResY)
-		{
-			if (iWdt<=0 || iHgt<=0) return;
-			CColorFadeMatrix clrs(int(iX), int(iY), int(iWdt), int(iHgt), dwClr1, dwClr2, dwClr3, dwClr4);
-			float iMaxH = float(iModResY - iBoxOffY);
-			float w,h;
-			for (float y = iY, H = iHgt; H > 0; (y += h), (H -= h), (iMaxH = float(iModResY)))
-			{
-				h = Min(H, iMaxH);
-				float iMaxW = float(iModResX - iBoxOffX);
-				for (float x = iX, W = iWdt; W > 0; (x += w), (W -= w), (iMaxW = float(iModResX)))
-				{
-					w = Min(W, iMaxW);
-					//DrawBoxFade(sfcDest, x,y,w,h, clrs.GetColorAt(x,y), clrs.GetColorAt(x+w,y), clrs.GetColorAt(x,y+h), clrs.GetColorAt(x+w,y+h), 0,0);
-					float vtx[8];
-					vtx[0] = x     ; vtx[1] = y;
-					vtx[2] = x     ; vtx[3] = y+h;
-					vtx[4] = x+w; vtx[5] = y+h;
-					vtx[6] = x+w; vtx[7] = y;
-					DrawQuadDw(sfcDest, vtx, clrs.GetColorAt(int(x),int(y)), clrs.GetColorAt(int(x),int(y+h)), clrs.GetColorAt(int(x+w),int(y+h)), clrs.GetColorAt(int(x+w),int(y)));
-				}
-			}
-			return;
-		}
-	}
 	// set vertex buffer data
 	// vertex order:
 	// 0=upper left   dwClr1
@@ -1284,5 +1000,17 @@ void C4Draw::DrawBoxFade(C4Surface * sfcDest, float iX, float iY, float iWdt, fl
 
 void C4Draw::DrawBoxDw(C4Surface * sfcDest, int iX1, int iY1, int iX2, int iY2, DWORD dwClr)
 {
-	DrawBoxFade(sfcDest, float(iX1), float(iY1), float(iX2-iX1+1), float(iY2-iY1+1), dwClr, dwClr, dwClr, dwClr, 0,0);
+	if (!sfcDest->IsRenderTarget())
+	{
+		// Box on non-render target: Emulate by setting pixels
+		if (!sfcDest->Lock()) return;
+		for (int y = iY1; y <= iY2; ++y)
+			for (int x = iX1; x <= iX2; ++x)
+				sfcDest->SetPixDw(x,y, dwClr);
+		sfcDest->Unlock();
+	}
+	else
+	{
+		DrawBoxFade(sfcDest, float(iX1), float(iY1), float(iX2 - iX1 + 1), float(iY2 - iY1 + 1), dwClr, dwClr, dwClr, dwClr, 0, 0);
+	}
 }
