@@ -17,6 +17,11 @@
 #include <C4DrawT.h>
 #include <StdMeshMaterial.h>
 
+CStdNoGfx::CStdNoGfx()
+{
+	Default();
+}
+
 bool CStdNoGfx::CreatePrimarySurfaces(bool Fullscreen, unsigned int iXRes, unsigned int iYRes, int iColorDepth, unsigned int iMonitor)
 {
 	Log("Graphics disabled.");
@@ -26,8 +31,7 @@ bool CStdNoGfx::CreatePrimarySurfaces(bool Fullscreen, unsigned int iXRes, unsig
 	return true;
 }
 
-bool CStdNoGfx::PrepareMaterial(StdMeshMaterial& mesh)
+bool CStdNoGfx::PrepareMaterial(StdMeshMatManager& mat_manager, StdMeshMaterialLoader& loader, StdMeshMaterial& mat)
 {
-   	mesh.BestTechniqueIndex=0; return true;
+	mat.BestTechniqueIndex=0; return true;
 }
-
