@@ -85,8 +85,9 @@ void C4Surface::Default()
 	Locked=0;
 	Attached=false;
 	fPrimary=false;
-	ppTex=NULL;
+	textures.clear();
 	pMainSfc=NULL;
+	pNormalSfc=NULL;
 #ifndef USE_CONSOLE
 	pCtx=NULL;
 #endif
@@ -119,8 +120,10 @@ void C4Surface::MoveFrom(C4Surface *psfcFrom)
 	Locked=psfcFrom->Locked;
 	Attached=psfcFrom->Attached;
 	fPrimary=psfcFrom->fPrimary; // shouldn't be true!
-	ppTex=psfcFrom->ppTex;
+	textures.swap(psfcFrom->textures);
+	psfcFrom->textures.clear();
 	pMainSfc=psfcFrom->pMainSfc;
+	pNormalSfc=psfcFrom->pNormalSfc;
 	ClrByOwnerClr=psfcFrom->ClrByOwnerClr;
 	iTexSize=psfcFrom->iTexSize;
 	iTexX=psfcFrom->iTexX; iTexY=psfcFrom->iTexY;
@@ -146,7 +149,6 @@ void C4Surface::Clear()
 	}
 #endif
 	FreeTextures();
-	ppTex=NULL;
 #ifdef _DEBUG
 	delete dbg_idx;
 	dbg_idx = NULL;
@@ -166,8 +168,8 @@ void C4Surface::NoClip()
 
 void C4Surface::Clip(int iX, int iY, int iX2, int iY2)
 {
-	ClipX=BoundBy(iX,0,Wdt-1); ClipY=BoundBy(iY,0,Hgt-1);
-	ClipX2=BoundBy(iX2,0,Wdt-1); ClipY2=BoundBy(iY2,0,Hgt-1);
+	ClipX=Clamp(iX,0,Wdt-1); ClipY=Clamp(iY,0,Hgt-1);
+	ClipX2=Clamp(iX2,0,Wdt-1); ClipY2=Clamp(iY2,0,Hgt-1);
 }
 
 bool C4Surface::Create(int iWdt, int iHgt, bool, bool fIsRenderTarget, int MaxTextureSize)
@@ -239,12 +241,10 @@ bool C4Surface::CreateTextures(int MaxTextureSize)
 	iTexX=(Wdt-1)/iTexSize +1;
 	iTexY=(Hgt-1)/iTexSize +1;
 	// get mem for texture array
-	ppTex = new C4TexRef * [iTexX*iTexY];
-	memset(ppTex, 0, iTexX*iTexY*sizeof(C4TexRef *));
+	textures.reserve(iTexX * iTexY);
 	// cvan't be render target if it's not a single surface
 	if (!IsSingleSurface()) fIsRenderTarget = false;
 	// create textures
-	C4TexRef **ppCTex=ppTex;
 	for (int y = 0; y < iTexY; ++y)
 	{
 		for(int x = 0; x < iTexX; ++x)
@@ -254,36 +254,11 @@ bool C4Surface::CreateTextures(int MaxTextureSize)
 			if(x == iTexX-1) sizeX = GetNeedTexSize( (Wdt - 1) % iTexSize + 1);
 			if(y == iTexY-1) sizeY = GetNeedTexSize( (Hgt - 1) % iTexSize + 1);
 
-			*ppCTex = new C4TexRef(sizeX, sizeY, fIsRenderTarget);
+			textures.emplace_back(sizeX, sizeY, fIsRenderTarget);
 			
-			if (fIsBackground && ppCTex) (*ppCTex)->FillBlack();
-
-			++ppCTex;
+			if (fIsBackground) textures.rbegin()->FillBlack();
 		}
 	}
-#if 0
-	for (int i=iTexX*iTexY; i; --i,++ppCTex)
-	{
-		// regular textures or if last texture fits exactly into the space by Wdt or Hgt
-		if (i-1 || !(Wdt%iTexSize) || !(Hgt%iTexSize))
-			*ppCTex = new C4TexRef(iTexSize, fIsRenderTarget);
-		else
-		{
-			// last texture might be smaller
-			iNeedSize=Max(Wdt%iTexSize, Hgt%iTexSize);
-#ifndef USE_CONSOLE
-			if (!pGL || !GLEW_ARB_texture_non_power_of_two)
-#endif
-			{
-				int n=0;
-				while ((1<<++n) < iNeedSize) {}
-				iNeedSize=1<<n;
-			}
-			*ppCTex = new C4TexRef(iNeedSize, fIsRenderTarget);
-		}
-		if (fIsBackground && ppCTex) (*ppCTex)->FillBlack();
-	}
-#endif
 
 #ifdef _DEBUG
 	static int dbg_counter = 0;
@@ -297,35 +272,9 @@ bool C4Surface::CreateTextures(int MaxTextureSize)
 
 void C4Surface::FreeTextures()
 {
-	if (ppTex)
-	{
-		// clear all textures
-		C4TexRef **ppTx=ppTex;
-		for (int i=0; i<iTexX*iTexY; ++i,++ppTx)
-			if (*ppTx) delete *ppTx;
-		// clear texture list
-		delete [] ppTex;
-		ppTex=NULL;
-	}
+	textures.clear();
 }
 
-/*bool ClrByOwner(DWORD &rClr) old style...
-  {
-  // red value must be approx. same to green
-  BYTE byR=GetBValue(rClr), byG=GetGValue(rClr);
-  int diff=Abs(byR-byG);
-  if (diff>byR/8) return false;
-  // get blue value; mustn't be 0 or equal to R/G (grey)
-  BYTE byB=GetRValue(rClr); if (!byB || Inside(byB, Min(byR, byG), Max(byR, byG))) return false;
-  // medium r/g and blue is very close (additional gray shade check)
-  if ((byR > 50) && (Abs(byB - byR) < 20)) return false;
-  // if blue is not fully lit, red and green should be very low
-  if (byB<240 && byR>15) return false;
-  // so, the color seems to be truly blue-ish
-  rClr=RGB(byB, byB, byB) | (rClr&0xff000000);
-  return true;
-  }*/
-
 #define  RANGE    255
 #define  HLSMAX   RANGE
 #define  RGBMAX   255
@@ -386,7 +335,7 @@ bool C4Surface::CreateColorByOwner(C4Surface *pBySurface)
 {
 	// safety
 	if (!pBySurface) return false;
-	if (!pBySurface->ppTex) return false;
+	if (pBySurface->textures.empty()) return false;
 	// create in same size
 	if (!Create(pBySurface->Wdt, pBySurface->Hgt, false)) return false;
 	// copy scale
@@ -508,73 +457,6 @@ bool C4Surface::ReadBMP(CStdStream &hGroup)
 	return true;
 }
 
-/*bool C4Surface::Save(const char *szFilename)
-  {
-  C4BMPInfo BitmapInfo2;
-  C4BMP256Info BitmapInfo;
-  // Set bitmap info
-  if (fPrimary)
-    {
-    if (byBytesPP==4)
-      BitmapInfo2.Set(Wdt,Hgt,32);
-    else
-      BitmapInfo2.Set(Wdt,Hgt,16);
-    }
-  else
-    BitmapInfo.Set(Wdt,Hgt,pPal->Colors);
-
-  // Lock - WARNING - maybe locking primary surface here...
-  if (!Lock()) return false;
-
-  // Create file & write info
-  CStdFile hFile;
-
-  if(fPrimary)
-    {
-    if ( !hFile.Create(szFilename)
-    || !hFile.Write(&BitmapInfo2,sizeof(BitmapInfo2)) )
-      { Unlock(); return false; }
-
-    // write lines
-    char bpEmpty[4]; int iEmpty = DWordAligned(Wdt*byBytesPP)-Wdt*byBytesPP;
-    for (int cnt=Hgt-1; cnt>=0; cnt--)
-      {
-      if (!hFile.Write(Bits+(Pitch*cnt),Wdt*byBytesPP))
-        { Unlock(); return false; }
-      if (iEmpty)
-        if (!hFile.Write(bpEmpty,iEmpty))
-          { Unlock(); return false; }
-      }
-
-    }
-  else
-    {
-    if ( !hFile.Create(szFilename)
-    || !hFile.Write(&BitmapInfo,sizeof(BitmapInfo)) )
-      { Unlock(); return false; }
-
-    // Write lines
-    char bpEmpty[4]; int iEmpty = DWordAligned(Wdt)-Wdt;
-    for (int cnt=Hgt-1; cnt>=0; cnt--)
-      {
-      if (!hFile.Write(Bits+(Pitch*cnt),Wdt))
-        { Unlock(); return false; }
-      if (iEmpty)
-        if (!hFile.Write(bpEmpty,iEmpty))
-          { Unlock(); return false; }
-      }
-    }
-
-  // Close file
-  hFile.Close();
-
-  // Unlock
-  Unlock();
-
-  // Success
-  return true;
-  }
-*/
 bool C4Surface::SavePNG(const char *szFilename, bool fSaveAlpha, bool fApplyGamma, bool fSaveOverlayOnly)
 {
 	// Lock - WARNING - maybe locking primary surface here...
@@ -646,7 +528,7 @@ bool C4Surface::Lock()
 	// lock main sfc
 	if (pMainSfc) if (!pMainSfc->Lock()) return false;
 	// lock texture
-	if (!Locked && !fPrimary && !ppTex)
+	if (!Locked && !fPrimary && textures.empty())
 		return false;
 	// count lock
 	Locked++; return true;
@@ -674,25 +556,20 @@ bool C4Surface::Unlock()
 		else
 		{
 			// non-primary unlock: unlock all texture surfaces (if locked)
-			C4TexRef **ppTx=ppTex;
-			for (int i=0; i<iTexX*iTexY; ++i,++ppTx)
-				(*ppTx)->Unlock();
+			for (auto &texture : textures)
+				texture.Unlock();
 		}
 	}
 	return true;
 }
 
-bool C4Surface::GetTexAt(C4TexRef **ppTexRef, int &rX, int &rY)
+bool C4Surface::GetTexAtImpl(C4TexRef **ppTexRef, int &rX, int &rY)
 {
-	// texture present?
-	if (!ppTex) return false;
 	// get pos
 	int iX=rX/iTexSize;
 	int iY=rY/iTexSize;
-	// clip
-	if (iX<0 || iY<0 || iX>=iTexX || iY>=iTexY) return false;
 	// get texture by pos
-	*ppTexRef=*(ppTex+iY*iTexX+iX);
+	*ppTexRef = &textures[iY*iTexX + iX];
 	// adjust pos
 	rX-=iX*iTexSize;
 	rY-=iY*iTexSize;
@@ -731,28 +608,18 @@ DWORD C4Surface::GetPixDw(int iX, int iY, bool fApplyModulation)
 #ifndef USE_CONSOLE
 		if (!PrimarySurfaceLockBits)
 		{
-			PrimarySurfaceLockBits = new unsigned char[Wdt*Hgt*3 + 1];
+			PrimarySurfaceLockBits = new unsigned char[Wdt*Hgt*3];
+			glPixelStorei(GL_PACK_ALIGNMENT, 1);
 			glReadPixels( 0, 0, Wdt, Hgt, GL_BGR, GL_UNSIGNED_BYTE, PrimarySurfaceLockBits);
 			PrimarySurfaceLockPitch = Wdt*3;
 		}
 		return * (DWORD *) (PrimarySurfaceLockBits+(Hgt-iY-1)*PrimarySurfaceLockPitch+iX*3);
-
-			// copy content into textures
-			/*if (!ppTex) if (!CreatePrimaryGLTextures()) return 0;
-			// get+lock affected texture - inverse Y as primary is locked upside down!
-			iY = Hgt-iY-1;
-			C4TexRef *pTexRef;
-			if (!GetLockTexAt(&pTexRef, iX, iY)) return 0;
-			pBuf=(BYTE *) pTexRef->texLock.pBits;
-			iPitch=pTexRef->texLock.Pitch;
-			// get pixel
-			return *(DWORD *)(pBuf+iY*iPitch+iX*4);*/
 #endif
 	}
 	else
 	{
 		// get+lock affected texture
-		if (!ppTex) return 0;
+		if (textures.empty()) return 0;
 		C4TexRef *pTexRef;
 		if (!GetLockTexAt(&pTexRef, iX, iY)) return 0;
 		pBuf=(BYTE *) pTexRef->texLock.pBits;
@@ -826,33 +693,12 @@ bool C4Surface::IsPixTransparent(int iX, int iY)
 	return (dwPix>>24) < 128;
 }
 
-/*bool C4Surface::SetPixEx(int iX, int iY, BYTE byCol, DWORD dwClr)
-  {
-  // clip
-  if ((iX<ClipX) || (iX>ClipX2) || (iY<ClipY) || (iY>ClipY2)) return true;
-  // primary?
-  if (fPrimary)
-#ifndef USE_CONSOLE
-      pGL->DrawPixInt(this, iX, iY, dwClr);
-    else
-#endif
-      {
-      }
-    return true;
-    }
-  else
-    {
-    SetPixDw(iX, iY, dwClr);
-    }
-  return true;
-  }*/
-
 bool C4Surface::SetPixDw(int iX, int iY, DWORD dwClr)
 {
 	// clip
 	if ((iX<ClipX) || (iX>ClipX2) || (iY<ClipY) || (iY>ClipY2)) return true;
 	// get+lock affected texture
-	if (!ppTex) return false;
+	if (textures.empty()) return false;
 	// if color is fully transparent, ensure it's black
 	if (dwClr>>24 == 0x00) dwClr=0x00000000;
 	C4TexRef *pTexRef;
@@ -915,7 +761,7 @@ bool C4Surface::SetPixAlpha(int iX, int iY, BYTE byAlpha)
 	// clip
 	if ((iX<ClipX) || (iX>ClipX2) || (iY<ClipY) || (iY>ClipY2)) return true;
 	// get+lock affected texture
-	if (!ppTex) return false;
+	if (textures.empty()) return false;
 	C4TexRef *pTexRef;
 	if (!GetLockTexAt(&pTexRef, iX, iY)) return false;
 	// set alpha value of pix in surface
@@ -1001,13 +847,13 @@ void C4Surface::ClearBoxDw(int iX, int iY, int iWdt, int iHgt)
 	int iTexY2=Min((iY+iHgt-1)/iTexSize +1, iTexY);
 	// clear basesfc?
 	bool fBaseSfc=false;
-	if (pMainSfc) if (pMainSfc->ppTex) fBaseSfc=true;
+	if (pMainSfc) if (!pMainSfc->textures.empty()) fBaseSfc = true;
 	// clear all these textures
 	for (int y=iTexY1; y<iTexY2; ++y)
 	{
 		for (int x=iTexX1; x<iTexX2; ++x)
 		{
-			C4TexRef *pTex = *(ppTex + y * iTexX + x);
+			C4TexRef *pTex = &textures[y * iTexX + x];
 			// get current offset in texture
 			int iBlitX=iTexSize*x;
 			int iBlitY=iTexSize*y;
@@ -1020,7 +866,7 @@ void C4Surface::ClearBoxDw(int iX, int iY, int iWdt, int iHgt)
 				// then get this surface as same offset as from other surface
 				// assuming this is only valid as long as there's no texture management,
 				// organizing partially used textures together!
-				C4TexRef *pBaseTex = *(pMainSfc->ppTex + y * iTexX + x);
+				C4TexRef *pBaseTex = &pMainSfc->textures[y * iTexX + x];
 				pBaseTex->ClearRect(rtClear);
 			}
 			// clear this texture
@@ -1032,7 +878,7 @@ void C4Surface::ClearBoxDw(int iX, int iY, int iWdt, int iHgt)
 bool C4Surface::CopyBytes(BYTE *pImageData)
 {
 	// copy image data directly into textures
-	C4TexRef **ppCurrTex = ppTex, *pTex = *ppTex;
+	auto pTex = textures.begin();
 	int iSrcPitch = Wdt * byBytesPP; int iLineTotal = 0;
 	for (int iY=0; iY<iTexY; ++iY)
 	{
@@ -1040,7 +886,7 @@ bool C4Surface::CopyBytes(BYTE *pImageData)
 		int iLastHeight=pTex->iSizeY; int iXImgPos=0;
 		for (int iX=0; iX<iTexX; ++iX)
 		{
-			pTex = *ppCurrTex++;
+			++pTex;
 			if (!pTex->Lock()) return false;
 			BYTE *pTarget = (BYTE*)pTex->texLock.pBits;
 			int iCpyNum = Min(pTex->iSizeX, Wdt-iXImgPos)*byBytesPP;
@@ -1188,9 +1034,8 @@ void C4TexRef::Unlock()
 				glBindTexture(GL_TEXTURE_2D, texName);
 				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
 				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
-				// Default, changed in PerformBlt if necessary
-				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
-				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
+				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
 				glTexImage2D(GL_TEXTURE_2D, 0, 4, iSizeX, iSizeY, 0, GL_BGRA, pDraw->byByteCnt == 2 ? GL_UNSIGNED_SHORT_4_4_4_4_REV : GL_UNSIGNED_INT_8_8_8_8_REV, texLock.pBits);
 			}
 			else
