@@ -18,10 +18,10 @@
 #include <C4Include.h>
 #include <C4GameLobby.h>
 
-#include "C4FullScreen.h"
+#include <C4Application.h>
+#include <C4Components.h>
 #include "C4Network2Dialogs.h"
 #include "C4GameOptions.h"
-#include "C4GameDialogs.h"
 #include "C4RTF.h"
 #include "C4ChatDlg.h"
 #include "C4PlayerInfoListBox.h"
@@ -50,6 +50,15 @@ namespace C4GameLobby
 	}
 
 
+// ----------- C4PacketSetScenarioParameter ---------------------------------------------
+
+	void C4PacketSetScenarioParameter::CompileFunc(StdCompiler *pComp)
+	{
+		pComp->Value(mkNamingAdapt(mkParAdapt(ID, StdCompiler::RCT_Idtf), "ID", StdCopyStrBuf()));
+		pComp->Value(mkNamingAdapt(Value, "Value", 0));
+	}
+
+
 // ----------- ScenDescs ---------------------------------------------
 
 	ScenDesc::ScenDesc(const C4Rect &rcBounds, bool fActive) : C4GUI::Window(), fDescFinished(false)
@@ -210,7 +219,7 @@ namespace C4GameLobby
 		pPlayerSheet->AddElement(pPlayerList);
 		pResList = new C4Network2ResDlg(pResSheet->GetContainedClientRect(), false);
 		pResSheet->AddElement(pResList);
-		pOptionsList = new C4GameOptionsList(pResSheet->GetContainedClientRect(), false, false);
+		pOptionsList = new C4GameOptionsList(pResSheet->GetContainedClientRect(), false, C4GameOptionsList::GOLS_Lobby);
 		pOptionsSheet->AddElement(pOptionsList);
 		pScenarioInfo = new ScenDesc(pResSheet->GetContainedClientRect(), false);
 		pScenarioSheet->AddElement(pScenarioInfo);
@@ -483,6 +492,13 @@ namespace C4GameLobby
 			OnCountdownPacket(Pkt);
 		}
 		break;
+		case PID_SetScenarioParameter: // set a scenario parameter value
+			{
+				GETPKT(C4PacketSetScenarioParameter, Pkt);
+				::Game.Parameters.ScenarioParameters.SetValue(Pkt.GetID(), Pkt.GetValue(), false);
+				// reflect updated value immediately on clients
+				if (pRightTab->GetActiveSheetIndex() == SheetIdx_Options) if (pOptionsList) pOptionsList->Update();
+			}
 		};
 #undef GETPKT
 	}
