@@ -18,9 +18,7 @@
 
 #include <C4Include.h>
 #include <C4Gui.h>
-#include <C4FullScreen.h>
-#include <C4LoaderScreen.h>
-#include <C4Application.h>
+
 #include <C4MouseControl.h>
 #include <C4GraphicsResource.h>
 
@@ -198,7 +196,7 @@ namespace C4GUI
 		return this;
 	}
 
-	bool Container::OnHotkey(char cHotkey)
+	bool Container::OnHotkey(uint32_t cHotkey)
 	{
 		if (!IsVisible()) return false;
 		// check all nested elements
@@ -370,7 +368,7 @@ namespace C4GUI
 				int32_t iWinScroll = pScrollWindow->iScrollY;
 				// scroll thumb height is currently hardcoded
 				// calc scroll pos
-				iScrollPos = BoundBy<int32_t>(iMaxBarScroll * iWinScroll / iMaxWinScroll, 0, iMaxBarScroll);
+				iScrollPos = Clamp<int32_t>(iMaxBarScroll * iWinScroll / iMaxWinScroll, 0, iMaxBarScroll);
 			}
 		}
 		else fScrolling = !!pScrollCallback;
@@ -386,7 +384,7 @@ namespace C4GUI
 		int32_t iMaxBarScroll = GetMaxScroll();
 		if (!iMaxBarScroll) iMaxBarScroll=1;
 		// CB - passes scroll pos
-		if (pScrollCallback) pScrollCallback->DoCall(BoundBy<int32_t>(iScrollPos * (iCBMaxRange-1) / iMaxBarScroll, 0, (iCBMaxRange-1)));
+		if (pScrollCallback) pScrollCallback->DoCall(Clamp<int32_t>(iScrollPos * (iCBMaxRange-1) / iMaxBarScroll, 0, (iCBMaxRange-1)));
 		// safety
 		if (!pScrollWindow || !fScrolling) return;
 		// get scrolling values
@@ -396,7 +394,7 @@ namespace C4GUI
 		int32_t iMaxWinScroll = iClientHgt - iVisHgt;
 		int32_t iWinScroll = pScrollWindow->iScrollY;
 		// calc new window scrolling
-		int32_t iNewWinScroll = BoundBy<int32_t>(iMaxWinScroll * iScrollPos / iMaxBarScroll, 0, iMaxWinScroll);
+		int32_t iNewWinScroll = Clamp<int32_t>(iMaxWinScroll * iScrollPos / iMaxBarScroll, 0, iMaxWinScroll);
 		// apply it, if it is different
 		if (iWinScroll != iNewWinScroll)
 			pScrollWindow->SetScroll(iNewWinScroll);
@@ -511,7 +509,7 @@ namespace C4GUI
 		// not if window is being refilled
 		if (iFrozen) return;
 		// do not scroll outside range
-		iScrollY = BoundBy<int32_t>(iScrollY, 0, Max<int32_t>(iClientHeight - GetBounds().Hgt, 0));
+		iScrollY = Clamp<int32_t>(iScrollY, 0, Max<int32_t>(iClientHeight - GetBounds().Hgt, 0));
 		// update client rect
 		rcClientRect.x = 0;
 		rcClientRect.y = -iScrollY;
@@ -552,7 +550,7 @@ namespace C4GUI
 		int32_t iVisHgt = GetBounds().Hgt;
 		int32_t iClientHgt = GetClientRect().Hgt;
 		int32_t iMaxScroll = iClientHgt - iVisHgt;
-		int iNewScrollY = BoundBy<int>(iScrollY + iAmount, 0, iMaxScroll);
+		int iNewScrollY = Clamp<int>(iScrollY + iAmount, 0, iMaxScroll);
 		if (iScrollY != iNewScrollY)
 		{
 			// scrolling possible: do it
