@@ -21,15 +21,11 @@
 #include <C4Gui.h>
 
 #include <C4DefList.h>
-#include <C4FullScreen.h>
-#include <C4LoaderScreen.h>
 #include <C4Application.h>
 #include <C4Viewport.h>
-#include <C4Console.h>
 #include <C4Def.h>
-#include <C4MouseControl.h>
 #include <C4GraphicsResource.h>
-#include <C4Game.h>
+#include <C4Application.h>
 
 #include <C4DrawGL.h>
 #include <StdRegistry.h>
@@ -641,14 +637,21 @@ namespace C4GUI
 
 	bool Dialog::DoModal()
 	{
+		// Cancel all dialogues if game is left (including e.g. league dialogues)
+		if (::Application.IsQuittingGame()) return false;
 		// main message loop
 		while (fShow)
 		{
 			// dialog idle proc
 			OnIdle();
+			// Modal dialogue during running game is tricky. Do not execute game!
+			bool fGameWasRunning = ::Game.IsRunning;
+			::Game.IsRunning = false;
 			// handle messages - this may block until the next timer
 			if (!Application.ScheduleProcs())
 				return false; // game GUI and lobby will deleted in Game::Clear()
+			// reset game run state
+			if (fGameWasRunning) ::Game.IsRunning = true;
 		}
 		// return whether dlg was OK
 		return fOK;
@@ -1186,6 +1189,10 @@ bool MessageDialog::KeyCopy()
 		pTextWin->UpdateHeight();
 		//pTextWin->ScrollToBottom();
 	}
+	InfoDialog::~InfoDialog()
+	{
+		Application.Remove(this);
+	}
 
 	void InfoDialog::CreateSubComponents()
 	{
