@@ -19,9 +19,6 @@
 #include <C4Include.h>
 #include <C4Gui.h>
 
-#include <C4FullScreen.h>
-#include <C4LoaderScreen.h>
-#include <C4Application.h>
 #include <C4MouseControl.h>
 #include <algorithm>
 
