@@ -303,11 +303,14 @@ namespace C4GUI
 		// not if focus is in submenu
 		if (pSubmenu) return false;
 		Element *pPrevSelectedItem = pSelectedItem;
-		C4KeyCode wKey = TOUPPERIFX11(key.Key);
+		StdStrBuf sKey = C4KeyCodeEx::KeyCode2String(key.Key, true, true);
+		// do hotkey procs for standard alphanumerics only
+		if (sKey.getLength() != 1) return false;
+		WORD wKey = WORD(*sKey.getData());
 		if (Inside<C4KeyCode, C4KeyCode, C4KeyCode>(wKey, 'A', 'Z') || Inside<C4KeyCode, C4KeyCode, C4KeyCode>(wKey, '0', '9'))
 		{
 			// process hotkeys
-			char ch=char(wKey);
+			uint32_t ch = wKey;
 			for (Element *pCurr = GetFirst(); pCurr; pCurr = pCurr->GetNext())
 				if (pCurr->OnHotkey(ch))
 				{
