@@ -22,16 +22,12 @@
 
 #include <C4DefList.h>
 #include <C4Object.h>
-#include <C4FullScreen.h>
-#include <C4ObjectCom.h>
 #include <C4Viewport.h>
 #include <C4Player.h>
 #include <C4MouseControl.h>
 #include <C4GraphicsResource.h>
-#include <C4GraphicsSystem.h>
 #include <C4Game.h>
 #include <C4PlayerList.h>
-#include <C4GameObjects.h>
 #include <C4GameControl.h>
 
 const int32_t     C4MN_DefInfoWdt     = 270, // default width of info windows
@@ -741,7 +737,7 @@ void C4Menu::InitLocation(C4Facet &cgoArea)
 	if (Alignment & C4MN_Align_Right) X=cgoArea.Wdt-2*C4SymbolSize-rcBounds.Wdt;
 	if (Alignment & C4MN_Align_Top) Y=C4SymbolSize;
 	if (Alignment & C4MN_Align_Bottom) Y=cgoArea.Hgt-C4SymbolSize-rcBounds.Hgt;
-	if (Alignment & C4MN_Align_Free) { X=BoundBy<int32_t>(X,0,cgoArea.Wdt-rcBounds.Wdt); Y=BoundBy<int32_t>(Y,0,cgoArea.Hgt-rcBounds.Hgt); }
+	if (Alignment & C4MN_Align_Free) { X=Clamp<int32_t>(X,0,cgoArea.Wdt-rcBounds.Wdt); Y=Clamp<int32_t>(Y,0,cgoArea.Hgt-rcBounds.Hgt); }
 	// Centered (due to small viewport size)
 	if (rcBounds.Wdt>cgoArea.Wdt-2*C4SymbolSize) X=(cgoArea.Wdt-rcBounds.Wdt)/2;
 	if (rcBounds.Hgt>cgoArea.Hgt-2*C4SymbolSize) Y=(cgoArea.Hgt-rcBounds.Hgt)/2;
