@@ -20,10 +20,8 @@
 #include <C4Include.h>
 #include <C4MouseControl.h>
 
-#include <C4DefList.h>
 #include <C4Viewport.h>
 #include <C4Object.h>
-#include <C4Command.h>
 #include <C4Application.h>
 #include <C4FullScreen.h>
 #include <C4Gui.h>
@@ -32,9 +30,7 @@
 #include <C4Player.h>
 #include "C4ChatDlg.h"
 #include <C4GraphicsResource.h>
-#include <C4GraphicsSystem.h>
 #include <C4PlayerList.h>
-#include <C4GameObjects.h>
 #include <C4GameControl.h>
 
 const int32_t C4MC_Drag_None            = 0,
@@ -43,16 +39,16 @@ const int32_t C4MC_Drag_None            = 0,
 
 const int32_t C4MC_Cursor_Select      = 0,		// click cursor to select/click stuff in the GUI
               C4MC_Cursor_Crosshair   = 1,		// standard ingame cursor
-			  C4MC_Cursor_DragDrop    = 3,		// cursor when drag&dropping
-              C4MC_Cursor_Up          = 4,		// cursors for scrolling the viewport ...
-              C4MC_Cursor_Down        = 5,		// ...
-              C4MC_Cursor_Left        = 6,
-              C4MC_Cursor_Right       = 7,
-              C4MC_Cursor_UpLeft      = 8,
-              C4MC_Cursor_UpRight     = 9,
-              C4MC_Cursor_DownLeft    = 10,
-              C4MC_Cursor_DownRight   = 11,
-			  C4MC_Cursor_Passive     = 12;		// passive cursor in records and an fog of war
+              C4MC_Cursor_DragDrop    = 2,		// cursor when drag&dropping
+              C4MC_Cursor_Up          = 3,		// cursors for scrolling the viewport ...
+              C4MC_Cursor_Down        = 4,		// ...
+              C4MC_Cursor_Left        = 5,
+              C4MC_Cursor_Right       = 6,
+              C4MC_Cursor_UpLeft      = 7,
+              C4MC_Cursor_UpRight     = 8,
+              C4MC_Cursor_DownLeft    = 9,
+              C4MC_Cursor_DownRight   = 10,
+              C4MC_Cursor_Passive     = 11;		// passive cursor in records and and fog of war and outside viewport
 
 const int32_t C4MC_Time_on_Target     = 10;
 
@@ -209,7 +205,7 @@ void C4MouseControl::Move(int32_t iButton, int32_t iX, int32_t iY, DWORD dwKeyFl
 	// get view position
 	C4Rect rcViewport = Viewport->GetOutputRect();
 	fctViewport.Set(NULL, rcViewport.x, rcViewport.y, rcViewport.Wdt, rcViewport.Hgt);
-	ViewX=Viewport->ViewX; ViewY=Viewport->ViewY;
+	ViewX=Viewport->GetViewX(); ViewY=Viewport->GetViewY();
 	fctViewportGame = Viewport->last_game_draw_cgo;
 	fctViewportGUI = Viewport->last_gui_draw_cgo;
 	// First time viewport attachment: center mouse
@@ -397,12 +393,16 @@ void C4MouseControl::Draw(C4TargetFacet &cgo, const ZoomData &GameZoom)
 				ImageWdt = Def->PictureRect.Wdt;
 				ImageHgt = Def->PictureRect.Hgt;
 			}
-			else
+			else if (pGfx->Type == C4DefGraphics::TYPE_Mesh)
 			{
 				// Note bounding box is in OGRE coordinate system
 				ImageWdt = pGfx->Mesh->GetBoundingBox().y2 - pGfx->Mesh->GetBoundingBox().y1;
 				ImageHgt = pGfx->Mesh->GetBoundingBox().z2 - pGfx->Mesh->GetBoundingBox().z1;
 			}
+			else
+			{
+				ImageWdt = ImageHgt = 1.0f;
+			}
 
 			// zoom mode: Drag in GUI or Game depending on source object
 			bool fIsGameZoom = true;
@@ -542,7 +542,7 @@ void C4MouseControl::UpdateCursorTarget()
 	}
 
 	// Make a script callback if the object being hovered changes
-	if(OldTargetObject != TargetObject)
+	if(!IsPassive() && OldTargetObject != TargetObject)
 	{
 		// TODO: This might put a heavy load on the network, depending on the number of
 		// selectable objects around. If it turns out to be a problem we might want to
@@ -638,16 +638,16 @@ void C4MouseControl::DragNone()
 		// check if target object allows scripted dragging
 		if (fAllowDrag && DownTarget && (!FogOfWar || (DownTarget->Category & C4D_IgnoreFoW)))
 		{
-			C4Object *drag_image_obj; C4ID drag_image_id;
+			C4Object *drag_image_obj; C4Def * drag_image_def;
 
 			// Drag only if MD_SOURCE is set and drag image is present
 			if ( (DownTarget->GetPropertyInt(P_MouseDrag) & C4MC_MD_DragSource) &&
-			      DownTarget->GetDragImage(&drag_image_obj, &drag_image_id))
+			      DownTarget->GetDragImage(&drag_image_obj, &drag_image_def))
 			{
 				Drag=C4MC_Drag_Script;
 
 				if(drag_image_obj) DragImageObject = drag_image_obj;
-				else DragImageDef = C4Id2Def(drag_image_id);
+				else DragImageDef = drag_image_def;
 
 				DragObject = DownTarget;
 			}
@@ -850,7 +850,9 @@ void C4MouseControl::UpdateFogOfWar()
 	// Assume no fog of war
 	FogOfWar=false;
 	// Check for fog of war
-	if ((pPlayer->fFogOfWar && !pPlayer->FoWIsVisible(int32_t(GameX),int32_t(GameY))) || GameX<0 || GameY<0 || int32_t(GameX)>=GBackWdt || int32_t(GameY)>=GBackHgt)
+	// TODO: Check C4FoWRegion... should maybe be passed as a parameter?
+	// pDraw->GetFoW() might not be current at this time.
+	if (/*(pPlayer->fFogOfWar && !pPlayer->FoWIsVisible(int32_t(GameX),int32_t(GameY))) || */GameX<0 || GameY<0 || int32_t(GameX)>=GBackWdt || int32_t(GameY)>=GBackHgt)
 	{
 		FogOfWar=true;
 		// allow dragging, scrolling, region selection and manipulations of objects not affected by FoW
@@ -898,9 +900,7 @@ void C4MouseControl::ScrollView(float iX, float iY, float ViewWdt, float ViewHgt
 	else if (Viewport)
 	{
 		// no player: Scroll fullscreen viewport
-		Viewport->ViewX = Viewport->ViewX+iX;
-		Viewport->ViewY = Viewport->ViewY+iY;
-		Viewport->UpdateViewPosition();
+		Viewport->ScrollView(iX, iY);
 	}
 
 }
