@@ -18,17 +18,13 @@
 #include <C4Include.h>
 #include <C4StartupNetDlg.h>
 
+#include <C4Application.h>
 #include <C4UpdateDlg.h>
 #include <C4StartupScenSelDlg.h>
-#include <C4StartupMainDlg.h>
 #include <C4Game.h>
-#include <C4Log.h>
 #include "C4ChatDlg.h"
 #include <C4GraphicsResource.h>
-
-#ifdef HAVE_SYS_SOCKET_H
-#include <sys/socket.h>
-#endif
+#include <C4Network2Reference.h>
 
 // ----------- C4StartupNetListEntry -----------------------------------------------------------------------
 
@@ -261,11 +257,6 @@ bool C4StartupNetListEntry::OnReference()
 	// special masterserver handling
 	if (eQueryType == NRQT_Masterserver)
 	{
-		// masterserver is official: So check version
-		StdStrBuf updURL;
-		StdStrBuf versInf;
-		if (pRefClient->GetUpdateURL(&updURL) && pRefClient->GetVersion(&versInf))
-			pNetDlg->CheckVersionUpdate(updURL.getData(),versInf.getData());
 		// masterserver: schedule next query
 		sInfoText[1].Format(LoadResStr("IDS_NET_INFOGAMES"), (int) iNewRefCount);
 		SetTimeout(TT_Masterserver);
@@ -278,8 +269,10 @@ bool C4StartupNetListEntry::OnReference()
 		return false;
 	}
 	else
+	{
 		// no ref found on custom adress: Schedule re-check
 		SetTimeout(TT_RefReqWait);
+	}
 	return true;
 }
 
@@ -580,7 +573,7 @@ C4Network2Reference *C4StartupNetListEntry::GrabReference()
 
 // ----------- C4StartupNetDlg ---------------------------------------------------------------------------------
 
-C4StartupNetDlg::C4StartupNetDlg() : C4StartupDlg(LoadResStr("IDS_DLG_NETSTART")), pChatTitleLabel(NULL), pMasterserverClient(NULL), fIsCollapsed(false), fUpdatingList(false), iGameDiscoverInterval(0), tLastRefresh(0)
+C4StartupNetDlg::C4StartupNetDlg() : C4StartupDlg(LoadResStr("IDS_DLG_NETSTART")), pChatTitleLabel(NULL), pMasterserverClient(NULL), fIsCollapsed(false), fUpdatingList(false), iGameDiscoverInterval(0), tLastRefresh(0), fUpdateCheckPending(false)
 {
 	// ctor
 	// key bindings
@@ -738,6 +731,8 @@ C4StartupNetDlg::~C4StartupNetDlg()
 {
 	// disable notifies
 	Application.InteractiveThread.ClearCallback(Ev_HTTP_Response, this);
+	Application.InteractiveThread.RemoveProc(&pUpdateClient);
+
 	DiscoverClient.Close();
 	Application.Remove(this);
 	if (pMasterserverClient) delete pMasterserverClient;
@@ -752,16 +747,15 @@ void C4StartupNetDlg::DrawElement(C4TargetFacet &cgo)
 	// draw background
 	typedef C4GUI::FullscreenDialog Base;
 	Base::DrawElement(cgo);
-#if 0
-	DrawBackground(cgo, C4Startup::Get()->Graphics.fctNetBG);
-#endif
 }
 
 void C4StartupNetDlg::OnShown()
 {
 	// callback when shown: Start searching for games
 	C4StartupDlg::OnShown();
+	CheckVersionUpdate();
 	UpdateList();
+	UpdateUpdateButton(); // in case update check was finished before callback registration
 	UpdateMasterserver();
 	OnSec1Timer();
 	tLastRefresh = time(0);
@@ -863,7 +857,7 @@ void C4StartupNetDlg::UpdateMasterserver()
 	else
 	{
 		pMasterserverClient = new C4StartupNetListEntry(pGameSelList, NULL, this);
-		StdStrBuf strVersion; strVersion.Format("%d.%d.%d", C4XVER1, C4XVER2, C4XVER3);
+		StdStrBuf strVersion; strVersion.Format("%d.%d", C4XVER1, C4XVER2);
 		StdStrBuf strQuery; strQuery.Format("%s?version=%s&platform=%s", Config.Network.GetLeagueServerAddress(), strVersion.getData(), C4_OS);
 		pMasterserverClient->SetRefQuery(strQuery.getData(), C4StartupNetListEntry::NRQT_Masterserver);
 	}
@@ -977,9 +971,28 @@ C4StartupNetDlg::DlgMode C4StartupNetDlg::GetDlgMode()
 
 void C4StartupNetDlg::OnThreadEvent(C4InteractiveEventType eEvent, void *pEventData)
 {
+	UpdateUpdateButton();
 	UpdateList(true);
 }
 
+void C4StartupNetDlg::UpdateUpdateButton()
+{
+	if (!fUpdateCheckPending) return;
+	if(!pUpdateClient.isSuccess() || pUpdateClient.isBusy()) return;
+
+	pUpdateClient.SetNotify(NULL);
+
+	StdCopyStrBuf versionInfo;
+
+	pUpdateClient.GetVersion(&versionInfo);
+	pUpdateClient.GetUpdateURL(&UpdateURL);
+
+#ifdef WITH_AUTOMATIC_UPDATE
+	btnUpdate->SetVisibility(C4UpdateDlg::IsValidUpdate(versionInfo.getData()));
+#endif
+	fUpdateCheckPending = false;
+}
+
 bool C4StartupNetDlg::DoOK()
 {
 	// OK in chat mode? Forward to chat control
@@ -1194,26 +1207,18 @@ void C4StartupNetDlg::OnReferenceEntryAdd(C4StartupNetListEntry *pEntry)
 		pEntry->UpdateCollapsed(true);
 }
 
-void C4StartupNetDlg::CheckVersionUpdate(const char *szUpdateURL, const char *szVersion)
+void C4StartupNetDlg::CheckVersionUpdate()
 {
 #ifdef WITH_AUTOMATIC_UPDATE
-	// Is a valid update
-	if (C4UpdateDlg::IsValidUpdate(szVersion))
+	StdStrBuf strVersion; strVersion.Format("%d.%d", C4XVER1, C4XVER2);
+	StdStrBuf strQuery; strQuery.Format("%s?version=%s&platform=%s&action=version", Config.Network.UpdateServerAddress, strVersion.getData(), C4_OS);
+
+	if (pUpdateClient.Init() && pUpdateClient.SetServer(strQuery.getData()) && pUpdateClient.QueryUpdateURL())
 	{
-		UpdateURL = szUpdateURL;
-		btnUpdate->SetVisibility(true);
-	}
-	// Otherwise: no update available
-	else
-	{
-		btnUpdate->SetVisibility(false);
-	}
-#else
-	if(szUpdateURL && *szUpdateURL)
-	{
-		// TODO: We could show an item notifying the user that an
-		// update is available externally.
+		pUpdateClient.SetNotify(&Application.InteractiveThread);
+		Application.InteractiveThread.AddProc(&pUpdateClient);
 	}
+	fUpdateCheckPending = true;
 #endif
 }
 
