@@ -42,7 +42,7 @@ int32_t FnParTexCol(C4String *mattex, int32_t default_col = -1)
 	const char *cmattex = mattex->GetCStr();
 	int32_t ift = IFT;
 	if (*cmattex == '^') { ift=0; ++cmattex; }
-	int32_t col = ::TextureMap.GetIndexMatTex(cmattex);
+	int32_t col = ::MapScript.pTexMap->GetIndexMatTex(cmattex);
 	return col ? col|ift : default_col;
 }
 
@@ -118,19 +118,19 @@ void C4MapScriptMatTexMask::UnmaskSpec(C4String *spec)
 		if (SCharCount('-', cspec))
 		{
 			// Material+Texture
-			int32_t col = ::TextureMap.GetIndexMatTex(cspec, NULL, false);
+			int32_t col = ::MapScript.pTexMap->GetIndexMatTex(cspec, NULL, false);
 			if (col) mat_mask[col] = true;
 		}
 		else
 		{
 			// Only material: Mask all textures of this material
-			int32_t mat = ::MaterialMap.Get(cspec);
+			int32_t mat = ::MapScript.pMatMap->Get(cspec);
 			if (mat!=MNone)
 			{
 				const char *tex_name;
 				int32_t col;
-				for (int32_t itex=0; (tex_name=::TextureMap.GetTexture(itex)); itex++)
-					if (col = ::TextureMap.GetIndex(cspec,tex_name,false))
+				for (int32_t itex=0; (tex_name=::MapScript.pTexMap->GetTexture(itex)); itex++)
+					if ((col = ::MapScript.pTexMap->GetIndex(cspec,tex_name,false)))
 						mat_mask[col] = true;
 			}
 		}
@@ -262,7 +262,7 @@ static bool FnLayerSetPixel(C4PropList * _this, int32_t x, int32_t y, const C4Va
 	C4MapScriptLayer *layer = _this->GetMapScriptLayer();
 	if (!layer) return false;
 	int32_t to_value; C4String *to_value_s;
-	if (to_value_s = to_value_c4v.getStr())
+	if ((to_value_s = to_value_c4v.getStr()))
 	{
 		to_value = FnParTexCol(to_value_s);
 	}
@@ -300,13 +300,13 @@ static bool FnLayerFindPosition(C4PropList * _this, C4PropList *out_pos, const C
 {
 	// Layer script function: Find a position (x,y) that has a color matching mask_spec. Set resulting position as X,Y properties in out_pos prop list
 	C4MapScriptLayer *layer = _this->GetMapScriptLayer();
-	if (!layer) return NULL;
+	if (!layer) return false;
 	C4MapScriptMatTexMask mat_mask(mask_spec);
 	C4Rect search_rect;
-	if (!FnParRect(layer, rect, &search_rect)) return NULL;
+	if (!FnParRect(layer, rect, &search_rect)) return false;
 	int32_t x,y; bool result;
 	if (!max_tries) max_tries = 500;
-	if (result = layer->FindPos(search_rect, mat_mask, &x, &y, max_tries))
+	if ((result = layer->FindPos(search_rect, mat_mask, &x, &y, max_tries)))
 	{
 		if (out_pos && !out_pos->IsFrozen())
 		{
@@ -409,7 +409,7 @@ bool C4MapScriptLayer::Blit(const C4Rect &rcBounds, const C4MapScriptAlgo *algo)
 	uint8_t col;
 	for (int32_t y=rcBounds.y; y<rcBounds.y+rcBounds.Hgt; ++y)
 		for (int32_t x=rcBounds.x; x<rcBounds.x+rcBounds.Wdt; ++x)
-			if (col=(*algo)(x,y))
+			if ((col=(*algo)(x,y)))
 				surface->_SetPix(x,y,col);
 	return true;
 }
@@ -459,15 +459,15 @@ bool C4MapScriptLayer::FindPos(const C4Rect &search_rect, const C4MapScriptMatTe
 	// (this guantuess to find a pixel if there is one, but favours border pixels)
 	int32_t sx=search_rect.x + Random(search_rect.Wdt);
 	int32_t sy=search_rect.y + Random(search_rect.Hgt);
-	for (int32_t x=sx; x<surface->Wdt; ++x)
+	for (int32_t x = sx; x < search_rect.x + search_rect.Wdt; ++x)
 		if (col_mask(surface->_GetPix(x,sy))) { *out_x=x; *out_y=sy; return true; }
-	for (int32_t y=sy+1; y<surface->Hgt; ++y)
-		for (int32_t x=0; x<surface->Wdt; ++x)
+	for (int32_t y = sy + 1; y<search_rect.y + search_rect.Hgt; ++y)
+		for (int32_t x = search_rect.x; x < search_rect.x + search_rect.Wdt; ++x)
 			if (col_mask(surface->_GetPix(x,y))) { *out_x=x; *out_y=y; return true; }
-	for (int32_t y=0; y<sy; ++y)
-		for (int32_t x=0; x<surface->Wdt; ++x)
+	for (int32_t y = search_rect.y; y<sy; ++y)
+		for (int32_t x = search_rect.x; x < search_rect.x + search_rect.Wdt; ++x)
 			if (col_mask(surface->_GetPix(x,y))) { *out_x=x; *out_y=y; return true; }
-	for (int32_t x=0; x<sx; ++x)
+	for (int32_t x = search_rect.x; x<sx; ++x)
 		if (col_mask(surface->_GetPix(x,sy))) { *out_x=x; *out_y=sy; return true; }
 	// Nothing found
 	return false;
@@ -494,7 +494,7 @@ C4MapScriptLayer *C4MapScriptMap::CreateLayer(int32_t wdt, int32_t hgt)
 	return new_layer;
 }
 
-C4MapScriptHost::C4MapScriptHost(): LayerPrototype(NULL), MapPrototype(NULL) { }
+C4MapScriptHost::C4MapScriptHost(): LayerPrototype(NULL), MapPrototype(NULL), pTexMap(NULL), pMatMap(NULL) { }
 
 C4MapScriptHost::~C4MapScriptHost() { Clear(); }
 
@@ -550,6 +550,12 @@ bool C4MapScriptHost::Load(C4Group & g, const char * f, const char * l, C4LangSt
 	return C4ScriptHost::Load(g, f, l, t);
 }
 
+bool C4MapScriptHost::LoadData(const char * f, const char * d, C4LangStringTable * t)
+{
+	assert(LayerPrototype && MapPrototype);
+	return C4ScriptHost::LoadData(f, d, t);
+}
+
 void C4MapScriptHost::Clear()
 {
 	delete LayerPrototype; delete MapPrototype;
@@ -568,10 +574,12 @@ C4MapScriptMap *C4MapScriptHost::CreateMap()
 	return new C4MapScriptMap(MapPrototype);
 }
 
-bool C4MapScriptHost::InitializeMap(C4Group &group, CSurface8 **pmap_surface)
+bool C4MapScriptHost::InitializeMap(C4SLandscape *pLandscape, C4TextureMap *pTexMap, C4MaterialMap *pMatMap, uint32_t iPlayerCount, CSurface8 **pmap_surface)
 {
 	// Init scripted map by calling InitializeMap in the proper scripts. If *pmap_surface is given, it will pass the existing map to be modified by script.
 	assert(pmap_surface);
+	this->pTexMap = pTexMap;
+	this->pMatMap = pMatMap;
 	// Don't bother creating surfaces if the functions aren't defined
 	if (!LayerPrototype->GetFunc(PSF_InitializeMap))
 	{
@@ -590,7 +598,7 @@ bool C4MapScriptHost::InitializeMap(C4Group &group, CSurface8 **pmap_surface)
 	{
 		// No existing map. Create new.
 		int32_t map_wdt,map_hgt;
-		::Game.C4S.Landscape.GetMapSize(map_wdt, map_hgt, ::Game.StartupPlayerCount);
+		pLandscape->GetMapSize(map_wdt, map_hgt, iPlayerCount);
 		if (!map->CreateSurface(map_wdt, map_hgt)) return false;
 	}
 	C4AulParSet Pars(C4VPropList(map.get()));
@@ -605,4 +613,4 @@ bool C4MapScriptHost::InitializeMap(C4Group &group, CSurface8 **pmap_surface)
 	return !!result;
 }
 
-C4MapScriptHost MapScript;
\ No newline at end of file
+C4MapScriptHost MapScript;
