@@ -125,10 +125,10 @@ bool C4MaterialShape::Load(C4Group &group, const char *filename)
 			{
 				min = max = *j;
 			}
-			if (j ->x<- overlap_left     ) overlap_left   =- j ->x;
-			if (j ->y<- overlap_top      ) overlap_top    =- j ->y;
-			if (j ->x> wdt+overlap_right ) overlap_right  =  j ->x- wdt;
-			if (j ->y> hgt+overlap_bottom) overlap_bottom =  j ->y- hgt;
+			if (j ->x<- overlap_left     ) overlap_left   = -j->x;
+			if (j ->y<- overlap_top      ) overlap_top    = -j->y;
+			if (j ->x> wdt+overlap_right ) overlap_right  =  j->x - wdt;
+			if (j ->y> hgt+overlap_bottom) overlap_bottom =  j->y - hgt;
 		}
 		center.x /= n; center.y /= n;
 		i->center = center; i->min = min; i->max = max;
@@ -253,6 +253,7 @@ void C4MaterialCore::Clear()
 	Corrode = 0;
 	Soil = 0;
 	Placement = 0;
+	Light = 0;
 	OverlayType = 0;
 	PXSGfxRt.Default();
 	PXSGfxSize = 0;
@@ -265,6 +266,7 @@ void C4MaterialCore::Clear()
 	TempConvStrength = 0;
 	MinHeightCount = 0;
 	SplashRate=10;
+	KeepSinglePixels=false;
 }
 
 void C4MaterialCore::Default()
@@ -339,6 +341,7 @@ void C4MaterialCore::CompileFunc(StdCompiler *pComp)
 	pComp->Value(mkNamingAdapt(Extinguisher,        "Extinguisher",        0));
 	pComp->Value(mkNamingAdapt(Soil,                "Soil",                0));
 	pComp->Value(mkNamingAdapt(Placement,           "Placement",           0));
+	pComp->Value(mkNamingAdapt(Light,               "Light",               0));
 	pComp->Value(mkNamingAdapt(mkParAdapt(sTextureOverlay, StdCompiler::RCT_IdtfAllowEmpty),
 	                                                "TextureOverlay",      ""));
 	pComp->Value(mkNamingAdapt(OverlayType,         "OverlayType",         0));
@@ -364,6 +367,7 @@ void C4MaterialCore::CompileFunc(StdCompiler *pComp)
 	                                                "BelowTempConvertTo",  ""));
 	pComp->Value(mkNamingAdapt(MinHeightCount,      "MinHeightCount",      0));
 	pComp->Value(mkNamingAdapt(SplashRate,          "SplashRate",          10));
+	pComp->Value(mkNamingAdapt(KeepSinglePixels,    "KeepSinglePixels",    false));
 	pComp->NameEnd();
 	// material reactions
 	pComp->Value(mkNamingAdapt(mkSTLContainerAdapt(CustomReactionList),
@@ -585,9 +589,12 @@ bool C4MaterialMap::CrossMapMaterials(const char* szEarthMaterial) // Called aft
 			{
 				// single material target
 				if (pReact->fInverseSpec)
-					for (int32_t cnt2=-1; cnt2<Num; cnt2++) if (cnt2!=tmat) SetMatReaction(cnt, cnt2, pReact);
+					for (int32_t cnt2=-1; cnt2<Num; cnt2++) {
+						if (cnt2!=tmat)
+							SetMatReaction(cnt, cnt2, pReact);
 						else
 							SetMatReaction(cnt, tmat, pReact);
+					}
 			}
 			else if (SEqualNoCase(pReact->TargetSpec.getData(), "All"))
 			{
@@ -664,14 +671,6 @@ bool C4MaterialMap::CrossMapMaterials(const char* szEarthMaterial) // Called aft
 		if (Map[cnt].sAboveTempConvertTo.getLength())
 			Map[cnt].AboveTempConvertTo=::TextureMap.GetIndexMatTex(Map[cnt].sAboveTempConvertTo.getData(), NULL, true, FormatString("AboveTempConvertTo of mat %s", Map[cnt].Name).getData());
 	}
-#if 0
-	int32_t i=0;
-	while (ReactionFuncMap[i].szRFName) {printf("%s: %p\n", ReactionFuncMap[i].szRFName, ReactionFuncMap[i].pFunc); ++i;}
-	for (int32_t cnt=-1; cnt<Num; cnt++)
-		for (int32_t cnt2=-1; cnt2<Num; cnt2++)
-			if (ppReactionMap[(cnt2+1)*(Num+1) + cnt+1])
-				printf("%s -> %s: %p\n", Map[cnt].Name, Map[cnt2].Name, ppReactionMap[(cnt2+1)*(Num+1) + cnt+1]->pFunc);
-#endif
 
 	// Get hardcoded system material indices
 	const C4TexMapEntry* earth_entry = ::TextureMap.GetEntry(::TextureMap.GetIndexMatTex(szEarthMaterial));
@@ -886,7 +885,7 @@ bool C4MaterialMap::mrfPoof(C4MaterialReaction *pReaction, int32_t &iX, int32_t
 	{
 	case meeMassMove: // MassMover-movement
 	case meePXSPos: // PXS check before movement: Kill both landscape and PXS mat
-		::Landscape.ExtractMaterial(iLSPosX,iLSPosY);
+		::Landscape.ExtractMaterial(iLSPosX,iLSPosY,false);
 		if (!Random(3)) Smoke(iX,iY,3);
 		if (!Random(3)) StartSoundEffectAt("Pshshsh", iX, iY);
 		return true;
@@ -898,7 +897,7 @@ bool C4MaterialMap::mrfPoof(C4MaterialReaction *pReaction, int32_t &iX, int32_t
 				// either splash or slide prevented interaction
 				return false;
 		// Always kill both landscape and PXS mat
-		::Landscape.ExtractMaterial(iLSPosX,iLSPosY);
+		::Landscape.ExtractMaterial(iLSPosX,iLSPosY,false);
 		if (!Random(3)) Smoke(iX,iY,3);
 		if (!Random(3)) StartSoundEffectAt("Pshshsh", iX, iY);
 		return true;
