@@ -23,6 +23,7 @@
 
 #ifndef USE_CONSOLE
 // headers for particle excution
+#include <C4Application.h>
 #include <C4AulDefFunc.h>
 #include <C4Value.h>
 #include <C4ValueArray.h>
@@ -30,7 +31,7 @@
 #include <C4DrawGL.h>
 #include <C4Random.h>
 #include <C4Landscape.h>
-#include <C4Weather.h>
+#include <C4Weather.h>	
 #endif
 
 
@@ -371,6 +372,15 @@ void C4ParticleValueProvider::Floatify(float denominator)
 	{
 		FloatifyParameterValue(&C4ParticleValueProvider::speedFactor, 1000.0f);
 	}
+	else if (valueFunction == &C4ParticleValueProvider::Step)
+	{
+		FloatifyParameterValue(&C4ParticleValueProvider::maxValue, denominator);
+	}
+	else if (valueFunction == &C4ParticleValueProvider::Sin)
+	{
+		FloatifyParameterValue(&C4ParticleValueProvider::parameterValue, 1.0f);
+		FloatifyParameterValue(&C4ParticleValueProvider::maxValue, denominator);
+	}
 }
 
 void C4ParticleValueProvider::RollRandom()
@@ -419,7 +429,9 @@ float C4ParticleValueProvider::Direction(C4Particle *forParticle)
 
 float C4ParticleValueProvider::Step(C4Particle *forParticle)
 {
-	return currentValue + startValue * forParticle->GetAge() / delay;
+	float value = currentValue + startValue * forParticle->GetAge() / delay;
+	if (maxValue != 0.0f && value > maxValue) value = maxValue;
+	return value;
 }
 
 float C4ParticleValueProvider::KeyFrames(C4Particle *forParticle)
@@ -448,6 +460,11 @@ float C4ParticleValueProvider::KeyFrames(C4Particle *forParticle)
 	return startValue;
 }
 
+float C4ParticleValueProvider::Sin(C4Particle *forParticle)
+{
+	return sin(parameterValue * M_PI / 180.0f) * maxValue + startValue;
+}
+
 float C4ParticleValueProvider::Speed(C4Particle *forParticle)
 {
 	float distX = forParticle->currentSpeedX;
@@ -489,6 +506,9 @@ void C4ParticleValueProvider::SetType(C4ParticleValueProviderID what)
 	case C4PV_KeyFrames:
 		valueFunction = &C4ParticleValueProvider::KeyFrames;
 		break;
+	case C4PV_Sin:
+		valueFunction = &C4ParticleValueProvider::Sin;
+		break;
 	case C4PV_Speed:
 		valueFunction = &C4ParticleValueProvider::Speed;
 		break;
@@ -555,12 +575,13 @@ void C4ParticleValueProvider::Set(const C4ValueArray &fromArray)
 		}
 		break;
 	case C4PV_Step:
-		if (arraySize >= 2)
+		if (arraySize >= 4)
 		{
 			SetType(C4PV_Step);
 			SetParameterValue(VAL_TYPE_FLOAT, fromArray[1], &C4ParticleValueProvider::startValue);
 			SetParameterValue(VAL_TYPE_FLOAT, fromArray[2], &C4ParticleValueProvider::currentValue);
 			SetParameterValue(VAL_TYPE_FLOAT, fromArray[3], &C4ParticleValueProvider::delay);
+			SetParameterValue(VAL_TYPE_FLOAT, fromArray[4], &C4ParticleValueProvider::maxValue);
 			if (delay == 0.f) delay = 1.f;
 		}
 		break;
@@ -593,6 +614,15 @@ void C4ParticleValueProvider::Set(const C4ValueArray &fromArray)
 			//	LogF("KF is %f @ %d of %d", keyFrames[i * 2 + 1], int(keyFrames[i * 2]), keyFrameCount);
 		}
 		break;
+	case C4PV_Sin:
+		if (arraySize >= 3)
+		{
+			SetType(C4PV_Sin); // Sin(parameterValue) * maxValue + startValue
+			SetParameterValue(VAL_TYPE_FLOAT, fromArray[1], &C4ParticleValueProvider::parameterValue);
+			SetParameterValue(VAL_TYPE_FLOAT, fromArray[2], &C4ParticleValueProvider::maxValue);
+			SetParameterValue(VAL_TYPE_FLOAT, fromArray[3], &C4ParticleValueProvider::startValue);
+		}
+		break;
 	case C4PV_Speed:
 		if (arraySize >= 3)
 		{
@@ -942,16 +972,14 @@ bool C4ParticleChunk::Exec(C4Object *obj, float timeDelta)
 #define glDeleteVertexArrays glDeleteVertexArraysAPPLE
 #endif
 
-void C4ParticleChunk::Draw(C4TargetFacet cgo, C4Object *obj)
+void C4ParticleChunk::Draw(C4TargetFacet cgo, C4Object *obj, int texUnit)
 {
 	if (particleCount == 0) return;
 	const int stride = sizeof(C4Particle::DrawingData::Vertex);
 	assert(sourceDefinition && "No source definition assigned to particle chunk.");
-	C4TexRef *textureRef = (*sourceDefinition->Gfx.GetFace().ppTex);
+	C4TexRef *textureRef = &sourceDefinition->Gfx.GetFace().textures[0];
 	assert(textureRef != 0 && "Particle definition had no texture assigned.");
 
-	
-
 	// use a relative offset?
 	bool resetMatrix(false);
 	if ((attachment & C4ATTACH_MoveRelative) && (obj != 0))
@@ -961,10 +989,10 @@ void C4ParticleChunk::Draw(C4TargetFacet cgo, C4Object *obj)
 		glTranslatef((float)obj->GetX(), (float)obj->GetY(), 0.0f);
 	}
 
+	// enable additive blending for particles with that blit mode
 	glBlendFunc(GL_SRC_ALPHA, (blitMode & C4GFXBLIT_ADDITIVE) ? GL_ONE : GL_ONE_MINUS_SRC_ALPHA);
 
-	glActiveTexture(GL_TEXTURE0);
-	glClientActiveTexture(GL_TEXTURE0);
+	glActiveTexture(texUnit);
 	glBindTexture(GL_TEXTURE_2D, textureRef->texName);
 
 	if (!Particles.useBufferObjectWorkaround)
@@ -1090,18 +1118,10 @@ void C4ParticleList::Exec(float timeDelta)
 
 	accessMutex.Enter();
 
-	for (std::list<C4ParticleChunk*>::iterator iter = particleChunks.begin(); iter != particleChunks.end();)
+	for (std::list<C4ParticleChunk*>::iterator iter = particleChunks.begin(); iter != particleChunks.end();++iter)
 	{
 		C4ParticleChunk *chunk = *iter;
-		if (chunk->Exec(targetObject, timeDelta))
-		{
-			++iter;
-		}
-		else
-		{
-			iter = particleChunks.erase(iter);
-			lastAccessedChunk = 0;
-		}
+		chunk->Exec(targetObject, timeDelta);
 	}
 
 	accessMutex.Leave();
@@ -1117,21 +1137,25 @@ void C4ParticleList::Draw(C4TargetFacet cgo, C4Object *obj)
 	pDraw->DeactivateBlitModulation();
 	pDraw->ResetBlitMode();
 	
-	glEnable(GL_TEXTURE_2D);
-
 	if (!Particles.usePrimitiveRestartIndexWorkaround)
 	{
 		glPrimitiveRestartIndex(0xffffffff);
 		glEnable(GL_PRIMITIVE_RESTART);
 	}
-	// apply zoom
-	glMatrixMode(GL_MODELVIEW);
-	glPushMatrix();
-	glLoadIdentity();
 
-	glTranslatef(cgo.X, cgo.Y, 0.0f);
-	glScalef(cgo.Zoom, cgo.Zoom, 1.0f);
-	glTranslatef(-cgo.TargetX, -cgo.TargetY, 0.0f);
+	// enable shader
+	C4ShaderCall call(pGL->GetSpriteShader(true, false, false));
+	// apply zoom and upload shader uniforms
+	pGL->SetupMultiBlt(call, NULL, 0, 0, 0, 0);
+	// go to correct output position
+	glTranslatef(cgo.X-cgo.TargetX, cgo.Y-cgo.TargetY, 0.0f);
+	// allocate texture unit for particle texture, and remember allocated
+	// texture unit. Will be used for each particle chunk to bind
+	// their texture to this unit.
+	const GLint texUnit = call.AllocTexUnit(C4SSU_BaseTex, GL_TEXTURE_2D);
+	// Texture coordinates are always associated to texture unit 0, since
+	// there is only one set of texture coordinates
+	glClientActiveTexture(GL_TEXTURE0);
 
 	if (Particles.useBufferObjectWorkaround)
 	{
@@ -1142,13 +1166,25 @@ void C4ParticleList::Draw(C4TargetFacet cgo, C4Object *obj)
 
 	accessMutex.Enter();
 
-	for (std::list<C4ParticleChunk*>::iterator iter = particleChunks.begin(); iter != particleChunks.end(); ++iter)
+	for (std::list<C4ParticleChunk*>::iterator iter = particleChunks.begin(); iter != particleChunks.end(); )
 	{
-		(*iter)->Draw(cgo, obj);
+		if ((*iter)->IsEmpty())
+		{
+			delete *iter;
+			iter = particleChunks.erase(iter);
+			lastAccessedChunk = 0;
+		}
+		else
+		{
+			(*iter)->Draw(cgo, obj, texUnit);
+			++iter;
+		}
 	}
 
 	accessMutex.Leave();
 
+	pGL->ResetMultiBlt();
+
 	if (Particles.useBufferObjectWorkaround)
 	{
 		glDisableClientState(GL_VERTEX_ARRAY);
@@ -1156,13 +1192,10 @@ void C4ParticleList::Draw(C4TargetFacet cgo, C4Object *obj)
 		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
 	}
 
-	glPopMatrix();
-
 	if (!Particles.usePrimitiveRestartIndexWorkaround)
 	{
 		glDisable(GL_PRIMITIVE_RESTART);
 	}
-	glDisable(GL_TEXTURE_2D);
 }
 
 void C4ParticleList::Clear()
@@ -1244,7 +1277,7 @@ C4ParticleSystem::~C4ParticleSystem()
 	CalculateNextStep();
 
 	for (std::vector<uint32_t *>::iterator iter = multiDrawElementsIndexArray.begin(); iter != multiDrawElementsIndexArray.end(); ++iter)
-		delete (*iter);
+		delete[] (*iter);
 }
 
 void C4ParticleSystem::DoInit()
@@ -1258,6 +1291,12 @@ void C4ParticleSystem::DoInit()
 
 	assert (glGenBuffers != 0 && "Your graphics card does not seem to support buffer objects.");
 	useBufferObjectWorkaround = false;
+
+	// Every window in developers' mode has an own OpenGL context at the moment. Certain objects are not shared between contexts.
+	// In that case we can just use the slower workaround without VBAs and VBOs to allow the developer to view particles in every viewport.
+	// The best solution would obviously be to make all windows use a single OpenGL context. This has to be considered as a workaround.
+	if (Application.isEditor)
+		useBufferObjectWorkaround = true;
 }
 
 void C4ParticleSystem::ExecuteCalculation()
@@ -1283,6 +1322,8 @@ void C4ParticleSystem::ExecuteCalculation()
 		particleListAccessMutex.Leave();
 	}
 }
+#else // ifdef USE_CONSOLE
+void C4ParticleSystem::DoInit() {}
 #endif
 
 C4ParticleList *C4ParticleSystem::GetNewParticleList(C4Object *forObject)
@@ -1497,9 +1538,11 @@ void C4ParticleSystem::Clear()
 
 void C4ParticleSystem::ClearAllParticles()
 {
+#ifndef USE_CONSOLE
 	particleListAccessMutex.Enter();
 	particleLists.clear();
 	particleListAccessMutex.Leave();
+#endif
 }
 
 C4ParticleDef *C4ParticleSystemDefinitionList::GetDef(const char *name, C4ParticleDef *exclude)
@@ -1514,12 +1557,10 @@ C4ParticleDef *C4ParticleSystemDefinitionList::GetDef(const char *name, C4Partic
 	return 0;
 }
 
-#ifndef USE_CONSOLE
 void C4ParticleSystemDefinitionList::Clear()
 {
 	// the particle definitions update the list in their destructor
 	while (first)
 		delete first;
 }
-#endif
 C4ParticleSystem Particles;
