@@ -26,7 +26,6 @@
 #include <cstring>
 #include <cassert>
 #include <cstdarg>
-#include <utility>
 #include <algorithm>
 
 // debug memory management
@@ -668,7 +667,7 @@ public:
 	// get an indexed section from the string like Section1;Section2;Section3
 	bool GetSection(size_t idx, StdStrBuf *psOutSection, char cSeparator=';') const;
 
-	// Checks wether the contents are valid UTF-8, and if not, convert them from windows-1252 to UTF-8.
+	// Checks whether the content is valid UTF-8, and if not, convert it from windows-1252 to UTF-8.
 	void EnsureUnicode();
 
 	// convert to lower case
@@ -736,16 +735,6 @@ public:
 
 };
 
-#if 0
-// const char* + StdStrBuf
-inline StdStrBuf operator + (const char* szString, const StdStrBuf& Buf2)
-{
-	StdStrBuf Buf(szString);
-	Buf.Append(Buf2);
-	return Buf;
-}
-#endif
-
 // Wrappers
 extern StdStrBuf FormatString(const char *szFmt, ...) GNUC_FORMAT_ATTRIBUTE;
 extern StdStrBuf FormatStringV(const char *szFmt, va_list args);
