@@ -86,6 +86,7 @@ public:
 	// for whatever reason (suppress warning messages).
 	virtual bool Name(const char *szName)         { return true; }
 	virtual void NameEnd(bool fBreak = false)     { }
+	virtual const char *GetNameByIndex(size_t idx) const { return NULL; }
 
 	// Special: A naming that follows to the currently active naming (on the same level).
 	// Note this will end the current naming, so no additional NameEnd() is needed.
@@ -627,7 +628,7 @@ public:
 
 	// Input
 	typedef StdStrBuf InT;
-	void setInput(const InT &In) { Buf.Ref(In); }
+	void setInput(const InT &In) { Buf.Ref(In); lineBreaks.clear(); }
 
 	// Properties
 	virtual bool isCompiler() { return true; }
@@ -637,6 +638,7 @@ public:
 	virtual bool Name(const char *szName);
 	virtual void NameEnd(bool fBreak = false);
 	virtual bool FollowName(const char *szName);
+	virtual const char *GetNameByIndex(size_t idx) const;
 
 	// Separators
 	virtual bool Separator(Sep eSep);
@@ -684,9 +686,10 @@ protected:
 		// Name number in parent map
 		const char *Pos;
 		// Constructor
-		NameNode(NameNode *pParent = NULL)
-				: Parent(pParent), FirstChild(NULL), PrevChild(NULL), NextChild(NULL), LastChild(NULL),
-				Indent(-1), Pos(NULL), Section(false)
+		NameNode(NameNode *pParent = NULL) :
+			Section(false), Parent(pParent),
+			FirstChild(NULL), PrevChild(NULL), NextChild(NULL), LastChild(NULL),
+			Indent(-1), Pos(NULL)
 		{ }
 	};
 	NameNode *pNameRoot, *pName;
@@ -725,6 +728,9 @@ protected:
 
 	void notFound(const char *szWhat);
 
+private:
+	uint32_t getLineNumberOfPos(const char *pos) const;
+	mutable std::vector<const char *> lineBreaks;
 };
 
 void StdCompilerWarnCallback(void *pData, const char *szPosition, const char *szError);
