@@ -26,6 +26,7 @@
 #include <C4PlayerList.h>
 #include <C4GameControl.h>
 #include <C4GraphicsResource.h>
+#include <C4Startup.h>
 
 #ifndef HAVE_WINSOCK
 #include <sys/socket.h>
@@ -180,9 +181,9 @@ void C4Network2ClientListBox::ClientListItem::Update()
 		int iWait = ::Control.Network.ClientPerfStat(iClientID);
 		pPing->SetText(FormatString("%d ms", iWait).getData());
 		pPing->SetColor(C4RGB(
-		                  BoundBy(255-Abs(iWait)*5, 0, 255),
-		                  BoundBy(255-iWait*5, 0, 255),
-		                  BoundBy(255+iWait*5, 0, 255)));
+		                  Clamp(255-Abs(iWait)*5, 0, 255),
+		                  Clamp(255-iWait*5, 0, 255),
+		                  Clamp(255+iWait*5, 0, 255)));
 	}
 	// update activation status
 	const C4Client *pClient = GetClient(); if (!pClient) return;
@@ -419,6 +420,11 @@ C4Network2ClientListBox::C4Network2ClientListBox(C4Rect &rcBounds, bool fStartup
 	Update();
 }
 
+C4Network2ClientListBox::~C4Network2ClientListBox()
+{
+	Application.Remove(this);
+}
+
 void C4Network2ClientListBox::Update()
 {
 	// sync with client list
@@ -506,7 +512,7 @@ C4Network2ClientListDlg::C4Network2ClientListDlg()
 	C4GUI::ComponentAligner caAll(GetContainedClientRect(), 0,0);
 	C4Rect rcStatus = caAll.GetFromBottom(pUseFont->GetLineHeight());
 	// create game options; max 1/2 of dialog height
-	pGameOptions = new C4GameOptionsList(caAll.GetFromTop(caAll.GetInnerHeight()/2), true, true);
+	pGameOptions = new C4GameOptionsList(caAll.GetFromTop(caAll.GetInnerHeight()/2), true, C4GameOptionsList::GOLS_Runtime);
 	pGameOptions->SetDecoration(false, NULL, true, false);
 	pGameOptions->SetSelectionDiabled();
 	// but resize to actually used height
@@ -523,6 +529,11 @@ C4Network2ClientListDlg::C4Network2ClientListDlg()
 	Update();
 }
 
+C4Network2ClientListDlg::~C4Network2ClientListDlg()
+{
+	if (this==pInstance) pInstance=NULL; Application.Remove(this);
+}
+
 void C4Network2ClientListDlg::Update()
 {
 	// Compose status text
@@ -610,8 +621,8 @@ C4GameOptionButtons::C4GameOptionButtons(const C4Rect &rcBounds, bool fNetwork,
 	}
 	else btnInternet = NULL;
 	bool fIsLeague = false;
-	// League button disabled (#479, re-enable when an OC league exists)
-	if (0 && fNetwork)
+	// League button
+	if (fNetwork)
 	{
 		C4GUI::Icons eLeagueIcon;
 		fIsLeague = fLobby ? Game.Parameters.isLeague() : !!Config.Network.LeagueServerSignUp;
@@ -675,12 +686,15 @@ void C4GameOptionButtons::OnBtnLeague(C4GUI::Control *btn)
 	btnRecord->SetEnabled(!fCheck);
 	// if the league is turned on, the game must be signed up at the masterserver
 	if (fCheck && !Config.Network.MasterServerSignUp) OnBtnInternet(btnInternet);
+	// refresh options in scenario selection dialogue
+	if (C4Startup::Get()) C4Startup::Get()->OnLeagueOptionChanged();
 }
 
 void C4GameOptionButtons::OnBtnRecord(C4GUI::Control *btn)
 {
-	bool fCheck = Config.General.DefRec = Game.Record = !Game.Record;
-	btnRecord->SetIcon(fCheck ? C4GUI::Ico_Ex_RecordOn : C4GUI::Ico_Ex_RecordOff);
+	Game.Record = !Game.Record;
+	Config.General.DefRec = Game.Record;
+	btnRecord->SetIcon(Game.Record ? C4GUI::Ico_Ex_RecordOn : C4GUI::Ico_Ex_RecordOff);
 }
 
 void C4GameOptionButtons::OnBtnPassword(C4GUI::Control *btn)
