@@ -19,6 +19,8 @@
 
 #include <C4Include.h>
 #include <C4Def.h>
+#include <C4DrawGL.h>
+#include <C4GraphicsResource.h>
 
 #include <C4Components.h>
 #include <C4Config.h>
@@ -28,10 +30,67 @@
 #include <C4RankSystem.h>
 #include <C4SoundSystem.h>
 #include <C4SolidMask.h>
+#include <CSurface8.h>
+
+// Helper class to load additional resources required for meshes from
+// a C4Group.
+class C4DefAdditionalResourcesLoader: public StdMeshMaterialLoader
+{
+public:
+	C4DefAdditionalResourcesLoader(C4Group& hGroup): Group(hGroup) {}
+
+	virtual C4Surface* LoadTexture(const char* filename)
+	{
+		if (!Group.AccessEntry(filename)) return NULL;
+		C4Surface* surface = new C4Surface;
+		// Suppress error message here, StdMeshMaterial loader
+		// will show one.
+		if (!surface->Read(Group, GetExtension(filename)))
+			{ delete surface; surface = NULL; }
+		return surface;
+	}
+
+	virtual StdStrBuf LoadShaderCode(const char* filename)
+	{
+		StdStrBuf ret;
+		if (!Group.LoadEntryString(filename, &ret)) return StdStrBuf();
+		return ret;
+	}
+
+	virtual void AddShaderSlices(C4Shader& shader, int ssc)
+	{
+		// Add mesh-independent slices
+		shader.AddFragmentSlice(-1, "#define OPENCLONK");
+		shader.AddVertexSlice(-1, "#define OPENCLONK");
+
+		if (ssc & C4SSC_MOD2) shader.AddFragmentSlice(-1, "#define CLRMOD_MOD2");
+		if (ssc & C4SSC_LIGHT) shader.AddFragmentSlice(-1, "#define HAVE_LIGHT");
+
+		shader.LoadSlices(&::GraphicsResource.Files, "UtilShader.glsl");
+		shader.LoadSlices(&::GraphicsResource.Files, "ObjectBaseShader.glsl");
+		shader.LoadSlices(&::GraphicsResource.Files, "MeshShader.glsl");
+
+		// Note that these shader slices are always loaded, even if lighting
+		// is disabled. The shaders then assume a default light if HAVE_LIGHT
+		// is not defined. This avoids completely flat shading for meshes
+		// that are shown as picture graphics for example.
+		shader.LoadSlices(&::GraphicsResource.Files, "ObjectLightShader.glsl");
+		shader.LoadSlices(&::GraphicsResource.Files, "LightShader.glsl");
+		shader.LoadSlices(&::GraphicsResource.Files, "AmbientShader.glsl");
+
+		if (ssc & C4SSC_BASE) shader.LoadSlices(&::GraphicsResource.Files, "SpriteTextureShader.glsl");
+		if (ssc & C4SSC_OVERLAY) shader.LoadSlices(&::GraphicsResource.Files, "SpriteOverlayShader.glsl");
+	}
+
+private:
+	C4Group& Group;
+};
+
+
 
 void C4Def::DefaultDefCore()
 {
-	rC4XVer[0]=rC4XVer[1]=rC4XVer[2]=0;
+	rC4XVer[0]=rC4XVer[1]=0;
 	RequireDef.Clear();
 	Shape.Default();
 	Entrance.Default();
@@ -41,7 +100,6 @@ void C4Def::DefaultDefCore()
 	TopFace.Default();
 	Component.Default();
 	BurnTurnTo=C4ID::None;
-	BuildTurnTo=C4ID::None;
 	GrowthType=0;
 	CrewMember=0;
 	NativeCrew=0;
@@ -58,7 +116,6 @@ void C4Def::DefaultDefCore()
 	BorderBound=0;
 	LiftTop=0;
 	GrabPutGet=0;
-	ContainBlast=0;
 	UprightAttach=0;
 	ContactFunctionCalls=0;
 	Line=0;
@@ -95,7 +152,7 @@ bool C4Def::LoadDefCore(C4Group &hGroup)
 		Source.Clear();
 
 		// Let's be bold: Rewrite, with current version
-		/*rC4XVer[0] = C4XVER1; rC4XVer[1] = C4XVER2; rC4XVer[2] = C4XVER3;
+		/*rC4XVer[0] = C4XVER1; rC4XVer[1] = C4XVER2;
 		hGroup.Rename(C4CFN_DefCore, C4CFN_DefCore ".old");
 		Save(hGroup);*/
 
@@ -139,6 +196,7 @@ void C4Def::CompileFunc(StdCompiler *pComp)
 	const StdBitfieldEntry<int32_t> Categories[] =
 	{
 
+		{ "C4D_None",                     C4D_None                },
 		{ "C4D_StaticBack",               C4D_StaticBack          },
 		{ "C4D_Structure",                C4D_Structure           },
 		{ "C4D_Vehicle",                  C4D_Vehicle             },
@@ -179,7 +237,6 @@ void C4Def::CompileFunc(StdCompiler *pComp)
 	pComp->Value(mkNamingAdapt(CrewMember,                    "CrewMember",         0                 ));
 	pComp->Value(mkNamingAdapt(NativeCrew,                    "NoStandardCrew",     0                 ));
 	pComp->Value(mkNamingAdapt(Constructable,                 "Construction",       0                 ));
-	pComp->Value(mkNamingAdapt(BuildTurnTo,                   "ConstructTo",        C4ID::None        ));
 
 	const StdBitfieldEntry<int32_t> GrabPutGetTypes[] =
 	{
@@ -196,7 +253,6 @@ void C4Def::CompileFunc(StdCompiler *pComp)
 	pComp->Value(mkNamingAdapt(Rotateable,                    "Rotate",             0                 ));
 	pComp->Value(mkNamingAdapt(RotatedEntrance,               "RotatedEntrance",    0                 ));
 	pComp->Value(mkNamingAdapt(Float,                         "Float",              0                 ));
-	pComp->Value(mkNamingAdapt(ContainBlast,                  "ContainBlast",       0                 ));
 	pComp->Value(mkNamingAdapt(ColorByOwner,                  "ColorByOwner",       0                 ));
 	pComp->Value(mkNamingAdapt(NoHorizontalMove,              "HorizontalFix",      0                 ));
 	pComp->Value(mkNamingAdapt(BorderBound,                   "BorderBound",        0                 ));
@@ -290,12 +346,11 @@ void C4Def::Clear()
 }
 
 bool C4Def::Load(C4Group &hGroup,
+				 StdMeshSkeletonLoader &loader,
                  DWORD dwLoadWhat,
                  const char *szLanguage,
                  C4SoundSystem *pSoundSystem)
 {
-	bool fSuccess=true;
-
 	bool AddFileMonitoring = false;
 	if (Game.pFileMonitor && !SEqual(hGroup.GetFullName().getData(),Filename) && !hGroup.IsPacked())
 		AddFileMonitoring = true;
@@ -309,11 +364,90 @@ bool C4Def::Load(C4Group &hGroup,
 
 	if (AddFileMonitoring) Game.pFileMonitor->AddDirectory(Filename);
 
+	// Pre-read all images and shader stuff because they ar eaccessed in unpredictable order during loading
+	hGroup.PreCacheEntries(C4CFN_ShaderFiles);
+	hGroup.PreCacheEntries(C4CFN_ImageFiles);
+
+	LoadMeshMaterials(hGroup);
+	bool fSuccess = LoadParticleDef(hGroup);
+
+	// Read DefCore
+	if (fSuccess) fSuccess = LoadDefCore(hGroup);
+
+	// Skip def: don't even read sounds!
+	if (fSuccess && Game.C4S.Definitions.SkipDefs.GetIDCount(id, 1)) return false;
+
+	// Read sounds, even if not a valid def (for pure ocd sound folders)
+	if (dwLoadWhat & C4D_Load_Sounds) LoadSounds(hGroup, pSoundSystem);
+
+	// cancel if not a valid definition
+	if (!fSuccess) return false;
+
+	// Read and parse SolidMask bitmap
+	if (!LoadSolidMask(hGroup)) return false;
+
+	// Read surface bitmap, meshes, skeletons
+	if ((dwLoadWhat & C4D_Load_Bitmap) && !LoadGraphics(hGroup, loader)) return false;
+
+	// Read string table
+	C4Language::LoadComponentHost(&StringTable, hGroup, C4CFN_ScriptStringTbl, szLanguage);
+
+	// Register ID with script engine
+	::ScriptEngine.RegisterGlobalConstant(id.ToString(), C4VPropList(this));
+	ParentKeyName = ::Strings.RegString(id.ToString());
+
+	// Read script
+	if (dwLoadWhat & C4D_Load_Script) LoadScript(hGroup, szLanguage);
+
+	// Read clonknames
+	if (dwLoadWhat & C4D_Load_ClonkNames) LoadClonkNames(hGroup, pClonkNames, szLanguage);
+
+	// Read clonkranks
+	if (dwLoadWhat & C4D_Load_RankNames) LoadRankNames(hGroup, szLanguage);
+
+	// Read rankfaces
+	if (dwLoadWhat & C4D_Load_RankFaces) LoadRankFaces(hGroup);
+
+	// Temporary flag
+	if (dwLoadWhat & C4D_Load_Temporary) Temporary=true;
+
+	return true;
+}
+
+void C4Def::LoadMeshMaterials(C4Group &hGroup)
+{
+	// Load all mesh materials from this folder
+	C4DefAdditionalResourcesLoader loader(hGroup);
+	hGroup.ResetSearch();
+	char MaterialFilename[_MAX_PATH + 1]; *MaterialFilename = 0;
+	while (hGroup.FindNextEntry(C4CFN_DefMaterials, MaterialFilename, NULL, !!*MaterialFilename))
+	{
+		StdStrBuf material;
+		if (hGroup.LoadEntryString(MaterialFilename, &material))
+		{
+			try
+			{
+				StdStrBuf buf;
+				buf.Copy(hGroup.GetName());
+				buf.Append("/"); buf.Append(MaterialFilename);
+				::MeshMaterialManager.Parse(material.getData(), buf.getData(), loader);
+			}
+			catch (const StdMeshMaterialError& ex)
+			{
+				DebugLogF("Failed to read material script: %s", ex.what());
+			}
+		}
+	}
+}
+
+bool C4Def::LoadParticleDef(C4Group &hGroup)
+{
+	bool fSuccess = true;
 	// particle def?
 	if (hGroup.AccessEntry(C4CFN_ParticleCore))
 	{
 		// def loading not successful; abort after reading sounds
-		fSuccess=false;
+		fSuccess = false;
 		// create new particle def
 		C4ParticleDef *pParticleDef = new C4ParticleDef();
 		// load it
@@ -324,26 +458,11 @@ bool C4Def::Load(C4Group &hGroup,
 		}
 		// done
 	}
+	return fSuccess;
+}
 
-
-	// Read DefCore
-	if (fSuccess) fSuccess=LoadDefCore(hGroup);
-
-	// skip def: don't even read sounds!
-	if (fSuccess && Game.C4S.Definitions.SkipDefs.GetIDCount(id, 1)) return false;
-
-	if (!fSuccess)
-	{
-
-		// Read sounds even if not a valid def (for pure ocd sound folders)
-		if (dwLoadWhat & C4D_Load_Sounds)
-			if (pSoundSystem)
-				pSoundSystem->LoadEffects(hGroup);
-
-		return false;
-	}
-
-	// Read and parse SolidMask bitmap
+bool C4Def::LoadSolidMask(C4Group &hGroup)
+{
 	if (hGroup.FindEntry(C4CFN_SolidMask))
 	{
 		pSolidMask = C4SolidMask::LoadMaskFromFile(hGroup, C4CFN_SolidMask);
@@ -352,125 +471,40 @@ bool C4Def::Load(C4Group &hGroup,
 			DebugLogF("  Error loading SolidMask of %s (%s)", hGroup.GetFullName().getData(), id.ToString());
 			return false;
 		}
+		// check SolidMask size
+		if (SolidMask.x<0 || SolidMask.y<0 || SolidMask.x + SolidMask.Wdt>pSolidMask->Wdt || SolidMask.y + SolidMask.Hgt>pSolidMask->Hgt) SolidMask.Default();
 	}
 	else if (SolidMask.Wdt)
 	{
 		// Warning in case someone wants to define SolidMasks the old way (in the main graphics file)
 		DebugLogF("WARNING: Definition %s (%s) defines SolidMask in DefCore but has no SolidMask file!", hGroup.GetFullName().getData(), id.ToString());
+		SolidMask.Default();
 	}
 
-	// Read surface bitmap
-	if (dwLoadWhat & C4D_Load_Bitmap)
-		if (!Graphics.Load(hGroup, !!ColorByOwner))
-		{
-			DebugLogF("  Error loading graphics of %s (%s)", hGroup.GetFullName().getData(), id.ToString());
-			return false;
-		}
+	return true;
+}
 
-	// Read string table
-	C4Language::LoadComponentHost(&StringTable, hGroup, C4CFN_ScriptStringTbl, szLanguage);
+bool C4Def::LoadGraphics(C4Group &hGroup, StdMeshSkeletonLoader &loader)
+{
+	// Try to load graphics
+	// No fail on error - just have an object without graphics.
+	Graphics.Load(hGroup, loader, !!ColorByOwner);
 
-	// Register ID with script engine
-	::ScriptEngine.RegisterGlobalConstant(id.ToString(), C4VPropList(this));
-	ParentKeyName = ::Strings.RegString(id.ToString());
-
-	// Read script
-	if (dwLoadWhat & C4D_Load_Script)
-	{
-		// reg script to engine
-		Script.Reg2List(&::ScriptEngine);
-		// Load script
-		Script.Load(hGroup, C4CFN_Script, szLanguage, &StringTable);
-	}
-
-	// read clonknames
-	if (dwLoadWhat & C4D_Load_ClonkNames)
-	{
-		// clear any previous
-		if (pClonkNames) delete pClonkNames; pClonkNames=NULL;
-		if (hGroup.FindEntry(C4CFN_ClonkNameFiles))
-		{
-			// create new
-			pClonkNames = new C4ComponentHost();
-			if (!C4Language::LoadComponentHost(pClonkNames, hGroup, C4CFN_ClonkNames, szLanguage))
-			{
-				delete pClonkNames; pClonkNames = NULL;
-			}
-			else
-				fClonkNamesOwned = true;
-		}
-	}
-
-	// read clonkranks
-	if (dwLoadWhat & C4D_Load_RankNames)
-	{
-		// clear any previous
-		if (pRankNames) delete pRankNames; pRankNames=NULL;
-		if (hGroup.FindEntry(C4CFN_RankNameFiles))
-		{
-			// create new
-			pRankNames = new C4RankSystem();
-			// load from group
-			if (!pRankNames->Load(hGroup, C4CFN_RankNames, 1000, szLanguage))
-			{
-				delete pRankNames; pRankNames=NULL;
-			}
-			else
-				fRankNamesOwned = true;
-		}
-	}
-
-	// read rankfaces
-	if (dwLoadWhat & C4D_Load_RankFaces)
-	{
-		// clear any previous
-		if (pRankSymbols) delete pRankSymbols; pRankSymbols=NULL;
-		// load new
-		if (hGroup.AccessEntry(C4CFN_RankFacesPNG))
-		{
-			pRankSymbols = new C4FacetSurface();
-			if (!pRankSymbols->GetFace().ReadPNG(hGroup)) { delete pRankSymbols; pRankSymbols=NULL; }
-		}
-		// set size
-		if (pRankSymbols)
-		{
-			pRankSymbols->Set(&pRankSymbols->GetFace(), 0,0, pRankSymbols->GetFace().Hgt,pRankSymbols->GetFace().Hgt);
-			int32_t Q; pRankSymbols->GetPhaseNum(iNumRankSymbols, Q);
-			if (!iNumRankSymbols) { delete pRankSymbols; pRankSymbols=NULL; }
-			else
-			{
-				if (pRankNames)
-				{
-					// if extended rank names are defined, subtract those from the symbol count. The last symbols are used as overlay
-					iNumRankSymbols = Max<int32_t>(1, iNumRankSymbols - pRankNames->GetExtendedRankNum());
-				}
-				fRankSymbolsOwned = true;
-			}
-		}
-	}
-
-	// Read sounds
-	if (dwLoadWhat & C4D_Load_Sounds)
-		if (pSoundSystem)
-			pSoundSystem->LoadEffects(hGroup);
-
-	if(Graphics.Type == C4DefGraphics::TYPE_Bitmap)
+	if (Graphics.Type == C4DefGraphics::TYPE_Bitmap)
 	{
 		// Bitmap post-load settings
 		if (Graphics.GetBitmap())
 		{
-			// check SolidMask
-			if (SolidMask.x<0 || SolidMask.y<0 || SolidMask.x+SolidMask.Wdt>Graphics.Bmp.Bitmap->Wdt || SolidMask.y+SolidMask.Hgt>Graphics.Bmp.Bitmap->Hgt) SolidMask.Default();
 			// Set MainFace (unassigned bitmap: will be set by GetMainFace())
-			MainFace.Set(NULL,0,0,Shape.Wdt,Shape.Hgt);
+			MainFace.Set(NULL, 0, 0, Shape.Wdt, Shape.Hgt);
 		}
 
 		// Adjust picture rect
-		if ((PictureRect.Wdt==0) || (PictureRect.Hgt==0))
-			PictureRect.Set(0,0,Shape.Wdt*Graphics.Bmp.Bitmap->Scale, Shape.Hgt*Graphics.Bmp.Bitmap->Scale);
+		if ((PictureRect.Wdt == 0) || (PictureRect.Hgt == 0))
+			PictureRect.Set(0, 0, Shape.Wdt*Graphics.Bmp.Bitmap->Scale, Shape.Hgt*Graphics.Bmp.Bitmap->Scale);
 
 		// validate TopFace
-		if (TopFace.x<0 || TopFace.y<0 || TopFace.x+TopFace.Wdt>Graphics.Bmp.Bitmap->Wdt || TopFace.y+TopFace.Hgt>Graphics.Bmp.Bitmap->Hgt)
+		if (TopFace.x<0 || TopFace.y<0 || TopFace.x + TopFace.Wdt>Graphics.Bmp.Bitmap->Wdt || TopFace.y + TopFace.Hgt>Graphics.Bmp.Bitmap->Hgt)
 		{
 			TopFace.Default();
 			// warn in debug mode
@@ -480,16 +514,88 @@ bool C4Def::Load(C4Group &hGroup,
 	else
 	{
 		TopFace.Default();
-		PictureRect.Default();
-		SolidMask.Default();
 	}
 
-	// Temporary flag
-	if (dwLoadWhat & C4D_Load_Temporary) Temporary=true;
-
 	return true;
 }
 
+void C4Def::LoadScript(C4Group &hGroup, const char* szLanguage)
+{
+	// reg script to engine
+	Script.Reg2List(&::ScriptEngine);
+	// Load script
+	Script.Load(hGroup, C4CFN_Script, szLanguage, &StringTable);
+}
+
+void C4Def::LoadClonkNames(C4Group &hGroup, C4ComponentHost* pClonkNames, const char* szLanguage)
+{
+	// clear any previous
+	if (pClonkNames) delete pClonkNames; pClonkNames = NULL;
+	if (hGroup.FindEntry(C4CFN_ClonkNameFiles))
+	{
+		// create new
+		pClonkNames = new C4ComponentHost();
+		if (!C4Language::LoadComponentHost(pClonkNames, hGroup, C4CFN_ClonkNames, szLanguage))
+		{
+			delete pClonkNames; pClonkNames = NULL;
+		}
+		else
+			fClonkNamesOwned = true;
+	}
+}
+
+void C4Def::LoadRankNames(C4Group &hGroup, const char* szLanguage)
+{
+	// clear any previous
+	if (pRankNames) delete pRankNames; pRankNames = NULL;
+	if (hGroup.FindEntry(C4CFN_RankNameFiles))
+	{
+		// create new
+		pRankNames = new C4RankSystem();
+		// load from group
+		if (!pRankNames->Load(hGroup, C4CFN_RankNames, 1000, szLanguage))
+		{
+			delete pRankNames; pRankNames = NULL;
+		}
+		else
+			fRankNamesOwned = true;
+	}
+}
+
+void C4Def::LoadRankFaces(C4Group &hGroup)
+{
+	// clear any previous
+	if (pRankSymbols) delete pRankSymbols; pRankSymbols = NULL;
+	// load new
+	if (hGroup.AccessEntry(C4CFN_RankFacesPNG))
+	{
+		pRankSymbols = new C4FacetSurface();
+		if (!pRankSymbols->GetFace().ReadPNG(hGroup)) { delete pRankSymbols; pRankSymbols = NULL; }
+	}
+	// set size
+	if (pRankSymbols)
+	{
+		pRankSymbols->Set(&pRankSymbols->GetFace(), 0, 0, pRankSymbols->GetFace().Hgt, pRankSymbols->GetFace().Hgt);
+		int32_t Q; pRankSymbols->GetPhaseNum(iNumRankSymbols, Q);
+		if (!iNumRankSymbols) { delete pRankSymbols; pRankSymbols = NULL; }
+		else
+		{
+			if (pRankNames)
+			{
+				// if extended rank names are defined, subtract those from the symbol count. The last symbols are used as overlay
+				iNumRankSymbols = Max<int32_t>(1, iNumRankSymbols - pRankNames->GetExtendedRankNum());
+			}
+			fRankSymbolsOwned = true;
+		}
+	}
+}
+
+void C4Def::LoadSounds(C4Group &hGroup, C4SoundSystem* pSoundSystem)
+{
+	if (pSoundSystem)
+		pSoundSystem->LoadEffects(hGroup);
+}
+
 void C4Def::Draw(C4Facet &cgo, bool fSelected, DWORD iColor, C4Object *pObj, int32_t iPhaseX, int32_t iPhaseY, C4DrawTransform* trans)
 {
 	if(fSelected)
