@@ -27,6 +27,8 @@
 #include <C4Game.h>
 #include <C4GameObjects.h>
 
+static const C4ObjectLink NULL_LINK = { NULL, NULL, NULL };
+
 C4ObjectList::C4ObjectList(): FirstIter(0)
 {
 	Default();
@@ -49,7 +51,16 @@ void C4ObjectList::Clear()
 	for (cLnk=First; cLnk; cLnk=nextLnk)
 		{ nextLnk=cLnk->Next; delete cLnk; }
 	First=Last=NULL;
-	if (pEnumerated) delete pEnumerated; pEnumerated=NULL;
+	if (pEnumerated)
+	{
+		delete pEnumerated;
+		pEnumerated=NULL;
+	}
+
+	for (iterator* it = FirstIter; it; it = it->Next)
+	{
+		it->link = NULL_LINK;
+	}
 }
 
 const int MaxTempListID = 500;
@@ -251,7 +262,19 @@ bool C4ObjectList::Remove(C4Object *pObj)
 	// Fix iterators
 	for (iterator * i = FirstIter; i; i = i->Next)
 	{
-		if (i->pLink == cLnk) i->pLink = cLnk->Next;
+		// adjust pointers of internal link field
+		if (i->link.Prev == cLnk)
+		{
+			i->link.Prev = cLnk->Prev;
+		}
+		else if (i->link.Next == cLnk)
+		{
+			i->link.Next = cLnk->Next;
+		}
+		else if (i->link.Obj == cLnk->Obj)
+		{
+			i->link.Obj = nullptr;
+		}
 	}
 
 	// Remove link from list
@@ -270,13 +293,13 @@ bool C4ObjectList::Remove(C4Object *pObj)
 	return true;
 }
 
-C4Object* C4ObjectList::Find(C4ID id, int owner, DWORD dwOCF)
+C4Object* C4ObjectList::Find(C4Def * def, int owner, DWORD dwOCF)
 {
 	C4ObjectLink *cLnk;
 	// Find link and object
 	for (cLnk=First; cLnk; cLnk=cLnk->Next)
 		if (cLnk->Obj->Status)
-			if (cLnk->Obj->Def->id==id)
+			if (cLnk->Obj->Def==def)
 				if ((owner==ANY_OWNER) || (cLnk->Obj->Owner==owner))
 					if (dwOCF & cLnk->Obj->OCF)
 						return cLnk->Obj;
@@ -295,7 +318,7 @@ C4Object* C4ObjectList::FindOther(C4ID id, int owner)
 	return NULL;
 }
 
-const C4Object* C4ObjectList::GetObject(int Index) const
+C4Object* C4ObjectList::GetObject(int Index) const
 {
 	int cIdx;
 	C4ObjectLink *cLnk;
@@ -608,6 +631,18 @@ void C4ObjectList::InsertLink(C4ObjectLink *pLnk, C4ObjectLink *pAfter)
 		if (First) First->Prev=pLnk; else Last=pLnk;
 		First=pLnk;
 	}
+
+	// adjust iterators
+	if (pAfter)
+	{
+		for (iterator* it = FirstIter; it; it = it->Next)
+		{
+			if (it->link.Obj == pAfter->Obj)
+			{
+				it->link.Next = pLnk;
+			}
+		}
+	}
 }
 
 void C4ObjectList::InsertLinkBefore(C4ObjectLink *pLnk, C4ObjectLink *pBefore)
@@ -626,6 +661,18 @@ void C4ObjectList::InsertLinkBefore(C4ObjectLink *pLnk, C4ObjectLink *pBefore)
 		if (Last) Last->Next = pLnk; else First = pLnk;
 		Last = pLnk;
 	}
+
+	// adjust iterators
+	if (pBefore)
+	{
+		for (iterator* it = FirstIter; it; it = it->Next)
+		{
+			if (it->link.Obj == pBefore->Obj)
+			{
+				it->link.Prev = pLnk;
+			}
+		}
+	}
 }
 
 
@@ -716,6 +763,7 @@ void C4ObjectList::DeleteObjects()
 	while (First)
 	{
 		C4Object *pObj = First->Obj;
+		if (pObj->Status) Game.ClearPointers(pObj); // clear pointers to removed objects that weren't deleted (game end or section change)
 		Remove(pObj);
 		delete pObj;
 	}
@@ -830,18 +878,13 @@ void C4ObjectList::CheckCategorySort()
 		}
 }
 
-C4ObjectList::iterator::iterator(C4ObjectList & List):
-		List(List), pLink(List.First)
-{
-	Next = List.AddIter(this);
-}
-C4ObjectList::iterator::iterator(C4ObjectList & List, C4ObjectLink * pLink):
-		List(List), pLink(pLink)
+C4ObjectList::iterator::iterator(const C4ObjectList & List, const C4ObjectLink * pLink, bool reverse):
+		List(List), link(pLink ? *pLink : NULL_LINK), reverse(reverse)
 {
 	Next=List.AddIter(this);
 }
 C4ObjectList::iterator::iterator(const C4ObjectList::iterator & iter):
-		List(iter.List), pLink(iter.pLink), Next()
+		List(iter.List), link(iter.link), Next(), reverse(iter.reverse)
 {
 	Next=List.AddIter(this);
 }
@@ -851,26 +894,64 @@ C4ObjectList::iterator::~iterator()
 }
 C4ObjectList::iterator& C4ObjectList::iterator::operator++ ()
 {
-	pLink = pLink ? pLink->Next : pLink;
+	C4ObjectLink* advance = reverse ? link.Prev : link.Next;
+	link = advance ? *advance : NULL_LINK;
 	return *this;
 }
 C4ObjectList::iterator C4ObjectList::iterator::operator++ (int)
 {
 	iterator old = *this;
-	pLink = pLink ? pLink->Next : pLink;
+	iterator::operator++();
 	return old;
 }
 C4Object * C4ObjectList::iterator::operator* ()
 {
-	return pLink ? pLink->Obj : 0;
+	return link.Obj;
 }
 bool C4ObjectList::iterator::operator== (const iterator & iter) const
 {
-	return &iter.List == &List && iter.pLink == pLink;
+	return
+		&iter.List == &List &&
+		iter.link.Obj == link.Obj /* checking for same object should be enough */ &&
+		iter.reverse == reverse;
 }
 bool C4ObjectList::iterator::operator!= (const iterator & iter) const
 {
-	return &iter.List != &List || iter.pLink != pLink;
+	return !(*this == iter);
+}
+
+bool C4ObjectList::iterator::find(C4Object* target)
+{
+	while (link.Obj)
+	{
+		if (link.Obj == target)
+		{
+			return true;
+		}
+		else
+		{
+			(*this)++;
+		}
+	}
+	return false;
+}
+
+bool C4ObjectList::iterator::atEnd() const
+{
+	return link.Obj == nullptr;
+}
+
+bool C4ObjectList::iterator::reset()
+{
+	C4ObjectLink* l = reverse ? List.Last : List.First;
+	link = l ? *l : NULL_LINK;
+	return !atEnd();
+}
+
+bool C4ObjectList::iterator::advance()
+{
+	(*this)++;
+	return !atEnd();
 }
 
 C4ObjectList::iterator& C4ObjectList::iterator::operator=(const iterator & iter)
@@ -878,25 +959,26 @@ C4ObjectList::iterator& C4ObjectList::iterator::operator=(const iterator & iter)
 	// Can only assign iterators into the same list
 	assert(&iter.List == &List);
 
-	pLink = iter.pLink;
+	link = iter.link;
+	reverse = iter.reverse;
 	return *this;
 }
 
-C4ObjectList::iterator C4ObjectList::begin()
+C4ObjectList::iterator C4ObjectList::begin() const
 {
-	return iterator(*this);
+	return iterator(*this, First, false);
 }
-const C4ObjectList::iterator C4ObjectList::end()
+const C4ObjectList::iterator C4ObjectList::end() const
 {
-	return iterator(*this, 0);
+	return iterator(*this, nullptr, false);
 }
-C4ObjectList::iterator * C4ObjectList::AddIter(iterator * iter)
+C4ObjectList::iterator * C4ObjectList::AddIter(iterator * iter) const
 {
 	iterator * r = FirstIter;
 	FirstIter = iter;
 	return r;
 }
-void C4ObjectList::RemoveIter(iterator * iter)
+void C4ObjectList::RemoveIter(iterator * iter) const
 {
 	if (iter == FirstIter)
 		FirstIter = iter->Next;
@@ -908,3 +990,13 @@ void C4ObjectList::RemoveIter(iterator * iter)
 		i->Next = iter->Next;
 	}
 }
+
+C4ObjectList::iterator C4ObjectList::ReverseView::begin() const
+{
+	return iterator(list, list.Last, true);
+}
+
+C4ObjectList::iterator C4ObjectList::ReverseView::end() const
+{
+	return iterator(list, nullptr, true);
+}
\ No newline at end of file
