@@ -155,10 +155,10 @@ static C4Void FnSetPosition(C4Object *Obj, long iX, long iY, bool fCheckBounds,
 	return C4Void();
 }
 
-static C4Void FnDoCon(C4Object *Obj, long iChange, long iPrec)
+static C4Void FnDoCon(C4Object *Obj, long iChange, long iPrec, bool bGrowFromCenter)
 {
 	if (!iPrec) iPrec = 100;
-	Obj->DoCon(FullCon*iChange/iPrec);
+	Obj->DoCon(FullCon*iChange / iPrec, bGrowFromCenter);
 	return C4Void();
 }
 
@@ -304,7 +304,7 @@ static C4Void FnDoDamage(C4Object *Obj, long iChange, Nillable<long> iDmgType, N
 		//  iCausedBy = cthr->Caller->Obj->Controller;
 		//else
 		iCausedBy = NO_OWNER;
-	Obj->DoDamage(iChange,iCausedBy, iDmgType);
+	Obj->DoDamage(iChange, iCausedBy, iDmgType);
 	return C4Void();
 }
 
@@ -824,16 +824,17 @@ static bool FnCreateMenu(C4Object *Obj, C4Def *pDef, C4Object *pCommandObj,
 	return true;
 }
 
-const int C4MN_Add_ImgRank     =   1,
-          C4MN_Add_ImgIndexed  =   2,
-          C4MN_Add_ImgObjRank  =   3,
-          C4MN_Add_ImgObject   =   4,
-          C4MN_Add_ImgTextSpec =   5,
-          C4MN_Add_ImgColor    =   6,
-          C4MN_Add_MaxImage    = 127, // mask for param which decides what to draw as the menu symbol
-          C4MN_Add_PassValue   = 128,
-          C4MN_Add_ForceCount  = 256,
-          C4MN_Add_ForceNoDesc = 512;
+const int C4MN_Add_ImgRank         =   1,
+          C4MN_Add_ImgIndexed      =   2,
+          C4MN_Add_ImgObjRank      =   3,
+          C4MN_Add_ImgObject       =   4,
+          C4MN_Add_ImgTextSpec     =   5,
+          C4MN_Add_ImgColor        =   6,
+          C4MN_Add_ImgPropListSpec =   7,
+          C4MN_Add_MaxImage        = 127, // mask for param which decides what to draw as the menu symbol
+          C4MN_Add_PassValue       = 128,
+          C4MN_Add_ForceCount      = 256,
+          C4MN_Add_ForceNoDesc     = 512;
 
 #ifndef _MSC_VER
 #define _snprintf snprintf
@@ -858,7 +859,7 @@ static bool FnAddMenuItem(C4Object *Obj, C4String * szCaption, C4String * szComm
 	{
 		const char * s = FnStringPar(szCaption);
 		const char * sep = strstr(s, "%s");
-		if (sep)
+		if (sep && pDef)
 		{
 			strncpy(caption, s, Min<intptr_t>(sep - s,256));
 			caption[Min<intptr_t>(sep - s,256)] = 0;
@@ -1074,6 +1075,15 @@ static bool FnAddMenuItem(C4Object *Obj, C4String * szCaption, C4String * szComm
 	}
 	break;
 
+	case C4MN_Add_ImgPropListSpec:
+	{
+		C4PropList *gfx_proplist = XPar.getPropList();
+		fctSymbol.Create(iSymbolSize,iSymbolSize);
+		if (!Game.DrawPropListSpecImage(fctSymbol, gfx_proplist))
+			return false;
+	}
+	break;
+
 	case C4MN_Add_ImgColor:
 		// draw colored def facet
 		fctSymbol.Create(iSymbolSize,iSymbolSize);
@@ -1156,7 +1166,7 @@ static C4Object *FnContents(C4Object *Obj, long index)
 	return NULL;
 }
 
-static bool FnShiftContents(C4Object *Obj, bool fShiftBack, C4ID idTarget, bool fDoCalls)
+static bool FnShiftContents(C4Object *Obj, bool fShiftBack, C4Def * idTarget, bool fDoCalls)
 {
 	// regular shift
 	if (!idTarget) return !!Obj->ShiftContents(fShiftBack, fDoCalls);
@@ -1186,7 +1196,7 @@ static long FnContentsCount(C4Object *Obj, C4ID id)
 	return Obj->Contents.ObjectCount(id);
 }
 
-static C4Object *FnFindContents(C4Object *Obj, C4ID c_id)
+static C4Object *FnFindContents(C4Object *Obj, C4Def * c_id)
 {
 	return Obj->Contents.Find(c_id);
 }
@@ -1325,10 +1335,17 @@ static C4Void FnSetColor(C4Object *Obj, long iValue)
 	return C4Void();
 }
 
-static C4Void FnSetPlrViewRange(C4Object *Obj, long iRange)
+static C4Void FnSetLightRange(C4Object *Obj, long iRange, Nillable<long> iFadeoutRange)
 {
+	if (iFadeoutRange.IsNil())
+	{
+		if(iRange == 0)
+			iFadeoutRange = 0;
+		else
+			iFadeoutRange = C4FOW_DefLightFadeoutRangeX;
+	}
 	// set range
-	Obj->SetPlrViewRange(iRange);
+	Obj->SetLightRange(iRange, iFadeoutRange);
 	// success
 	return C4Void();
 }
@@ -1521,7 +1538,7 @@ static bool FnSetMenuSize(C4Object *Obj, long iCols, long iRows)
 	// get menu
 	C4Menu *pMnu=Obj->Menu;
 	if (!pMnu || !pMnu->IsActive()) return false;
-	pMnu->SetSize(BoundBy<long>(iCols, 0, 50), BoundBy<long>(iRows, 0, 50));
+	pMnu->SetSize(Clamp<long>(iCols, 0, 50), Clamp<long>(iRows, 0, 50));
 	return true;
 }
 
@@ -1594,9 +1611,9 @@ static C4Void FnSetObjectLayer(C4Object *Obj, C4Object *pNewLayer)
 	// set layer object
 	Obj->Layer = pNewLayer;
 	// set for all contents as well
-	for (C4ObjectLink *pLnk=Obj->Contents.First; pLnk; pLnk=pLnk->Next)
-		if ((Obj=pLnk->Obj) && Obj->Status)
-			Obj->Layer = pNewLayer;
+	for (C4Object* contentObj : Obj->Contents)
+		if (contentObj && contentObj->Status)
+			contentObj->Layer = pNewLayer;
 	// success
 	return C4Void();
 }
@@ -1855,7 +1872,7 @@ static Nillable<int> FnTransformBone(C4Object *Obj, C4String *szBoneName, C4Valu
 		if (!s_node || s_node->GetSlot() != iSlot) return C4Void();
 	}
 
-	const StdMeshBone* bone = Instance->GetMesh().GetBoneByName(szBoneName->GetData());
+	const StdMeshBone* bone = Instance->GetMesh().GetSkeleton().GetBoneByName(szBoneName->GetData());
 	if(!bone) return C4Void();
 
 	StdMeshInstance::ValueProvider* w_provider = CreateValueProviderFromArray(Obj, *WeightProvider);
@@ -1936,7 +1953,7 @@ static Nillable<int> FnGetAnimationLength(C4Object *Obj, C4String *szAnimation,
 		Instance = Attached->Child;
 	}
 
-	const StdMeshAnimation* animation = Instance->GetMesh().GetAnimationByName(szAnimation->GetData());
+	const StdMeshAnimation* animation = Instance->GetMesh().GetSkeleton().GetAnimationByName(szAnimation->GetData());
 	if (!animation) return C4Void();
 	return fixtoi(ftofix(animation->Length), 1000); // sync critical!
 }
@@ -2150,13 +2167,28 @@ static bool FnSetAttachTransform(C4Object *Obj, long iAttachNumber, C4ValueArray
 	return true;
 }
 
-static Nillable<C4String*> FnGetMeshMaterial(C4Object *Obj, int iSubMesh)
+static Nillable<C4String*> FnGetMeshMaterial(C4PropList * _this, int iSubMesh)
 {
-	if (!Obj || !Obj->pMeshInstance) return C4Void();
-	if (iSubMesh < 0 || (unsigned int)iSubMesh >= Obj->pMeshInstance->GetNumSubMeshes()) return C4Void();
-
-	StdSubMeshInstance& submesh = Obj->pMeshInstance->GetSubMesh(iSubMesh);
-	return String(submesh.GetMaterial().Name.getData());
+	// Called in object or definition context?
+	C4Object *Obj = Object(_this);
+	if (!Obj)
+	{
+		if (!_this || !_this->GetDef()) throw new NeedNonGlobalContext("GetMeshMaterial");
+		// Called in definition context: Get definition default mesh material
+		C4Def *def = _this->GetDef();
+		if (!def->Graphics.IsMesh()) return C4Void();
+		if (iSubMesh < 0 || (unsigned int)iSubMesh >= def->Graphics.Mesh->GetNumSubMeshes()) return C4Void();
+		const StdSubMesh &submesh = def->Graphics.Mesh->GetSubMesh(iSubMesh);
+		return String(submesh.GetMaterial().Name.getData());
+	}
+	else
+	{
+		// Called in object context: Get material of mesh instance
+		if (!Obj->pMeshInstance) return C4Void();
+		if (iSubMesh < 0 || (unsigned int)iSubMesh >= Obj->pMeshInstance->GetNumSubMeshes()) return C4Void();
+		StdSubMeshInstance& submesh = Obj->pMeshInstance->GetSubMesh(iSubMesh);
+		return String(submesh.GetMaterial().Name.getData());
+	}
 }
 
 static bool FnSetMeshMaterial(C4Object *Obj, C4String* Material, int iSubMesh)
@@ -2168,8 +2200,7 @@ static bool FnSetMeshMaterial(C4Object *Obj, C4String* Material, int iSubMesh)
 	const StdMeshMaterial* material = ::MeshMaterialManager.GetMaterial(Material->GetData().getData());
 	if (!material) return false;
 
-	StdSubMeshInstance& submesh = Obj->pMeshInstance->GetSubMesh(iSubMesh);
-	submesh.SetMaterial(*material);
+	Obj->pMeshInstance->SetMaterial(iSubMesh, *material);
 	return true;
 }
 
@@ -2181,8 +2212,12 @@ static bool FnCreateParticleAtBone(C4Object* Obj, C4String* szName, C4String* sz
 	// Get bone
 	if(!Obj->pMeshInstance) return false;
 	const StdMesh& mesh = Obj->pMeshInstance->GetMesh();
-	const StdMeshBone* bone = mesh.GetBoneByName(szBoneName->GetData());
+	const StdMeshBone* bone = mesh.GetSkeleton().GetBoneByName(szBoneName->GetData());
 	if(!bone) return false;
+	// get particle
+	C4ParticleDef *pDef=::Particles.definitions.GetDef(FnStringPar(szName));
+	if (!pDef) return false;
+#ifndef USE_CONSOLE
 	// Get transform
 	Obj->pMeshInstance->UpdateBoneTransforms();
 	const StdMeshMatrix transform = Obj->pMeshInstance->GetBoneTransform(bone->Index) * StdMeshMatrix::Transform(bone->Transformation);
@@ -2250,37 +2285,42 @@ static bool FnCreateParticleAtBone(C4Object* Obj, C4String* szName, C4String* sz
 	const float dy = ty + ry*thgt;
 	x.x += dx;
 	x.y += dy;
-	// Finally, apply DrawTransform to the world coordinates
-	StdMeshMatrix DrawTransform;
+	// This was added in the block before and could also just be removed from tx/ty.
+	// However, the block would no longer be equal to where it came from.
+	x.x -= fixtof(Obj->fix_x);
+	x.y -= fixtof(Obj->fix_y);
+	// Finally, apply DrawTransform to the world coordinates,
+	// and incorporate object rotation into the transformation
+	C4DrawTransform draw_transform;
 	if(Obj->pDrawTransform)
 	{
-		C4DrawTransform transform(*Obj->pDrawTransform, fixtof(Obj->fix_x), fixtof(Obj->fix_y));
-
-		DrawTransform(0, 0) = transform.mat[0];
-		DrawTransform(0, 1) = transform.mat[1];
-		DrawTransform(0, 2) = 0.0f;
-		DrawTransform(0, 3) = transform.mat[2];
-		DrawTransform(1, 0) = transform.mat[3];
-		DrawTransform(1, 1) = transform.mat[4];
-		DrawTransform(1, 2) = 0.0f;
-		DrawTransform(1, 3) = transform.mat[5];
-		DrawTransform(2, 0) = 0.0f;
-		DrawTransform(2, 1) = 0.0f;
-		DrawTransform(2, 2) = 1.0f;
-		DrawTransform(2, 3) = 0.0f;
+		draw_transform.SetTransformAt(*Obj->pDrawTransform, fixtof(Obj->fix_x), fixtof(Obj->fix_y));
+		draw_transform.Rotate(fixtof(Obj->fix_r), 0.0f, 0.0f);
 	}
 	else
 	{
-		DrawTransform = StdMeshMatrix::Identity();
+		draw_transform.SetRotate(fixtof(Obj->fix_r), 0.0f, 0.0f);
 	}
+
+	StdMeshMatrix DrawTransform;
+	DrawTransform(0, 0) = draw_transform.mat[0];
+	DrawTransform(0, 1) = draw_transform.mat[1];
+	DrawTransform(0, 2) = 0.0f;
+	DrawTransform(0, 3) = draw_transform.mat[2];
+	DrawTransform(1, 0) = draw_transform.mat[3];
+	DrawTransform(1, 1) = draw_transform.mat[4];
+	DrawTransform(1, 2) = 0.0f;
+	DrawTransform(1, 3) = draw_transform.mat[5];
+	DrawTransform(2, 0) = 0.0f;
+	DrawTransform(2, 1) = 0.0f;
+	DrawTransform(2, 2) = 1.0f;
+	DrawTransform(2, 3) = 0.0f;
+
 	x = DrawTransform * x;
 	dir = DrawTransform * dir;
 	x.x += DrawTransform(0,3);
 	x.y += DrawTransform(1,3);
 	x.z += DrawTransform(2,3);
-	// get particle
-	C4ParticleDef *pDef=::Particles.definitions.GetDef(FnStringPar(szName));
-	if (!pDef) return false;
 
 	// construct data
 	C4ParticleValueProvider valueX, valueY, valueSpeedX, valueSpeedY, valueLifetime;
@@ -2291,7 +2331,9 @@ static bool FnCreateParticleAtBone(C4Object* Obj, C4String* szName, C4String* sz
 	valueLifetime.Set(lifetime);
 
 	// cast
+	if (amount < 1) amount = 1;
 	::Particles.Create(pDef, valueX, valueY, valueSpeedX, valueSpeedY, valueLifetime, properties, amount, Obj);
+#endif
 	// success, even if not created
 	return true;
 
@@ -2301,6 +2343,7 @@ static bool FnCreateParticleAtBone(C4Object* Obj, C4String* szName, C4String* sz
 
 C4ScriptConstDef C4ScriptObjectConstMap[]=
 {
+	{ "C4D_None"               ,C4V_Int,          C4D_None},
 	{ "C4D_All"                ,C4V_Int,          C4D_All},
 	{ "C4D_StaticBack"         ,C4V_Int,          C4D_StaticBack},
 	{ "C4D_Structure"          ,C4V_Int,          C4D_Structure},
@@ -2384,6 +2427,7 @@ C4ScriptConstDef C4ScriptObjectConstMap[]=
 	{ "C4MN_Add_ImgObjRank"    ,C4V_Int,          C4MN_Add_ImgObjRank},
 	{ "C4MN_Add_ImgObject"     ,C4V_Int,          C4MN_Add_ImgObject},
 	{ "C4MN_Add_ImgTextSpec"   ,C4V_Int,          C4MN_Add_ImgTextSpec},
+	{ "C4MN_Add_ImgPropListSpec",C4V_Int,         C4MN_Add_ImgPropListSpec},
 	{ "C4MN_Add_ImgColor"      ,C4V_Int,          C4MN_Add_ImgColor},
 	{ "C4MN_Add_PassValue"     ,C4V_Int,          C4MN_Add_PassValue},
 	{ "C4MN_Add_ForceCount"    ,C4V_Int,          C4MN_Add_ForceCount},
@@ -2452,9 +2496,11 @@ C4ScriptConstDef C4ScriptObjectConstMap[]=
 	{ "C4AVP_R"                   ,C4V_Int,      C4AVP_R },
 	{ "C4AVP_AbsX"                ,C4V_Int,      C4AVP_AbsX },
 	{ "C4AVP_AbsY"                ,C4V_Int,      C4AVP_AbsY },
+	{ "C4AVP_Dist"                ,C4V_Int,      C4AVP_Dist },
 	{ "C4AVP_XDir"                ,C4V_Int,      C4AVP_XDir },
 	{ "C4AVP_YDir"                ,C4V_Int,      C4AVP_YDir },
 	{ "C4AVP_RDir"                ,C4V_Int,      C4AVP_RDir },
+	{ "C4AVP_AbsRDir"             ,C4V_Int,      C4AVP_AbsRDir },
 	{ "C4AVP_CosR"                ,C4V_Int,      C4AVP_CosR },
 	{ "C4AVP_SinR"                ,C4V_Int,      C4AVP_SinR },
 	{ "C4AVP_CosV"                ,C4V_Int,      C4AVP_CosV },
@@ -2467,6 +2513,7 @@ C4ScriptConstDef C4ScriptObjectConstMap[]=
 
 	{ "AM_None"                   ,C4V_Int,      StdMeshInstance::AM_None },
 	{ "AM_DrawBefore"             ,C4V_Int,      StdMeshInstance::AM_DrawBefore },
+	{ "AM_MatchSkeleton"          ,C4V_Int,      StdMeshInstance::AM_MatchSkeleton },
 
 	{ NULL, C4V_Nil, 0}
 };
@@ -2570,7 +2617,7 @@ void InitObjectFunctionMap(C4AulScriptEngine *pEngine)
 	AddFunc(pEngine, "SetMass", FnSetMass);
 	AddFunc(pEngine, "GetColor", FnGetColor);
 	AddFunc(pEngine, "SetColor", FnSetColor);
-	AddFunc(pEngine, "SetPlrViewRange", FnSetPlrViewRange);
+	AddFunc(pEngine, "SetLightRange", FnSetLightRange);
 	AddFunc(pEngine, "SetPicture", FnSetPicture);
 	AddFunc(pEngine, "GetProcedure", FnGetProcedure);
 	AddFunc(pEngine, "CanConcatPictureWith", FnCanConcatPictureWith);
