@@ -26,12 +26,70 @@
 #include <C4Config.h>
 #include <C4Application.h>
 #include <C4Viewport.h>
+#include <C4SoundIncludes.h>
 #include <C4SoundLoaders.h>
 
-#ifdef HAVE_LIBSDL_MIXER
-#define USE_RWOPS
-#include <SDL_mixer.h>
-#endif
+class C4SoundEffect
+{
+	friend class C4SoundInstance;
+public:
+	C4SoundEffect();
+	~C4SoundEffect();
+public:
+	char Name[C4MaxSoundName+1];
+	int32_t Instances;
+	int32_t SampleRate, Length;
+	C4SoundHandle pSample;
+	C4SoundInstance *FirstInst;
+	C4SoundEffect *Next;
+public:
+	void Clear();
+	bool Load(const char *szFileName, C4Group &hGroup);
+	bool Load(BYTE *pData, size_t iDataLen, bool fRaw=false); // load directly from memory
+	void Execute();
+	C4SoundInstance *New(bool fLoop = false, int32_t iVolume = 100, C4Object *pObj = NULL, int32_t iCustomFalloffDistance = 0);
+	C4SoundInstance *GetInstance(C4Object *pObj);
+	void ClearPointers(C4Object *pObj);
+	int32_t GetStartedInstanceCount(int32_t iX, int32_t iY, int32_t iRad); // local
+	int32_t GetStartedInstanceCount(); // global
+protected:
+	void AddInst(C4SoundInstance *pInst);
+	void RemoveInst(C4SoundInstance *pInst);
+};
+
+class C4SoundInstance
+{
+	friend class C4SoundEffect;
+protected:
+	C4SoundInstance();
+public:
+	~C4SoundInstance();
+protected:
+	C4SoundEffect *pEffect;
+	int32_t iVolume, iPan, iChannel;
+	C4TimeMilliseconds tStarted;
+	int32_t iNearInstanceMax;
+	bool fLooping;
+	C4Object *pObj;
+	int32_t iFalloffDistance;
+	C4SoundInstance *pNext;
+public:
+	C4Object *getObj() const { return pObj; }
+	bool isStarted() const { return iChannel != -1; }
+	void Clear();
+	bool Create(C4SoundEffect *pEffect, bool fLoop = false, int32_t iVolume = 100, C4Object *pObj = NULL, int32_t iNearInstanceMax = 0, int32_t iFalloffDistance = 0);
+	bool CheckStart();
+	bool Start();
+	bool Stop();
+	bool Playing();
+	void Execute();
+	void SetVolume(int32_t inVolume) { iVolume = inVolume; }
+	void SetPan(int32_t inPan) { iPan = inPan; }
+	void SetVolumeByPos(int32_t x, int32_t y);
+	void SetObj(C4Object *pnObj) { pObj = pnObj; }
+	void ClearPointers(C4Object *pObj);
+	bool Inside(int32_t iX, int32_t iY, int32_t iRad);
+};
 
 using namespace C4SoundLoaders;
 
@@ -52,13 +110,11 @@ C4SoundEffect::~C4SoundEffect()
 void C4SoundEffect::Clear()
 {
 	while (FirstInst) RemoveInst(FirstInst);
-#ifdef HAVE_FMOD
+#if AUDIO_TK == AUDIO_TK_FMOD
 	if (pSample) FSOUND_Sample_Free(pSample);
-#endif
-#ifdef HAVE_LIBSDL_MIXER
+#elif AUDIO_TK == AUDIO_TK_SDL_MIXER
 	if (pSample) Mix_FreeChunk(pSample);
-#endif
-#ifdef USE_OPEN_AL
+#elif AUDIO_TK == AUDIO_TK_OPENAL
 	if (pSample) alDeleteBuffers(1, &pSample);
 #endif
 	pSample = 0;
@@ -72,7 +128,7 @@ bool C4SoundEffect::Load(const char *szFileName, C4Group &hGroup)
 	StdBuf WaveBuffer;
 	if (!hGroup.LoadEntry(szFileName, &WaveBuffer)) return false;
 	// load it from mem
-	if (!Load((BYTE*)WaveBuffer.getData(), WaveBuffer.getSize())) return false;
+	if (!Load((BYTE*)WaveBuffer.getMData(), WaveBuffer.getSize())) return false;
 	// Set name
 	SCopy(szFileName,Name,C4MaxSoundName);
 	return true;
@@ -98,7 +154,7 @@ bool C4SoundEffect::Load(BYTE *pData, size_t iDataLen, bool fRaw)
 			}
 			else
 			{
-#ifdef USE_OPEN_AL
+#if AUDIO_TK == AUDIO_TK_OPENAL
 				Application.MusicSystem.SelectContext();
 				alGenBuffers(1, &pSample);
 				alBufferData(pSample, info.format, &info.sound_data[0], info.sound_data.size(), info.sample_rate);
@@ -253,7 +309,7 @@ bool C4SoundInstance::CheckStart()
 
 bool C4SoundInstance::Start()
 {
-#ifdef HAVE_FMOD
+#if AUDIO_TK == AUDIO_TK_FMOD
 	// Start
 	if ((iChannel = FSOUND_PlaySound(FSOUND_FREE, pEffect->pSample)) == -1)
 		return false;
@@ -266,12 +322,12 @@ bool C4SoundInstance::Start()
 		int32_t iTime = (C4TimeMilliseconds::Now() - tStarted) % pEffect->Length;
 		FSOUND_SetCurrentPosition(iChannel, iTime / 10 * pEffect->SampleRate / 100);
 	}
-#elif defined HAVE_LIBSDL_MIXER
+#elif AUDIO_TK == AUDIO_TK_SDL_MIXER
 	// Be paranoid about SDL_Mixer initialisation
 	if (!Application.MusicSystem.MODInitialized) return false;
 	if ((iChannel = Mix_PlayChannel(-1, pEffect->pSample, fLooping? -1 : 0)) == -1)
 		return false;
-#elif defined(USE_OPEN_AL)
+#elif AUDIO_TK == AUDIO_TK_OPENAL
 	Application.MusicSystem.SelectContext();
 	alGenSources(1, (ALuint*)&iChannel);
 	alSourcei(iChannel, AL_BUFFER, pEffect->pSample);
@@ -292,16 +348,14 @@ bool C4SoundInstance::Stop()
 	if (!pEffect) return false;
 	// Stop sound
 	bool fRet = true;
-#ifdef HAVE_FMOD
+#if AUDIO_TK == AUDIO_TK_FMOD
 	if (Playing())
 		fRet = !! FSOUND_StopSound(iChannel);
-#endif
-#ifdef HAVE_LIBSDL_MIXER
+#elif AUDIO_TK == AUDIO_TK_SDL_MIXER
 	// iChannel == -1 will halt all channels. Is that right?
 	if (Playing())
 		Mix_HaltChannel(iChannel);
-#endif
-#ifdef USE_OPEN_AL
+#elif AUDIO_TK == AUDIO_TK_OPENAL
 	if (iChannel != -1)
 	{
 		if (Playing()) alSourceStop(iChannel);
@@ -318,15 +372,13 @@ bool C4SoundInstance::Stop()
 bool C4SoundInstance::Playing()
 {
 	if (!pEffect) return false;
-#ifdef HAVE_FMOD
+#if AUDIO_TK == AUDIO_TK_FMOD
 	if (fLooping) return true;
 	return isStarted() ? FSOUND_GetCurrentSample(iChannel) == pEffect->pSample
 	       : C4TimeMilliseconds::Now() < tStarted + pEffect->Length;
-#endif
-#ifdef HAVE_LIBSDL_MIXER
+#elif AUDIO_TK == AUDIO_TK_SDL_MIXER
 	return Application.MusicSystem.MODInitialized && (iChannel != -1) && Mix_Playing(iChannel);
-#endif
-#ifdef USE_OPEN_AL
+#elif AUDIO_TK == AUDIO_TK_OPENAL
 	if (iChannel == -1)
 		return false;
 	else
@@ -352,7 +404,7 @@ void C4SoundInstance::Execute()
 		// apply custom falloff distance
 		if (iFalloffDistance)
 		{
-			iAudibility = BoundBy<int32_t>(100 + (iAudibility - 100) * C4AudibilityRadius / iFalloffDistance, 0,100);
+			iAudibility = Clamp<int32_t>(100 + (iAudibility - 100) * C4AudibilityRadius / iFalloffDistance, 0,100);
 		}
 		iVol = iVol * iAudibility / 100;
 		iPan += pObj->AudiblePan;
@@ -363,13 +415,11 @@ void C4SoundInstance::Execute()
 		// stop, if started
 		if (isStarted())
 		{
-#ifdef HAVE_FMOD
+#if AUDIO_TK == AUDIO_TK_FMOD
 			FSOUND_StopSound(iChannel);
-#endif
-#ifdef HAVE_LIBSDL_MIXER
+#elif AUDIO_TK == AUDIO_TK_SDL_MIXER
 			Mix_HaltChannel(iChannel);
-#endif
-#ifdef USE_OPEN_AL
+#elif AUDIO_TK == AUDIO_TK_OPENAL
 			alDeleteSources(1, (ALuint*)&iChannel);
 #endif
 			iChannel = -1;
@@ -382,16 +432,14 @@ void C4SoundInstance::Execute()
 			if (!CheckStart())
 				return;
 		// set volume & panning
-#ifdef HAVE_FMOD
-		FSOUND_SetVolume(iChannel, BoundBy(iVol / 100, 0, 255));
-		FSOUND_SetPan(iChannel, BoundBy(256*(iPan+100)/200,0,255));
-#endif
-#ifdef HAVE_LIBSDL_MIXER
+#if AUDIO_TK == AUDIO_TK_FMOD
+		FSOUND_SetVolume(iChannel, Clamp(iVol / 100, 0, 255));
+		FSOUND_SetPan(iChannel, Clamp(256*(iPan+100)/200,0,255));
+#elif AUDIO_TK == AUDIO_TK_SDL_MIXER
 		Mix_Volume(iChannel, (iVol * MIX_MAX_VOLUME) / (100 * 256));
 		//Mix_SetPanning(iChannel, ((100 + iPan) * 256) / 200, ((100 - iPan) * 256) / 200);
-		Mix_SetPanning(iChannel, BoundBy((100 - iPan) * 256 / 100, 0, 255), BoundBy((100 + iPan) * 256 / 100, 0, 255));
-#endif
-#ifdef USE_OPEN_AL
+		Mix_SetPanning(iChannel, Clamp((100 - iPan) * 256 / 100, 0, 255), Clamp((100 + iPan) * 256 / 100, 0, 255));
+#elif AUDIO_TK == AUDIO_TK_OPENAL
 		alSource3f(iChannel, AL_POSITION, 0, 0, 0); // FIXME
 		alSourcef(iChannel, AL_GAIN, float(iVol) / (100.0f*256.0f));
 #endif
@@ -448,7 +496,7 @@ bool C4SoundSystem::Init()
 		if (!Reloc.Open(SoundFile, C4CFN_Sound)) return false;
 	// Load static sound from Sound.ocg
 	LoadEffects(SoundFile);
-#ifdef HAVE_LIBSDL_MIXER
+#if AUDIO_TK == AUDIO_TK_SDL_MIXER
 	Mix_AllocateChannels(C4MaxSoundInstances);
 #endif
 	return true;
@@ -475,7 +523,7 @@ void C4SoundSystem::ClearEffects()
 
 void C4SoundSystem::Execute()
 {
-#ifdef USE_OPEN_AL
+#if AUDIO_TK == AUDIO_TK_OPENAL
 	Application.MusicSystem.SelectContext();
 #endif
 	C4SoundEffect *csfx;
