@@ -118,8 +118,6 @@ LRESULT APIENTRY FullScreenWinProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM l
 			{
 				if (pTexMgr)
 					pTexMgr->IntLock();
-				if (pGL)
-					pGL->TaskOut();
 				if (Application.FullScreenMode())
 				{
 					::ChangeDisplaySettings(NULL, 0);
@@ -303,10 +301,11 @@ LRESULT APIENTRY ViewportWinProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lPa
 	}
 		//----------------------------------------------------------------------------------------------------------------------------------
 	case WM_USER_DROPDEF:
-		Game.DropDef(C4ID(lParam),cvp->ViewX+float(LOWORD(wParam))/cvp->Zoom,cvp->ViewY+float(HIWORD(wParam)/cvp->Zoom));
+		Game.DropDef(C4ID(lParam),cvp->GetViewX()+float(LOWORD(wParam))/cvp->Zoom,cvp->GetViewY()+float(HIWORD(wParam)/cvp->Zoom));
 		break;
 		//----------------------------------------------------------------------------------------------------------------------------------
 	case WM_SIZE:
+	case WM_SIZING:
 		cvp->UpdateOutputSize();
 		break;
 		//----------------------------------------------------------------------------------------------------------------------------------
@@ -317,11 +316,12 @@ LRESULT APIENTRY ViewportWinProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lPa
 	case WM_HSCROLL:
 		switch (LOWORD(wParam))
 		{
-		case SB_THUMBTRACK: case SB_THUMBPOSITION: cvp->ViewX=HIWORD(wParam); break;
-		case SB_LINELEFT: cvp->ViewX-=ViewportScrollSpeed; break;
-		case SB_LINERIGHT: cvp->ViewX+=ViewportScrollSpeed; break;
-		case SB_PAGELEFT: cvp->ViewX-=cvp->ViewWdt; break;
-		case SB_PAGERIGHT: cvp->ViewX+=cvp->ViewWdt; break;
+			case SB_THUMBTRACK:
+			case SB_THUMBPOSITION: cvp->SetViewX(float(HIWORD(wParam))/cvp->Zoom); break;
+			case SB_LINELEFT: cvp->ScrollView(-ViewportScrollSpeed, 0.0f); break;
+			case SB_LINERIGHT: cvp->ScrollView(+ViewportScrollSpeed, 0.0f); break;
+			case SB_PAGELEFT: cvp->ScrollView(-cvp->ViewWdt/cvp->Zoom, 0.0f); break;
+			case SB_PAGERIGHT: cvp->ScrollView(+cvp->ViewWdt/cvp->Zoom, 0.0f); break;
 		}
 		cvp->Execute();
 		cvp->ScrollBarsByViewPosition();
@@ -330,11 +330,12 @@ LRESULT APIENTRY ViewportWinProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lPa
 	case WM_VSCROLL:
 		switch (LOWORD(wParam))
 		{
-		case SB_THUMBTRACK: case SB_THUMBPOSITION: cvp->ViewY=HIWORD(wParam); break;
-		case SB_LINEUP: cvp->ViewY-=ViewportScrollSpeed; break;
-		case SB_LINEDOWN: cvp->ViewY+=ViewportScrollSpeed; break;
-		case SB_PAGEUP: cvp->ViewY-=cvp->ViewWdt; break;
-		case SB_PAGEDOWN: cvp->ViewY+=cvp->ViewWdt; break;
+			case SB_THUMBTRACK:
+			case SB_THUMBPOSITION: cvp->SetViewY(float(HIWORD(wParam))/cvp->Zoom); break;
+			case SB_LINEUP: cvp->ScrollView(0.0f,-ViewportScrollSpeed); break;
+			case SB_LINEDOWN: cvp->ScrollView(0.0f,+ViewportScrollSpeed); break;
+			case SB_PAGEUP: cvp->ScrollView(0.0f,-cvp->ViewWdt/cvp->Zoom); break;
+			case SB_PAGEDOWN: cvp->ScrollView(0.0f,+cvp->ViewWdt/cvp->Zoom); break;
 		}
 		cvp->Execute();
 		cvp->ScrollBarsByViewPosition();
@@ -533,6 +534,7 @@ C4Window::~C4Window ()
 C4Window * C4Window::Init(C4Window::WindowKind windowKind, C4AbstractApp * pApp, const char * Title, const C4Rect * size)
 {
 	Active = true;
+	eKind = windowKind;
 	if (windowKind == W_Viewport)
 	{
 		static bool fViewportClassRegistered = false;
@@ -647,6 +649,11 @@ C4Window * C4Window::Init(C4Window::WindowKind windowKind, C4AbstractApp * pApp,
 		hRenderWindow = hWindow;
 		return hWindow ? this : 0;
 	}
+	else if (windowKind == W_Control)
+	{
+		// controlled externally
+		hWindow = hRenderWindow = NULL;
+	}
 	return this;
 }
 
@@ -674,9 +681,12 @@ bool C4Window::ReInit(C4AbstractApp* pApp)
 
 void C4Window::Clear()
 {
-	// Destroy window
-	if (hRenderWindow) DestroyWindow(hRenderWindow);
-	if (hWindow && hWindow != hRenderWindow) DestroyWindow(hWindow);
+	// Destroy window if we own it
+	if (eKind != W_Control)
+	{
+		if (hRenderWindow) DestroyWindow(hRenderWindow);
+		if (hWindow && hWindow != hRenderWindow) DestroyWindow(hWindow);
+	}
 	hRenderWindow = NULL;
 	hWindow = NULL;
 }
@@ -773,7 +783,7 @@ C4AbstractApp::C4AbstractApp() :
 	ZeroMemory(&pfd, sizeof(pfd)); pfd.nSize = sizeof(pfd);
 	ZeroMemory(&dspMode, sizeof(dspMode)); dspMode.dmSize =  sizeof(dspMode);
 	ZeroMemory(&OldDspMode, sizeof(OldDspMode)); OldDspMode.dmSize =  sizeof(OldDspMode);
-	hMainThread = NULL;
+	idMainThread = 0;
 #ifdef _WIN32
 	MessageProc.SetApp(this);
 	Add(&MessageProc);
@@ -787,14 +797,14 @@ C4AbstractApp::~C4AbstractApp()
 bool C4AbstractApp::Init(int argc, char * argv[])
 {
 	// Set instance vars
-	hMainThread = ::GetCurrentThread();
+	idMainThread = ::GetCurrentThreadId();
 	// Custom initialization
 	return DoInit (argc, argv);
 }
 
 void C4AbstractApp::Clear()
 {
-	hMainThread = NULL;
+	idMainThread = 0;
 }
 
 void C4AbstractApp::Quit()
