@@ -402,31 +402,17 @@ C4Value C4AulExec::Exec(C4AulBCC *pCPos, bool fPassErrors)
 				PopValue();
 				break;
 			}
-			case AB_Identical:  // ===
-			{
-				C4Value *pPar1 = pCurVal - 1, *pPar2 = pCurVal;
-				pPar1->SetBool(pPar1->GetType() == pPar2->GetType() && pPar1->GetData() == pPar2->GetData());
-				PopValue();
-				break;
-			}
-			case AB_NotIdentical: // !==
-			{
-				C4Value *pPar1 = pCurVal - 1, *pPar2 = pCurVal;
-				pPar1->SetBool(pPar1->GetType() != pPar2->GetType() || pPar1->GetData() != pPar2->GetData());
-				PopValue();
-				break;
-			}
 			case AB_Equal:  // ==
 			{
 				C4Value *pPar1 = pCurVal - 1, *pPar2 = pCurVal;
-				pPar1->SetBool(*pPar1 == *pPar2);
+				pPar1->SetBool(pPar1->IsIdenticalTo(*pPar2));
 				PopValue();
 				break;
 			}
 			case AB_NotEqual: // !=
 			{
 				C4Value *pPar1 = pCurVal - 1, *pPar2 = pCurVal;
-				pPar1->SetBool(*pPar1 != *pPar2);
+				pPar1->SetBool(!pPar1->IsIdenticalTo(*pPar2));
 				PopValue();
 				break;
 			}
@@ -836,13 +822,7 @@ C4AulBCC *C4AulExec::Call(C4AulFunc *pFunc, C4Value *pReturn, C4Value *pPars, C4
 		pContext = pCurCtx->Obj;
 	}
 
-	// Convert parameters (typecheck)
-	const C4V_Type *pTypes = pFunc->GetParType();
-	for (int i = 0; i < pFunc->GetParCount(); i++)
-		if (!pPars[i].CheckParConversion(pTypes[i]))
-			throw new C4AulExecError(FormatString("call to \"%s\" parameter %d: passed %s, but expected %s",
-			                                      pFunc->GetName(), i + 1, pPars[i].GetTypeName(), GetC4VName(pTypes[i])
-			                                     ).getData());
+	pFunc->CheckParTypes(pPars);
 
 	// Script function?
 	C4AulScriptFunc *pSFunc = pFunc->SFunc();
