@@ -43,7 +43,7 @@ StdStrBuf FnStringFormat(C4PropList * _this, C4String *szFormatPar, C4Value * Pa
 		if (*cpFormat=='%')
 		{
 			// Scan field type
-			for (cpType=cpFormat+1; *cpType && (*cpType=='.' || Inside(*cpType,'0','9')); cpType++) {}
+			for (cpType=cpFormat+1; *cpType && (*cpType == '+' || *cpType == '-' || *cpType == '.' || *cpType == '#' || *cpType == ' ' || Inside(*cpType,'0','9')); cpType++) {}
 			// Copy field
 			SCopy(cpFormat,szField,Min<unsigned int>(sizeof szField - 1, cpType - cpFormat + 1));
 			// Insert field by type
@@ -67,13 +67,6 @@ StdStrBuf FnStringFormat(C4PropList * _this, C4String *szFormatPar, C4Value * Pa
 			}
 			// C4ID
 			case 'i':
-			{
-				if (cPar >= ParCount) throw new C4AulExecError("format placeholder without parameter");
-				C4ID id = Pars[cPar++].getC4ID();
-				StringBuf.Append(id.ToString());
-				cpFormat+=SLen(szField);
-				break;
-			}
 			// C4Value
 			case 'v':
 			{
@@ -161,6 +154,132 @@ C4Value C4AulDefFunc::Exec(C4PropList * p, C4Value pPars[], bool fPassErrors)
 
 //=============================== C4Script Functions ====================================
 
+#define MAKE_AND_RETURN_ARRAY(values) do { \
+	C4ValueArray *matrix = new C4ValueArray(sizeof(values) / sizeof(*values)); \
+	for (size_t i = 0; i < sizeof(values) / sizeof(*values); ++i) \
+		(*matrix)[i] = C4VInt(values[i]); \
+	return matrix; \
+} while (0)
+
+static C4ValueArray *FnTrans_Identity(C4PropList * _this)
+{
+	long values[] = 
+	{
+		1000, 0, 0, 0,
+		0, 1000, 0, 0,
+		0, 0, 1000, 0
+	};
+	MAKE_AND_RETURN_ARRAY(values);
+}
+
+static C4ValueArray *FnTrans_Translate(C4PropList * _this, long dx, long dy, long dz)
+{
+	long values[] = 
+	{
+		1000, 0, 0, dx,
+		0, 1000, 0, dy,
+		0, 0, 1000, dz
+	};
+	MAKE_AND_RETURN_ARRAY(values);
+}
+
+static C4ValueArray *FnTrans_Scale(C4PropList * _this, long sx, long sy, long sz)
+{
+	if (sy == 0 && sz == 0)
+		sy = sz = sx;
+	long values[] = 
+	{
+		sx, 0, 0, 0,
+		0, sy, 0, 0,
+		0, 0, sz, 0
+	};
+	MAKE_AND_RETURN_ARRAY(values);
+}
+
+static C4ValueArray *FnTrans_Rotate(C4PropList * _this, long angle, long rx, long ry, long rz)
+{
+	long c = fixtoi(Cos(itofix(angle, 1)), 1000);
+	long s = fixtoi(Sin(itofix(angle, 1)), 1000);
+
+	long sqrt_val = rx * rx + ry * ry + rz * rz;
+	long n = long(sqrt(double(sqrt_val)));
+	if (n * n < sqrt_val) n++;
+	else if (n * n > sqrt_val) n--;
+
+	rx = (1000 * rx) / n;
+	ry = (1000 * ry) / n;
+	rz = (1000 * rz) / n;
+
+	long values[] = 
+	{
+		rx*rx*(1000-c)/1000000+c, rx*ry*(1000-c)/1000000-rz*s/1000, rx*rz*(1000-c)/1000000+ry*s/1000, 0,
+		ry*rx*(1000-c)/1000000+rz*s/1000, ry*ry*(1000-c)/1000000+c, ry*rz*(1000-c)/1000000-rx*s/1000, 0,
+		rz*rx*(1000-c)/1000000-ry*s/1000, ry*rz*(1000-c)/1000000+rx*s/1000, rz*rz*(1000-c)/1000000+c, 0
+	};
+	MAKE_AND_RETURN_ARRAY(values);
+}
+
+static C4Value FnTrans_Mul(C4PropList * _this, C4Value *pars)
+{
+	const int32_t matrixSize = 12;
+	long values[] = 
+	{
+		0, 0, 0, 0,
+		0, 0, 0, 0,
+		0, 0, 0, 0
+	};
+
+	// Read all parameters
+	bool first = true;
+	for (int32_t i = 0; i < C4AUL_MAX_Par; i++)
+	{
+		C4Value Data = *(pars++);
+		// No data given?
+		if (!Data) break;
+		C4ValueArray *factorArray = Data.getArray();
+		if (!factorArray || factorArray->GetSize() != matrixSize) continue;
+
+		if (first)
+		{
+			first = false;
+			
+			for (int32_t c = 0; c < matrixSize; ++c)
+				values[c] = (*factorArray)[c].getInt();
+			continue;
+		}
+
+		// multiply current matrix with new one
+		long values_rhs[matrixSize], values_result[matrixSize];
+		for (int32_t c = 0; c < matrixSize; ++c)
+			values_rhs[c] = (*factorArray)[c].getInt();
+
+		// matrix multiplication
+		values_result[ 0] = values[0]*values_rhs[0]/1000 + values[1]*values_rhs[4]/1000 + values[ 2]*values_rhs[ 8]/1000;
+		values_result[ 1] = values[0]*values_rhs[1]/1000 + values[1]*values_rhs[5]/1000 + values[ 2]*values_rhs[ 9]/1000;
+		values_result[ 2] = values[0]*values_rhs[2]/1000 + values[1]*values_rhs[6]/1000 + values[ 2]*values_rhs[10]/1000;
+		values_result[ 3] = values[0]*values_rhs[3]/1000 + values[1]*values_rhs[7]/1000 + values[ 2]*values_rhs[11]/1000 + values[3];
+		values_result[ 4] = values[4]*values_rhs[0]/1000 + values[5]*values_rhs[4]/1000 + values[ 6]*values_rhs[ 8]/1000;
+		values_result[ 5] = values[4]*values_rhs[1]/1000 + values[5]*values_rhs[5]/1000 + values[ 6]*values_rhs[ 9]/1000;
+		values_result[ 6] = values[4]*values_rhs[2]/1000 + values[5]*values_rhs[6]/1000 + values[ 6]*values_rhs[10]/1000;
+		values_result[ 7] = values[4]*values_rhs[3]/1000 + values[5]*values_rhs[7]/1000 + values[ 6]*values_rhs[11]/1000 + values[7];
+		values_result[ 8] = values[8]*values_rhs[0]/1000 + values[9]*values_rhs[4]/1000 + values[10]*values_rhs[ 8]/1000;
+		values_result[ 9] = values[8]*values_rhs[1]/1000 + values[9]*values_rhs[5]/1000 + values[10]*values_rhs[ 9]/1000;
+		values_result[10] = values[8]*values_rhs[2]/1000 + values[9]*values_rhs[6]/1000 + values[10]*values_rhs[10]/1000;
+		values_result[11] = values[8]*values_rhs[3]/1000 + values[9]*values_rhs[7]/1000 + values[10]*values_rhs[11]/1000 + values[11];
+
+		for (int32_t c = 0; c < matrixSize; ++c)
+			values[c] = values_result[c];
+	}
+
+	// unlike in the other Trans_*-functions, we have to put the array into a C4Value manually here
+	C4ValueArray *matrix = new C4ValueArray(sizeof(values) / sizeof(*values));
+	for (size_t i = 0; i < sizeof(values) / sizeof(*values); ++i)
+		(*matrix)[i] = C4VInt(values[i]);
+	return C4VArray(matrix);
+}
+
+#undef MAKE_AND_RETURN_ARRAY
+
 static C4PropList * FnCreatePropList(C4PropList * _this, C4PropList * prototype)
 {
 	return C4PropList::New(prototype);
@@ -211,7 +330,7 @@ static C4ValueArray * FnGetProperties(C4PropList * _this, C4PropList * p)
 static C4Value FnCall(C4PropList * _this, C4Value * Pars)
 {
 	if (!_this) _this = ::ScriptEngine.GetPropList();
-	C4AulParSet ParSet(&Pars[1], 9);
+	C4AulParSet ParSet(&Pars[1], C4AUL_MAX_Par - 1);
 	C4AulFunc * fn = Pars[0].getFunction();
 	C4String * name;
 	if (!fn)
@@ -231,6 +350,7 @@ static C4Value FnCall(C4PropList * _this, C4Value * Pars)
 	}
 	if (!fn)
 		throw new C4AulExecError(FormatString("Call: no function %s", Pars[0].GetDataString().getData()).getData());
+	fn->CheckParTypes(ParSet.Par);
 	return fn->Exec(_this, &ParSet, true);
 }
 
@@ -251,11 +371,6 @@ static C4Value FnFormat(C4PropList * _this, C4Value * Pars)
 	return C4VString(FnStringFormat(_this, Pars[0].getStr(), &Pars[1], 9));
 }
 
-static C4ID FnC4Id(C4PropList * _this, C4String *szID)
-{
-	return(C4ID(FnStringPar(szID)));
-}
-
 static long FnAbs(C4PropList * _this, long iVal)
 {
 	return Abs(iVal);
@@ -362,7 +477,7 @@ static long FnDistance(C4PropList * _this, long iX1, long iY1, long iX2, long iY
 
 static long FnBoundBy(C4PropList * _this, long iVal, long iRange1, long iRange2)
 {
-	return BoundBy(iVal,iRange1,iRange2);
+	return Clamp(iVal,iRange1,iRange2);
 }
 
 static bool FnInside(C4PropList * _this, long iVal, long iRange1, long iRange2)
@@ -414,13 +529,19 @@ static int FnGetIndexOf(C4PropList * _this, C4ValueArray * pArray, const C4Value
 	if (!pArray) return -1;
 	int32_t iSize = pArray->GetSize();
 	for (int32_t i = 0; i < iSize; ++i)
-		if (Needle == pArray->GetItem(i))
+		if (Needle.IsIdenticalTo(pArray->GetItem(i)))
 			// element found
 			return i;
 	// element not found
 	return -1;
 }
 
+static bool FnDeepEqual(C4PropList * _this, const C4Value & v1, const C4Value & v2)
+{
+	// return if v1==v2 with deep comparison on arrays and proplists
+	return v1 == v2;
+}
+
 static C4Void FnSetLength(C4PropList * _this, C4ValueArray *pArray, int iNewSize)
 {
 	// safety
@@ -532,16 +653,12 @@ static bool FnStartCallTrace(C4PropList * _this)
 	return true;
 }
 
-static bool FnStartScriptProfiler(C4PropList * _this, C4ID idScript)
+static bool FnStartScriptProfiler(C4PropList * _this, C4Def * pDef)
 {
 	// get script to profile
 	C4AulScript *pScript;
-	if (idScript)
-	{
-		C4Def *pDef = C4Id2Def(idScript);
-		if (!pDef) return false;
+	if (pDef)
 		pScript = &pDef->Script;
-	}
 	else
 		pScript = &::ScriptEngine;
 	// profile it
@@ -629,7 +746,6 @@ C4ScriptConstDef C4ScriptConstMap[]=
 
 	{ "C4X_Ver1",        C4V_Int, C4XVER1},
 	{ "C4X_Ver2",        C4V_Int, C4XVER2},
-	{ "C4X_Ver3",        C4V_Int, C4XVER3},
 
 	{ NULL, C4V_Nil, 0}
 };
@@ -640,6 +756,7 @@ C4ScriptFnDef C4ScriptFnMap[]=
 	{ "Log",           1, C4V_Bool,   { C4V_String  ,C4V_Any     ,C4V_Any     ,C4V_Any     ,C4V_Any     ,C4V_Any     ,C4V_Any    ,C4V_Any    ,C4V_Any    ,C4V_Any}, FnLog      },
 	{ "DebugLog",      1, C4V_Bool,   { C4V_String  ,C4V_Any     ,C4V_Any     ,C4V_Any     ,C4V_Any     ,C4V_Any     ,C4V_Any    ,C4V_Any    ,C4V_Any    ,C4V_Any}, FnDebugLog },
 	{ "Format",        1, C4V_String, { C4V_String  ,C4V_Any     ,C4V_Any     ,C4V_Any     ,C4V_Any     ,C4V_Any     ,C4V_Any    ,C4V_Any    ,C4V_Any    ,C4V_Any}, FnFormat   },
+	{ "Trans_Mul",     1, C4V_Array,  { C4V_Array   ,C4V_Any     ,C4V_Any     ,C4V_Any     ,C4V_Any     ,C4V_Any     ,C4V_Any    ,C4V_Any    ,C4V_Any    ,C4V_Any}, FnTrans_Mul},
 
 	{ NULL,            0, C4V_Nil,    { C4V_Nil     ,C4V_Nil     ,C4V_Nil     ,C4V_Nil     ,C4V_Nil     ,C4V_Nil     ,C4V_Nil    ,C4V_Nil    ,C4V_Nil    ,C4V_Nil}, 0          }
 };
@@ -676,7 +793,6 @@ void InitCoreFunctionMap(C4AulScriptEngine *pEngine)
 	F(GetProperty);
 	F(SetProperty);
 	F(ResetProperty);
-	F(C4Id);
 	F(Distance);
 	F(Angle);
 	F(GetChar);
@@ -686,6 +802,7 @@ void InitCoreFunctionMap(C4AulScriptEngine *pEngine)
 	F(GetLength);
 	F(SetLength);
 	F(GetIndexOf);
+	F(DeepEqual);
 	F(FatalError);
 	F(StartCallTrace);
 	F(StartScriptProfiler);
@@ -693,6 +810,10 @@ void InitCoreFunctionMap(C4AulScriptEngine *pEngine)
 	F(SortArray);
 	F(SortArrayByProperty);
 	F(SortArrayByArrayElement);
+	F(Trans_Identity);
+	F(Trans_Translate);
+	F(Trans_Scale);
+	F(Trans_Rotate);
 	F(LocateFunc);
 	F(FileWrite);
 
